@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 1992, 1993, 1994, 1997 Free Software Foundation, Inc.
 ;; Copyright (C) 1995 Tinker Systems.
-;; Copyright (C) 1995, 1996, 2000 Ben Wing.
+;; Copyright (C) 1995, 1996, 2000, 2002 Ben Wing.
 
 ;; Author: Richard Mlynarik
 ;; Created: 2-Oct-92
@@ -271,7 +271,8 @@ in `substitute-in-file-name'."
 If `zmacs-regions' is true, and the zmacs region is active in this buffer,
 then this key deactivates the region without beeping."
   (interactive)
-  (if (region-active-p)
+  (if (and (region-active-p)
+	   (eq (current-buffer) (zmacs-region-buffer)))
       ;; pseudo-zmacs compatibility: don't beep if this ^G is simply
       ;; deactivating the region.  If it is inactive, beep.
       nil
@@ -339,11 +340,7 @@ minibuffer history if its length is less than that value."
 		 integer)
   :group 'minibuffer)
 
-(define-error 'input-error "Keyboard input error")
-
-(put 'input-error 'display-error
-     #'(lambda (error-object stream)
-	 (princ (cadr error-object) stream)))
+(define-error 'input-error "Keyboard input error" 'io-error)
 
 (defun read-from-minibuffer (prompt &optional initial-contents
                                     keymap
@@ -1104,9 +1101,11 @@ This is not enabled by default because
 		    (if (and filename-kludge-p
 			     ;; #### evil evil evil evil
 			     (or (and (fboundp 'ange-ftp-ftp-path)
-				      (ange-ftp-ftp-path string))
+				      (declare-fboundp
+				       (ange-ftp-ftp-path string)))
 				 (and (fboundp 'efs-ftp-path)
-				      (efs-ftp-path string))))
+				      (declare-fboundp
+				       (efs-ftp-path string)))))
 			(setq comp t)
 		      (setq comp
 			    (try-completion string
@@ -1131,11 +1130,12 @@ This is not enabled by default because
     (set-buffer mouse-grabbed-buffer) ; the minibuf
     (let ((kludge-string (concat (buffer-string) string)))
       (if (or (and (fboundp 'ange-ftp-ftp-path)
-		   (ange-ftp-ftp-path kludge-string))
-	       (and (fboundp 'efs-ftp-path) (efs-ftp-path kludge-string)))
-	   ;; #### evil evil evil, but more so.
-	   string
-	 (append-expand-filename (buffer-string) string)))))
+		   (declare-fboundp (ange-ftp-ftp-path kludge-string)))
+	       (and (fboundp 'efs-ftp-path)
+		    (declare-fboundp (efs-ftp-path kludge-string))))
+	  ;; #### evil evil evil, but more so.
+	  string
+	(append-expand-filename (buffer-string) string)))))
 
 (defun minibuffer-smart-select-highlighted-completion (event)
   "Select the highlighted text under the mouse as a minibuffer response.
@@ -1792,10 +1792,12 @@ DIR defaults to current buffer's directory default."
                     ((eq action 't)
                      ;; all completions
                      (mapcar #'(lambda (p) (concat "~" p))
-                             (user-name-all-completions user)))
+                             (declare-fboundp
+			      (user-name-all-completions user))))
                     (t;; 'nil
                      ;; complete
-                     (let* ((val+uniq (user-name-completion-1 user))
+                     (let* ((val+uniq (declare-fboundp
+				       (user-name-completion-1 user)))
                             (val  (car val+uniq))
                             (uniq (cdr val+uniq)))
                        (cond ((stringp val)
@@ -2196,9 +2198,7 @@ whether it is a file(/result) or a directory (/result/)."
 				"/usr/local/lib/X11R5/X11/"
 				"/usr/X11/lib/X11/"
 				"/usr/lib/X11/"
-				"/usr/share/X11/"
 				"/usr/local/lib/X11/"
-				"/usr/local/share/X11/"
 				"/usr/X386/lib/X11/"
 				"/usr/x386/lib/X11/"
 				"/usr/XFree86/lib/X11/"
@@ -2246,9 +2246,9 @@ whether it is a file(/result) or a directory (/result/)."
 	 (setq x-read-color-completion-table clist)
 	 x-read-color-completion-table)))
     (mswindows
-     (mapcar #'list (mswindows-color-list)))
+     (mapcar #'list (declare-fboundp (mswindows-color-list))))
     (tty
-     (mapcar #'list (tty-color-list)))))
+     (mapcar #'list (declare-fboundp (tty-color-list))))))
 
 (defun read-color (prompt &optional must-match initial-contents)
   "Read the name of a color from the minibuffer.
@@ -2261,12 +2261,6 @@ On mswindows devices, this uses `mswindows-color-list'."
 		     initial-contents)))
 
 
-;; #### The doc string for read-non-nil-coding system gets lost if we
-;; only include these if the mule feature is present.  Strangely,
-;; read-coding-system doesn't.
-
-;;(if (featurep 'mule)
-
 (defun read-coding-system (prompt &optional default-coding-system)
   "Read a coding-system (or nil) from the minibuffer.
 Prompting with string PROMPT.
@@ -2290,8 +2284,6 @@ Prompt with string PROMPT."
 					    t))))
     retval))
 
-;;) ;; end of (featurep 'mule)
-
 
 
 (defcustom force-dialog-box-use nil
