@@ -104,7 +104,6 @@ Set this to `nil' to use the `default' face."
     (define-key m "q" 'pui-quit)
     (define-key m "g" 'pui-list-packages)
     (define-key m "i" 'pui-display-info)
-    (define-key m "m" 'pui-display-maintainer)
     (define-key m "?" 'describe-mode)
     (define-key m "v" 'pui-toggle-verbosity-redisplay)
     (define-key m "d" 'pui-toggle-package-delete-key)
@@ -463,14 +462,13 @@ and whether or not it is up-to-date."
   "Display additional package info in the modeline.
 EXTENT determines the package to display (the package information is
 attached to the extent as properties)."
-  (let (pkg-sym info inst-ver inst-auth-ver auth-ver date maintainer balloon req)
+  (let (pkg-sym info inst-ver auth-ver date maintainer balloon req)
     (if (or force-update (not (current-message))
 	    (string-match ".*: .*: " (current-message)))
 	(progn
 	  (setq pkg-sym (extent-property extent 'pui-package)
 		info (extent-property extent 'pui-info)
 		inst-ver (package-get-key pkg-sym :version)
-		inst-auth-ver (package-get-key pkg-sym :author-version)
 		auth-ver (package-get-info-prop info 'author-version)
 		date (package-get-info-prop info 'date)
 		maintainer (package-get-info-prop info 'maintainer)
@@ -480,17 +478,22 @@ attached to the extent as properties)."
 	  (if (featurep 'balloon-help)
 	      (progn
 		(setq balloon (format "
-Package Information:  [For package: \"%s\"]\n================
-Installed Upstream Ver: %s  Available Upstream Ver: %s
+Package Information:  [For package: \"%s\"]
+================
+Installed Version : %.2f
+Author Version : %s
 Maintainer : %s
 Released : %s
 Required Packages : %s\n\n"
-				      pkg-sym inst-auth-ver auth-ver maintainer 
+				      pkg-sym inst-ver auth-ver maintainer 
 				      date req))
 		(set-extent-property extent 'balloon-help balloon)))
-	  (format 
-	   "Installed upstream ver: %s  Available upstream ver: %s" 
-	   inst-auth-ver auth-ver)))))
+	  (if pui-list-verbose
+	      (format 
+	       "Inst V: %.2f Auth V: %s Maint: %s" 
+	       inst-ver auth-ver maintainer)
+	    (format "%.2f : %s : %s"
+		    inst-ver auth-ver maintainer))))))
 
 (defun pui-display-info (&optional no-error event)
   "Display additional package info in the modeline.
@@ -506,24 +509,6 @@ Designed to be called interactively (from a keypress)."
 	  (error 'invalid-operation
 		 "No package under cursor!"))))))
 
-(defun pui-display-maintainer (&optional no-error event)
-  "Display a package's maintainer in the minibuffer."
-  (interactive)
-  (let (extent ;pkg-sym
-	info maintainer)
-    (save-excursion
-      (beginning-of-line)
-      (if (setq extent 	(extent-at (point) (current-buffer) 'pui))
-	  (progn
-	    (setq ;pkg-sym (extent-property extent 'pui-package)
-	     info (extent-property extent 'pui-info)
-	     maintainer (package-get-info-prop info 'maintainer))
-	    (message (format "Maintainer: %s" maintainer)))
-	(if no-error
-	    (clear-message nil)
-	  (error 'invalid-operation
-		 "No package under cursor!"))))))
-
 (defvar pui-menu
   '("Packages"
     ["Toggle install " pui-toggle-package-key :active (pui-current-package) :suffix (format "`%s'" (or (pui-current-package) "..."))]
@@ -565,8 +550,7 @@ Useful keys:
   `\\[pui-toggle-package-delete-key]' to select/unselect the current package for removal.
   `\\[pui-add-required-packages]' to add any packages required by those selected.
   `\\[pui-install-selected-packages]' to install/delete selected packages.
-  `\\[pui-display-info]' to display additional information about the package in the minibuffer.
-  `\\[pui-display-maintainer]' to display the package's maintainer in the minibuffer
+  `\\[pui-display-info]' to display additional information about the package in the modeline.
   `\\[pui-list-packages]' to refresh the package list.
   `\\[pui-toggle-verbosity-redisplay]' to toggle between a verbose and non-verbose display.
   `\\[pui-quit]' to kill this buffer.
@@ -607,11 +591,11 @@ Warning: No download sites specified.  Package index may be out of date.
 "))
     
     (if pui-list-verbose
-	(insert "                       Latest Installed
-  Package name         Vers.  Vers.   Description
+	(insert "                 Latest Installed
+  Package name   Vers.  Vers.   Description
 ")
-      (insert "                       Latest
-  Package name         Vers.  Description
+      (insert "                 Latest
+  Package name   Vers.  Description
 "))
     (insert sep-string)
     (setq start (point))
@@ -640,13 +624,13 @@ Warning: No download sites specified.  Package index may be out of date.
 		  ((numberp current-vers)
 		   (setq current-vers (format "%.2f" current-vers))))
 		 (insert
-		  (format "%s %-20s %-5.2f  %-5s  %s\n"
+		  (format "%s %-15s %-5.2f  %-5s  %s\n"
 			  (car disp) pkg-sym 
 			  (if (stringp version)
 			      (string-to-number version)
 			    version)
 			  current-vers desc)))
-	     (insert (format "%s %-20s %-5s %s\n"
+	     (insert (format "%s %-15s %-5s %s\n"
 			     (car disp)
 			     pkg-sym version desc)))
 	   (save-excursion
