@@ -1,6 +1,7 @@
 /* Execution of byte code produced by bytecomp.el.
    Implementation of compiled-function objects.
    Copyright (C) 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1995, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -55,6 +56,7 @@ by Hallvard:
 #include "bytecode.h"
 #include "opaque.h"
 #include "syntax.h"
+#include "window.h"
 
 EXFUN (Ffetch_bytecode, 1);
 
@@ -208,22 +210,12 @@ enum Opcode /* Byte codes */
   Bconstant 		= 0300
 };
 typedef enum Opcode Opcode;
-typedef unsigned char Opbyte;
 
 
-static void check_opcode (Opcode opcode);
-static void invalid_byte_code_error (char *error_message, ...);
-
 Lisp_Object * execute_rare_opcode (Lisp_Object *stack_ptr,
 				   const Opbyte *program_ptr,
 				   Opcode opcode);
 
-static Lisp_Object execute_optimized_program (const Opbyte *program,
-					      int stack_depth,
-					      Lisp_Object *constants_data);
-
-extern Lisp_Object Qand_rest, Qand_optional;
-
 /* Define BYTE_CODE_METER to enable generation of a byte-op usage histogram.
    This isn't defined in FSF Emacs and isn't defined in XEmacs v19. */
 /* #define BYTE_CODE_METER */
@@ -255,9 +247,7 @@ bytecode_negate (Lisp_Object obj)
  retry:
 
   if (INTP    (obj)) return make_int (- XINT (obj));
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP  (obj)) return make_float (- XFLOAT_DATA (obj));
-#endif
   if (CHARP   (obj)) return make_int (- ((int) XCHAR (obj)));
   if (MARKERP (obj)) return make_int (- ((int) marker_position (obj)));
 
@@ -291,7 +281,6 @@ bytecode_arithcompare (Lisp_Object obj1, Lisp_Object obj2)
 {
   retry:
 
-#ifdef LISP_FLOAT_TYPE
   {
     EMACS_INT ival1, ival2;
 
@@ -335,37 +324,11 @@ bytecode_arithcompare (Lisp_Object obj1, Lisp_Object obj2)
 
     return dval1 < dval2 ? -1 : dval1 > dval2 ? 1 : 0;
   }
-#else /* !LISP_FLOAT_TYPE */
-  {
-    EMACS_INT ival1, ival2;
-
-    if      (INTP    (obj1)) ival1 = XINT  (obj1);
-    else if (CHARP   (obj1)) ival1 = XCHAR (obj1);
-    else if (MARKERP (obj1)) ival1 = marker_position (obj1);
-    else
-      {
-	obj1 = wrong_type_argument (Qnumber_char_or_marker_p, obj1);
-	goto retry;
-      }
-
-    if      (INTP    (obj2)) ival2 = XINT  (obj2);
-    else if (CHARP   (obj2)) ival2 = XCHAR (obj2);
-    else if (MARKERP (obj2)) ival2 = marker_position (obj2);
-    else
-      {
-	obj2 = wrong_type_argument (Qnumber_char_or_marker_p, obj2);
-	goto retry;
-      }
-
-    return ival1 < ival2 ? -1 : ival1 > ival2 ? 1 : 0;
-  }
-#endif /* !LISP_FLOAT_TYPE */
 }
 
 static Lisp_Object
 bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 {
-#ifdef LISP_FLOAT_TYPE
   EMACS_INT ival1, ival2;
   int float_p;
 
@@ -427,111 +390,6 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	}
       return make_float (dval1);
     }
-#else /* !LISP_FLOAT_TYPE */
-  EMACS_INT ival1, ival2;
-
- retry:
-
-  if      (INTP    (obj1)) ival1 = XINT  (obj1);
-  else if (CHARP   (obj1)) ival1 = XCHAR (obj1);
-  else if (MARKERP (obj1)) ival1 = marker_position (obj1);
-  else
-    {
-      obj1 = wrong_type_argument (Qnumber_char_or_marker_p, obj1);
-      goto retry;
-    }
-
-  if      (INTP    (obj2)) ival2 = XINT  (obj2);
-  else if (CHARP   (obj2)) ival2 = XCHAR (obj2);
-  else if (MARKERP (obj2)) ival2 = marker_position (obj2);
-  else
-    {
-      obj2 = wrong_type_argument (Qnumber_char_or_marker_p, obj2);
-      goto retry;
-    }
-
-  switch (opcode)
-    {
-    case Bplus: ival1 += ival2; break;
-    case Bdiff: ival1 -= ival2; break;
-    case Bmult: ival1 *= ival2; break;
-    case Bquo:
-      if (ival2 == 0) Fsignal (Qarith_error, Qnil);
-      ival1 /= ival2;
-      break;
-    case Bmax:  if (ival1 < ival2) ival1 = ival2; break;
-    case Bmin:  if (ival1 > ival2) ival1 = ival2; break;
-    }
-  return make_int (ival1);
-#endif /* !LISP_FLOAT_TYPE */
-}
-
-/* Apply compiled-function object FUN to the NARGS evaluated arguments
-   in ARGS, and return the result of evaluation. */
-Lisp_Object
-funcall_compiled_function (Lisp_Object fun, int nargs, Lisp_Object args[])
-{
-  /* This function can GC */
-  int speccount = specpdl_depth();
-  REGISTER int i = 0;
-  Lisp_Compiled_Function *f = XCOMPILED_FUNCTION (fun);
-  int optional = 0;
-
-  if (!OPAQUEP (f->instructions))
-    /* Lazily munge the instructions into a more efficient form */
-    optimize_compiled_function (fun);
-
-  /* optimize_compiled_function() guaranteed that f->specpdl_depth is
-     the required space on the specbinding stack for binding the args
-     and local variables of fun.   So just reserve it once. */
-  SPECPDL_RESERVE (f->specpdl_depth);
-
-  {
-    /* Fmake_byte_code() guaranteed that f->arglist is a valid list
-       containing only non-constant symbols. */
-    LIST_LOOP_3 (symbol, f->arglist, tail)
-      {
-	if (EQ (symbol, Qand_rest))
-	  {
-	    tail = XCDR (tail);
-	    symbol  = XCAR (tail);
-	    SPECBIND_FAST_UNSAFE (symbol, Flist (nargs - i, &args[i]));
-	    goto run_code;
-	  }
-	else if (EQ (symbol, Qand_optional))
-	  optional = 1;
-	else if (i == nargs && !optional)
-	  goto wrong_number_of_arguments;
-	else
-	  SPECBIND_FAST_UNSAFE (symbol, i < nargs ? args[i++] : Qnil);
-      }
-  }
-
-  if (i < nargs)
-    goto wrong_number_of_arguments;
-
- run_code:
-
-  {
-    Lisp_Object value =
-      execute_optimized_program ((Opbyte *) XOPAQUE_DATA (f->instructions),
-				 f->stack_depth,
-				 XVECTOR_DATA (f->constants));
-
-    /* The attempt to optimize this by only unbinding variables failed
-       because using buffer-local variables as function parameters
-       leads to specpdl_ptr->func != 0 */
-    /* UNBIND_TO_GCPRO_VARIABLES_ONLY (speccount, value); */
-    UNBIND_TO_GCPRO (speccount, value);
-    return value;
-  }
-
- wrong_number_of_arguments:
-  /* The actual printed compiled_function object is incomprehensible.
-     Check the backtrace to see if we can get a more meaningful symbol. */
-  if (EQ (fun, indirect_function (*backtrace_list->function, 0)))
-    fun = *backtrace_list->function;
-  return Fsignal (Qwrong_number_of_arguments, list2 (fun, make_int (nargs)));
 }
 
 
@@ -589,21 +447,23 @@ funcall_compiled_function (Lisp_Object fun, int nargs, Lisp_Object args[])
    but don't pop it. */
 #define TOP (*stack_ptr)
 
+#define GCPRO_STACK  (gcpro1.nvars = stack_ptr - stack_beg)
+
 /* The actual interpreter for byte code.
    This function has been seriously optimized for performance.
    Don't change the constructs unless you are willing to do
    real benchmarking and profiling work -- martin */
 
 
-static Lisp_Object
+Lisp_Object
 execute_optimized_program (const Opbyte *program,
 			   int stack_depth,
 			   Lisp_Object *constants_data)
 {
   /* This function can GC */
   REGISTER const Opbyte *program_ptr = (Opbyte *) program;
-  REGISTER Lisp_Object *stack_ptr
-    = alloca_array (Lisp_Object, stack_depth + 1);
+  Lisp_Object *stack_beg = alloca_array (Lisp_Object, stack_depth + 1);
+  REGISTER Lisp_Object *stack_ptr = stack_beg;
   int speccount = specpdl_depth ();
   struct gcpro gcpro1;
 
@@ -613,32 +473,19 @@ execute_optimized_program (const Opbyte *program,
 #endif
 
 #ifdef ERROR_CHECK_BYTE_CODE
-  Lisp_Object *stack_beg = stack_ptr;
   Lisp_Object *stack_end = stack_beg + stack_depth;
 #endif
 
-  /* Initialize all the objects on the stack to Qnil,
-     so we can GCPRO the whole stack.
-     The first element of the stack is actually a dummy. */
-  {
-    int i;
-    Lisp_Object *p;
-    for (i = stack_depth, p = stack_ptr; i--;)
-      *++p = Qnil;
-  }
-
   GCPRO1 (stack_ptr[1]);
-  gcpro1.nvars = stack_depth;
-
+  
   while (1)
     {
       REGISTER Opcode opcode = (Opcode) READ_UINT_1;
 #ifdef ERROR_CHECK_BYTE_CODE
       if (stack_ptr > stack_end)
-	invalid_byte_code_error ("byte code stack overflow");
+	stack_overflow ("byte code stack overflow", Qunbound);
       if (stack_ptr < stack_beg)
-	invalid_byte_code_error ("byte code stack underflow");
-      check_opcode (opcode);
+	stack_overflow ("byte code stack underflow", Qunbound);
 #endif
 
 #ifdef BYTE_CODE_METER
@@ -655,7 +502,10 @@ execute_optimized_program (const Opbyte *program,
 	  if (opcode >= Bconstant)
 	    PUSH (constants_data[opcode - Bconstant]);
 	  else
-	    stack_ptr = execute_rare_opcode (stack_ptr, program_ptr, opcode);
+	    {
+	      GCPRO_STACK;
+	      stack_ptr = execute_rare_opcode (stack_ptr, program_ptr, opcode);
+	    }
 	  break;
 
 	case Bvarref:
@@ -720,6 +570,10 @@ execute_optimized_program (const Opbyte *program,
 	      specpdl_depth_counter++;
 
 	      symbol_ptr->value = new_value;
+
+#ifdef ERROR_CHECK_CATCH
+	      check_specbind_stack_sanity ();
+#endif
 	    }
 	  else
 	    specbind_magic (symbol, new_value);
@@ -736,6 +590,7 @@ execute_optimized_program (const Opbyte *program,
 	case Bcall+7:
 	  n = (opcode <  Bcall+6 ? opcode - Bcall :
 	       opcode == Bcall+6 ? READ_UINT_1 : READ_UINT_2);
+	  GCPRO_STACK;
 	  DISCARD (n);
 #ifdef BYTE_CODE_METER
 	  if (byte_metering_on && SYMBOLP (TOP))
@@ -844,7 +699,7 @@ execute_optimized_program (const Opbyte *program,
 #ifdef ERROR_CHECK_BYTE_CODE
 	  /* Binds and unbinds are supposed to be compiled balanced.  */
 	  if (specpdl_depth() != speccount)
-	    invalid_byte_code_error ("unbalanced specbinding stack");
+	    invalid_byte_code ("unbalanced specbinding stack", Qunbound);
 #endif
 	  return TOP;
 
@@ -875,7 +730,7 @@ execute_optimized_program (const Opbyte *program,
 	case Bunbind_all:
 	  /* To unbind back to the beginning of this frame.  Not used yet,
 	     but will be needed for tail-recursion elimination. */
-	  unbind_to (speccount, Qnil);
+	  unbind_to (speccount);
 	  break;
 
 	case Bnth:
@@ -1151,8 +1006,8 @@ execute_optimized_program (const Opbyte *program,
 
 	case Bcurrent_buffer:
 	  {
-	    Lisp_Object buffer;
-	    XSETBUFFER (buffer, current_buffer);
+	    Lisp_Object buffer = wrap_buffer (current_buffer);
+
 	    PUSH (buffer);
 	    break;
 	  }
@@ -1237,21 +1092,21 @@ execute_rare_opcode (Lisp_Object *stack_ptr,
       {
 	int count = specpdl_depth ();
 	record_unwind_protect (save_window_excursion_unwind,
-			       Fcurrent_window_configuration (Qnil));
+			       call1 (Qcurrent_window_configuration, Qnil));
 	TOP = Fprogn (TOP);
-	unbind_to (count, Qnil);
+	unbind_to (count);
 	break;
       }
 
     case Bsave_restriction:
       record_unwind_protect (save_restriction_restore,
-			     save_restriction_save ());
+			     save_restriction_save (current_buffer));
       break;
 
     case Bcatch:
       {
 	Lisp_Object arg = POP;
-	TOP = internal_catch (TOP, Feval, arg, 0);
+	TOP = internal_catch (TOP, Feval, arg, 0, 0);
 	break;
       }
 
@@ -1436,7 +1291,7 @@ execute_rare_opcode (Lisp_Object *stack_ptr,
 	TOP = arg;
 	/* GAG ME!! */
 	/* pop binding of standard-output */
-	unbind_to (specpdl_depth() - 1, Qnil);
+	unbind_to (specpdl_depth() - 1);
 	break;
       }
 
@@ -1476,27 +1331,17 @@ execute_rare_opcode (Lisp_Object *stack_ptr,
       }
 
     default:
-      ABORT();
+      abort();
       break;
     }
   return stack_ptr;
 }
 
 
-static void
-invalid_byte_code_error (char *error_message, ...)
+DOESNT_RETURN
+invalid_byte_code (const CIbyte *reason, Lisp_Object frob)
 {
-  Lisp_Object obj;
-  va_list args;
-  char *buf = alloca_array (char, strlen (error_message) + 128);
-
-  sprintf (buf, "%s", error_message);
-  va_start (args, error_message);
-  obj = emacs_doprnt_string_va ((const Bufbyte *) GETTEXT (buf), Qnil, -1,
-				args);
-  va_end (args);
-
-  signal_error (Qinvalid_byte_code, list1 (obj));
+  signal_error (Qinvalid_byte_code, reason, frob);
 }
 
 /* Check for valid opcodes.  Change this when adding new opcodes.  */
@@ -1506,8 +1351,8 @@ check_opcode (Opcode opcode)
   if ((opcode < Bvarref) ||
       (opcode == 0251)   ||
       (opcode > Bassq && opcode < Bconstant))
-    invalid_byte_code_error
-      ("invalid opcode %d in instruction stream", opcode);
+    invalid_byte_code ("invalid opcode in instruction stream",
+		       make_int (opcode));
 }
 
 /* Check that IDX is a valid offset into the `constants' vector */
@@ -1515,19 +1360,20 @@ static void
 check_constants_index (int idx, Lisp_Object constants)
 {
   if (idx < 0 || idx >= XVECTOR_LENGTH (constants))
-    invalid_byte_code_error
-      ("reference %d to constants array out of range 0, %d",
+    signal_ferror
+      (Qinvalid_byte_code,
+       "reference %d to constants array out of range 0, %ld",
        idx, XVECTOR_LENGTH (constants) - 1);
 }
 
 /* Get next character from Lisp instructions string. */
-#define READ_INSTRUCTION_CHAR(lvalue) do {		\
-  (lvalue) = charptr_emchar (ptr);			\
-  INC_CHARPTR (ptr);					\
-  *icounts_ptr++ = program_ptr - program;		\
-  if (lvalue > UCHAR_MAX)				\
-    invalid_byte_code_error				\
-      ("Invalid character %c in byte code string");	\
+#define READ_INSTRUCTION_CHAR(lvalue) do {				\
+  (lvalue) = itext_ichar (ptr);					\
+  INC_IBYTEPTR (ptr);							\
+  *icounts_ptr++ = program_ptr - program;				\
+  if (lvalue > UCHAR_MAX)						\
+    invalid_byte_code							\
+      ("Invalid character in byte code string", make_char (lvalue));	\
 } while (0)
 
 /* Get opcode from Lisp instructions string. */
@@ -1612,8 +1458,8 @@ optimize_byte_code (/* in */
 		    int * const program_length,
 		    int * const varbind_count)
 {
-  size_t instructions_length = XSTRING_LENGTH (instructions);
-  size_t comfy_size = 2 * instructions_length;
+  Bytecount instructions_length = XSTRING_LENGTH (instructions);
+  Elemcount comfy_size = (Elemcount) (2 * instructions_length);
 
   int * const icounts = alloca_array (int, comfy_size);
   int * icounts_ptr = icounts;
@@ -1629,8 +1475,8 @@ optimize_byte_code (/* in */
 
   Opbyte *program_ptr = program;
 
-  const Bufbyte *ptr = XSTRING_DATA (instructions);
-  const Bufbyte * const end = ptr + instructions_length;
+  const Ibyte *ptr = XSTRING_DATA (instructions);
+  const Ibyte * const end = ptr + instructions_length;
 
   *varbind_count = 0;
 
@@ -1655,10 +1501,9 @@ optimize_byte_code (/* in */
 	  check_constants_index (arg, constants);
 	   val = XVECTOR_DATA (constants) [arg];
 	   if (!SYMBOLP (val))
-	     invalid_byte_code_error ("variable reference to non-symbol %S", val);
+	     invalid_byte_code ("variable reference to non-symbol", val);
 	   if (EQ (val, Qnil) || EQ (val, Qt) || (SYMBOL_IS_KEYWORD (val)))
-	     invalid_byte_code_error ("variable reference to constant symbol %s",
-				      string_data (XSYMBOL (val)->name));
+	     invalid_byte_code ("variable reference to constant symbol", val);
 	   WRITE_NARGS (Bvarref);
 	   break;
 
@@ -1671,10 +1516,9 @@ optimize_byte_code (/* in */
 	  check_constants_index (arg, constants);
 	  val = XVECTOR_DATA (constants) [arg];
 	  if (!SYMBOLP (val))
-	    invalid_byte_code_error ("attempt to set non-symbol %S", val);
+	    wtaerror ("attempt to set non-symbol", val);
 	  if (EQ (val, Qnil) || EQ (val, Qt))
-	    invalid_byte_code_error ("attempt to set constant symbol %s",
-				     string_data (XSYMBOL (val)->name));
+	    signal_error (Qsetting_constant, 0, val);
 	  /* Ignore assignments to keywords by converting to Bdiscard.
 	     For backward compatibility only - we'd like to make this an error.  */
 	  if (SYMBOL_IS_KEYWORD (val))
@@ -1693,10 +1537,10 @@ optimize_byte_code (/* in */
 	  check_constants_index (arg, constants);
 	  val = XVECTOR_DATA (constants) [arg];
 	  if (!SYMBOLP (val))
-	    invalid_byte_code_error ("attempt to let-bind non-symbol %S", val);
+	    wtaerror ("attempt to let-bind non-symbol", val);
 	  if (EQ (val, Qnil) || EQ (val, Qt) || (SYMBOL_IS_KEYWORD (val)))
-	    invalid_byte_code_error ("attempt to let-bind constant symbol %s",
-				     string_data (XSYMBOL (val)->name));
+	    signal_error (Qsetting_constant,
+			  "attempt to let-bind constant symbol", val);
 	  WRITE_NARGS (Bvarbind);
 	  break;
 
@@ -1742,8 +1586,7 @@ optimize_byte_code (/* in */
 	  jumps_ptr->to   = jumps_ptr->from + arg;
 	  jumps_ptr++;
 	  if (arg >= -1 && arg <= argsize)
-	    invalid_byte_code_error
-	      ("goto instruction is its own target");
+	    invalid_byte_code ("goto instruction is its own target", Qunbound);
 	  if (arg <= SCHAR_MIN ||
 	      arg >  SCHAR_MAX)
 	    {
@@ -1859,7 +1702,7 @@ optimize_byte_code (/* in */
 		break;
 
 	    default:
-	      ABORT();
+	      abort();
 	      break;
 	    }
 	}
@@ -1888,13 +1731,12 @@ optimize_compiled_function (Lisp_Object compiled_function)
 
   if (STRINGP (f->instructions))
     {
-      /* XSTRING_LENGTH() is more efficient than XSTRING_CHAR_LENGTH(),
+      /* XSTRING_LENGTH() is more efficient than string_char_length(),
 	 which would be slightly more `proper' */
       program = alloca_array (Opbyte, 1 + 2 * XSTRING_LENGTH (f->instructions));
       optimize_byte_code (f->instructions, f->constants,
 			  program, &program_length, &varbind_count);
-      f->specpdl_depth = (unsigned short) (XINT (Flength (f->arglist)) +
-                                           varbind_count);
+      f->specpdl_depth = XINT (Flength (f->arglist)) + varbind_count;
       f->instructions =
 	make_opaque (program, program_length * sizeof (Opbyte));
     }
@@ -1915,27 +1757,22 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
   int docp = f->flags.documentationp;
   int intp = f->flags.interactivep;
   struct gcpro gcpro1, gcpro2;
-  char buf[100];
   GCPRO2 (obj, printcharfun);
 
-  write_c_string (print_readably ? "#[" : "#<compiled-function ", printcharfun);
+  write_c_string (printcharfun, print_readably ? "#[" : "#<compiled-function ");
 #ifdef COMPILED_FUNCTION_ANNOTATION_HACK
   if (!print_readably)
     {
       Lisp_Object ann = compiled_function_annotation (f);
       if (!NILP (ann))
-	{
-	  write_c_string ("(from ", printcharfun);
-	  print_internal (ann, printcharfun, 1);
-	  write_c_string (") ", printcharfun);
-	}
+	write_fmt_string_lisp (printcharfun, "(from %S) ", 1, ann);
     }
 #endif /* COMPILED_FUNCTION_ANNOTATION_HACK */
   /* COMPILED_ARGLIST = 0 */
   print_internal (compiled_function_arglist (f), printcharfun, escapeflag);
 
   /* COMPILED_INSTRUCTIONS = 1 */
-  write_c_string (" ", printcharfun);
+  write_c_string (printcharfun, " ");
   {
     struct gcpro ngcpro1;
     Lisp_Object instructions = compiled_function_instructions (f);
@@ -1943,9 +1780,8 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
     if (STRINGP (instructions) && !print_readably)
       {
 	/* We don't usually want to see that junk in the bytecode. */
-	sprintf (buf, "\"...(%ld)\"",
-		 (long) XSTRING_CHAR_LENGTH (instructions));
-	write_c_string (buf, printcharfun);
+	write_fmt_string (printcharfun, "\"...(%ld)\"",
+			  (long) string_char_length (instructions));
       }
     else
       print_internal (instructions, printcharfun, escapeflag);
@@ -1953,17 +1789,16 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
   }
 
   /* COMPILED_CONSTANTS = 2 */
-  write_c_string (" ", printcharfun);
+  write_c_string (printcharfun, " ");
   print_internal (compiled_function_constants (f), printcharfun, escapeflag);
 
   /* COMPILED_STACK_DEPTH = 3 */
-  sprintf (buf, " %d", compiled_function_stack_depth (f));
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, " %d", compiled_function_stack_depth (f));
 
   /* COMPILED_DOC_STRING = 4 */
   if (docp || intp)
     {
-      write_c_string (" ", printcharfun);
+      write_c_string (printcharfun, " ");
       print_internal (compiled_function_documentation (f), printcharfun,
 		      escapeflag);
     }
@@ -1971,13 +1806,13 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
   /* COMPILED_INTERACTIVE = 5 */
   if (intp)
     {
-      write_c_string (" ", printcharfun);
+      write_c_string (printcharfun, " ");
       print_internal (compiled_function_interactive (f), printcharfun,
 		      escapeflag);
     }
 
   UNGCPRO;
-  write_c_string (print_readably ? "]" : ">", printcharfun);
+  write_c_string (printcharfun, print_readably ? "]" : ">");
 }
 
 
@@ -1985,6 +1820,7 @@ static Lisp_Object
 mark_compiled_function (Lisp_Object obj)
 {
   Lisp_Compiled_Function *f = XCOMPILED_FUNCTION (obj);
+  int i;
 
   mark_object (f->instructions);
   mark_object (f->arglist);
@@ -1992,6 +1828,9 @@ mark_compiled_function (Lisp_Object obj)
 #ifdef COMPILED_FUNCTION_ANNOTATION_HACK
   mark_object (f->annotated);
 #endif
+  for (i = 0; i < f->args_in_array; i++)
+    mark_object (f->args[i]);
+
   /* tail-recurse on constants */
   return f->constants;
 }
@@ -2013,7 +1852,7 @@ compiled_function_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
 		     f2->doc_and_interactive, depth + 1));
 }
 
-static unsigned long
+static Hashcode
 compiled_function_hash (Lisp_Object obj, int depth)
 {
   Lisp_Compiled_Function *f = XCOMPILED_FUNCTION (obj);
@@ -2024,7 +1863,10 @@ compiled_function_hash (Lisp_Object obj, int depth)
 		internal_hash (f->constants,    depth + 1));
 }
 
-static const struct lrecord_description compiled_function_description[] = {
+static const struct memory_description compiled_function_description[] = {
+  { XD_INT,         offsetof (Lisp_Compiled_Function, args_in_array) },
+  { XD_STRUCT_PTR,  offsetof (Lisp_Compiled_Function, args),
+      XD_INDIRECT (0, 0), &lisp_object_description },
   { XD_LISP_OBJECT, offsetof (Lisp_Compiled_Function, instructions) },
   { XD_LISP_OBJECT, offsetof (Lisp_Compiled_Function, constants) },
   { XD_LISP_OBJECT, offsetof (Lisp_Compiled_Function, arglist) },
@@ -2036,6 +1878,7 @@ static const struct lrecord_description compiled_function_description[] = {
 };
 
 DEFINE_BASIC_LRECORD_IMPLEMENTATION ("compiled-function", compiled_function,
+				     1, /*dumpable_flag*/
 				     mark_compiled_function,
 				     print_compiled_function, 0,
 				     compiled_function_equal,
@@ -2071,9 +1914,9 @@ compiled_function_instructions (Lisp_Compiled_Function *f)
     /* Invert action performed by optimize_byte_code() */
     Lisp_Opaque *opaque = XOPAQUE (f->instructions);
 
-    Bufbyte * const buffer =
-      alloca_array (Bufbyte, OPAQUE_SIZE (opaque) * MAX_EMCHAR_LEN);
-    Bufbyte *bp = buffer;
+    Ibyte * const buffer =
+      alloca_array (Ibyte, OPAQUE_SIZE (opaque) * MAX_ICHAR_LEN);
+    Ibyte *bp = buffer;
 
     const Opbyte * const program = (const Opbyte *) OPAQUE_DATA (opaque);
     const Opbyte *program_ptr = program;
@@ -2082,7 +1925,7 @@ compiled_function_instructions (Lisp_Compiled_Function *f)
     while (program_ptr < program_end)
       {
 	Opcode opcode = (Opcode) READ_UINT_1;
-	bp += set_charptr_emchar (bp, opcode);
+	bp += set_itext_ichar (bp, opcode);
 	switch (opcode)
 	  {
 	  case Bvarref+7:
@@ -2091,8 +1934,8 @@ compiled_function_instructions (Lisp_Compiled_Function *f)
 	  case Bcall+7:
 	  case Bunbind+7:
 	  case Bconstant2:
-	    bp += set_charptr_emchar (bp, READ_UINT_1);
-	    bp += set_charptr_emchar (bp, READ_UINT_1);
+	    bp += set_itext_ichar (bp, READ_UINT_1);
+	    bp += set_itext_ichar (bp, READ_UINT_1);
 	    break;
 
 	  case Bvarref+6:
@@ -2103,7 +1946,7 @@ compiled_function_instructions (Lisp_Compiled_Function *f)
 	  case BlistN:
 	  case BconcatN:
 	  case BinsertN:
-	    bp += set_charptr_emchar (bp, READ_UINT_1);
+	    bp += set_itext_ichar (bp, READ_UINT_1);
 	    break;
 
 	  case Bgoto:
@@ -2117,8 +1960,8 @@ compiled_function_instructions (Lisp_Compiled_Function *f)
 	      Opbyte *buf2p = buf2;
 	      /* Convert back to program-relative address */
 	      WRITE_INT16 (jump + (program_ptr - 2 - program), buf2p);
-	      bp += set_charptr_emchar (bp, buf2[0]);
-	      bp += set_charptr_emchar (bp, buf2[1]);
+	      bp += set_itext_ichar (bp, buf2[0]);
+	      bp += set_itext_ichar (bp, buf2[1]);
 	      break;
 	    }
 
@@ -2127,7 +1970,7 @@ compiled_function_instructions (Lisp_Compiled_Function *f)
 	  case BRgotoifnonnil:
 	  case BRgotoifnilelsepop:
 	  case BRgotoifnonnilelsepop:
-	    bp += set_charptr_emchar (bp, READ_INT_1 + 127);
+	    bp += set_itext_ichar (bp, READ_INT_1 + 127);
 	    break;
 
 	  default:
@@ -2302,8 +2145,7 @@ If non-nil, the return value will be a list whose first element is
 
 #ifdef COMPILED_FUNCTION_ANNOTATION_HACK
 
-/* Remove the `xx' if you wish to restore this feature */
-xxDEFUN ("compiled-function-annotation", Fcompiled_function_annotation, 1, 1, 0, /*
+DEFUN ("compiled-function-annotation", Fcompiled_function_annotation, 1, 1, 0, /*
 Return the annotation of the compiled-function object FUNCTION, or nil.
 The annotation is a piece of information indicating where this
 compiled-function object came from.  Generally this will be
@@ -2350,7 +2192,8 @@ If the byte code for compiled function FUNCTION is lazy-loaded, fetch it now.
     {
       Lisp_Object tem = read_doc_string (f->instructions);
       if (!CONSP (tem))
-	signal_simple_error ("Invalid lazy-loaded byte code", tem);
+	signal_error (Qinvalid_byte_code,
+			   "Invalid lazy-loaded byte code", tem);
       /* v18 or v19 bytecode file.  Need to Ebolify. */
       if (f->flags.ebolified && VECTORP (XCDR (tem)))
 	ebolify_bytecode_constants (XCDR (tem));
@@ -2358,8 +2201,8 @@ If the byte code for compiled function FUNCTION is lazy-loaded, fetch it now.
       f->constants    = XCDR (tem);
       return function;
     }
-  ABORT ();
-  return Qnil; /* not reached */
+  abort ();
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("optimize-compiled-function", Foptimize_compiled_function, 1, 1, 0, /*
@@ -2415,8 +2258,8 @@ syms_of_bytecode (void)
   INIT_LRECORD_IMPLEMENTATION (compiled_function);
 
   DEFERROR_STANDARD (Qinvalid_byte_code, Qinvalid_state);
-  defsymbol (&Qbyte_code, "byte-code");
-  defsymbol (&Qcompiled_functionp, "compiled-function-p");
+  DEFSYMBOL (Qbyte_code);
+  DEFSYMBOL_MULTIWORD_PREDICATE (Qcompiled_functionp);
 
   DEFSUBR (Fbyte_code);
   DEFSUBR (Ffetch_bytecode);
@@ -2435,7 +2278,7 @@ syms_of_bytecode (void)
 #endif
 
 #ifdef BYTE_CODE_METER
-  defsymbol (&Qbyte_code_meter, "byte-code-meter");
+  DEFSYMBOL (Qbyte_code_meter);
 #endif
 }
 
