@@ -1,7 +1,7 @@
 /* Primitive operations on Lisp data types for XEmacs Lisp interpreter.
    Copyright (C) 1985, 1986, 1988, 1992, 1993, 1994, 1995
    Free Software Foundation, Inc.
-   Copyright (C) 2000 Ben Wing.
+   Copyright (C) 2000, 2001, 2002, 2003 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -31,29 +31,26 @@ Boston, MA 02111-1307, USA.  */
 #include "buffer.h"
 #include "bytecode.h"
 #include "syssignal.h"
-
-#ifdef LISP_FLOAT_TYPE
-/* Need to define a differentiating symbol -- see sysfloat.h */
-# define THIS_FILENAME data_c
-# include "sysfloat.h"
-#endif /* LISP_FLOAT_TYPE */
+#include "sysfloat.h"
 
 Lisp_Object Qnil, Qt, Qquote, Qlambda, Qunbound;
 Lisp_Object Qerror_conditions, Qerror_message;
 Lisp_Object Qerror, Qquit, Qsyntax_error, Qinvalid_read_syntax;
-Lisp_Object Qlist_formation_error;
+Lisp_Object Qlist_formation_error, Qstructure_formation_error;
 Lisp_Object Qmalformed_list, Qmalformed_property_list;
 Lisp_Object Qcircular_list, Qcircular_property_list;
-Lisp_Object Qinvalid_argument, Qwrong_type_argument, Qargs_out_of_range;
+Lisp_Object Qinvalid_argument, Qinvalid_constant, Qwrong_type_argument;
+Lisp_Object Qargs_out_of_range;
 Lisp_Object Qwrong_number_of_arguments, Qinvalid_function, Qno_catch;
-Lisp_Object Qinternal_error, Qinvalid_state;
+Lisp_Object Qinternal_error, Qinvalid_state, Qstack_overflow, Qout_of_memory;
 Lisp_Object Qvoid_variable, Qcyclic_variable_indirection;
 Lisp_Object Qvoid_function, Qcyclic_function_indirection;
-Lisp_Object Qinvalid_operation, Qinvalid_change;
+Lisp_Object Qinvalid_operation, Qinvalid_change, Qprinting_unreadable_object;
 Lisp_Object Qsetting_constant;
 Lisp_Object Qediting_error;
 Lisp_Object Qbeginning_of_buffer, Qend_of_buffer, Qbuffer_read_only;
-Lisp_Object Qio_error, Qend_of_file;
+Lisp_Object Qio_error, Qfile_error, Qconversion_error, Qend_of_file;
+Lisp_Object Qtext_conversion_error;
 Lisp_Object Qarith_error, Qrange_error, Qdomain_error;
 Lisp_Object Qsingularity_error, Qoverflow_error, Qunderflow_error;
 Lisp_Object Qintegerp, Qnatnump, Qsymbolp;
@@ -65,6 +62,7 @@ Lisp_Object Qinteger_or_char_p, Qinteger_char_or_marker_p;
 Lisp_Object Qnumberp, Qnumber_char_or_marker_p;
 Lisp_Object Qbit_vectorp, Qbitp, Qcdr;
 
+Lisp_Object Qerror_lacks_explanatory_string;
 Lisp_Object Qfloatp;
 
 #ifdef DEBUG_XEMACS
@@ -81,12 +79,13 @@ eq_with_ebola_notice (Lisp_Object obj1, Lisp_Object obj2)
     {
       /* #### It would be really nice if this were a proper warning
          instead of brain-dead print to Qexternal_debugging_output.  */
-      write_c_string ("Comparison between integer and character is constant nil (",
-		      Qexternal_debugging_output);
+      write_c_string
+	(Qexternal_debugging_output,
+	 "Comparison between integer and character is constant nil (");
       Fprinc (obj1, Qexternal_debugging_output);
-      write_c_string (" and ", Qexternal_debugging_output);
+      write_c_string (Qexternal_debugging_output, " and ");
       Fprinc (obj2, Qexternal_debugging_output);
-      write_c_string (")\n", Qexternal_debugging_output);
+      write_c_string (Qexternal_debugging_output, ")\n");
       debug_short_backtrace (debug_ebola_backtrace_length);
     }
   return EQ (obj1, obj2);
@@ -113,7 +112,7 @@ wrong_type_argument (Lisp_Object predicate, Lisp_Object value)
 DOESNT_RETURN
 dead_wrong_type_argument (Lisp_Object predicate, Lisp_Object value)
 {
-  signal_error (Qwrong_type_argument, list2 (predicate, value));
+  signal_error_1 (Qwrong_type_argument, list2 (predicate, value));
 }
 
 DEFUN ("wrong-type-argument", Fwrong_type_argument, 2, 2, 0, /*
@@ -131,25 +130,27 @@ PREDICATE.  At that point, the gotten value is returned.
 DOESNT_RETURN
 c_write_error (Lisp_Object obj)
 {
-  signal_simple_error ("Attempt to modify read-only object (c)", obj);
+  signal_error (Qsetting_constant,
+		"Attempt to modify read-only object (c)", obj);
 }
 
 DOESNT_RETURN
 lisp_write_error (Lisp_Object obj)
 {
-  signal_simple_error ("Attempt to modify read-only object (lisp)", obj);
+  signal_error (Qsetting_constant,
+		"Attempt to modify read-only object (lisp)", obj);
 }
 
 DOESNT_RETURN
 args_out_of_range (Lisp_Object a1, Lisp_Object a2)
 {
-  signal_error (Qargs_out_of_range, list2 (a1, a2));
+  signal_error_1 (Qargs_out_of_range, list2 (a1, a2));
 }
 
 DOESNT_RETURN
 args_out_of_range_3 (Lisp_Object a1, Lisp_Object a2, Lisp_Object a3)
 {
-  signal_error (Qargs_out_of_range, list3 (a1, a2, a3));
+  signal_error_1 (Qargs_out_of_range, list3 (a1, a2, a3));
 }
 
 void
@@ -170,10 +171,10 @@ EMACS_INT sign_extend_lisp_int (EMACS_INT num);
 EMACS_INT
 sign_extend_lisp_int (EMACS_INT num)
 {
-  if (num & (1L << (INT_VALBITS - 1)))
-    return num | ((-1L) << INT_VALBITS);
+  if (num & (1L << (VALBITS - 1)))
+    return num | ((-1L) << VALBITS);
   else
-    return num & (EMACS_INT) ((1UL << INT_VALBITS) - 1);
+    return num & ((1L << VALBITS) - 1);
 }
 
 
@@ -363,10 +364,10 @@ If non-nil, the return value will be a list whose first element is
 */
        (subr))
 {
-  const char *prompt;
+  const CIbyte *prompt;
   CHECK_SUBR (subr);
   prompt = XSUBR (subr)->prompt;
-  return prompt ? list2 (Qinteractive, build_string (prompt)) : Qnil;
+  return prompt ? list2 (Qinteractive, build_msg_string (prompt)) : Qnil;
 }
 
 
@@ -534,7 +535,6 @@ Return t if OBJECT is a number, character or a marker.
     ? Qt : Qnil;
 }
 
-#ifdef LISP_FLOAT_TYPE
 DEFUN ("floatp", Ffloatp, 1, 1, 0, /*
 Return t if OBJECT is a floating point number.
 */
@@ -542,7 +542,6 @@ Return t if OBJECT is a floating point number.
 {
   return FLOATP (object) ? Qt : Qnil;
 }
-#endif /* LISP_FLOAT_TYPE */
 
 DEFUN ("type-of", Ftype_of, 1, 1, 0, /*
 Return a symbol representing the type of OBJECT.
@@ -715,13 +714,14 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
     }
   else if (BIT_VECTORP (array))
     {
-      if (idx >= bit_vector_length (XBIT_VECTOR (array))) goto range_error;
+      if (idx >= (EMACS_INT) bit_vector_length (XBIT_VECTOR (array)))
+	goto range_error;
       return make_int (bit_vector_bit (XBIT_VECTOR (array), idx));
     }
   else if (STRINGP (array))
     {
-      if (idx >= XSTRING_CHAR_LENGTH (array)) goto range_error;
-      return make_char (string_char (XSTRING (array), idx));
+      if (idx >= string_char_length (array)) goto range_error;
+      return make_char (string_ichar (array, idx));
     }
 #ifdef LOSING_BYTECODE
   else if (COMPILED_FUNCTIONP (array))
@@ -739,7 +739,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
 
  range_error:
   args_out_of_range (array, index_);
-  return Qnil; /* not reached */
+  RETURN_NOT_REACHED (Qnil);
 }
 
 DEFUN ("aset", Faset, 3, 3, 0, /*
@@ -762,6 +762,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
 
   if (idx < 0) goto range_error;
 
+  CHECK_LISP_WRITEABLE (array);
   if (VECTORP (array))
     {
       if (idx >= XVECTOR_LENGTH (array)) goto range_error;
@@ -769,15 +770,16 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
     }
   else if (BIT_VECTORP (array))
     {
-      if (idx >= bit_vector_length (XBIT_VECTOR (array))) goto range_error;
+      if (idx >= (EMACS_INT) bit_vector_length (XBIT_VECTOR (array)))
+	goto range_error;
       CHECK_BIT (newval);
       set_bit_vector_bit (XBIT_VECTOR (array), idx, !ZEROP (newval));
     }
   else if (STRINGP (array))
     {
       CHECK_CHAR_COERCE_INT (newval);
-      if (idx >= XSTRING_CHAR_LENGTH (array)) goto range_error;
-      set_string_char (XSTRING (array), idx, XCHAR (newval));
+      if (idx >= string_char_length (array)) goto range_error;
+      set_string_char (array, idx, XCHAR (newval));
       bump_string_modiff (array);
     }
   else
@@ -790,7 +792,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
 
  range_error:
   args_out_of_range (array, index_);
-  return Qnil; /* not reached */
+  RETURN_NOT_REACHED (Qnil);
 }
 
 
@@ -815,9 +817,7 @@ number_char_or_marker_to_int_or_double (Lisp_Object obj, int_or_double *p)
   if      (INTP    (obj)) p->c.ival = XINT  (obj);
   else if (CHARP   (obj)) p->c.ival = XCHAR (obj);
   else if (MARKERP (obj)) p->c.ival = marker_position (obj);
-#ifdef LISP_FLOAT_TYPE
   else if (FLOATP  (obj)) p->c.dval = XFLOAT_DATA (obj), p->int_p = 0;
-#endif
   else
     {
       obj = wrong_type_argument (Qnumber_char_or_marker_p, obj);
@@ -832,9 +832,7 @@ number_char_or_marker_to_double (Lisp_Object obj)
   if      (INTP    (obj)) return (double) XINT  (obj);
   else if (CHARP   (obj)) return (double) XCHAR (obj);
   else if (MARKERP (obj)) return (double) marker_position (obj);
-#ifdef LISP_FLOAT_TYPE
   else if (FLOATP  (obj)) return XFLOAT_DATA (obj);
-#endif
   else
     {
       obj = wrong_type_argument (Qnumber_char_or_marker_p, obj);
@@ -961,10 +959,8 @@ Return t if NUMBER is zero.
  retry:
   if (INTP (number))
     return EQ (number, Qzero) ? Qt : Qnil;
-#ifdef LISP_FLOAT_TYPE
   else if (FLOATP (number))
     return XFLOAT_DATA (number) == 0.0 ? Qt : Qnil;
-#endif /* LISP_FLOAT_TYPE */
   else
     {
       number = wrong_type_argument (Qnumberp, number);
@@ -1008,11 +1004,8 @@ NUMBER may be an integer or a floating point number.
 */
        (number))
 {
-  char buffer[VALBITS];
-
   CHECK_INT_OR_FLOAT (number);
 
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP (number))
     {
       char pigbuf[350];	/* see comments in float_to_string */
@@ -1020,10 +1013,13 @@ NUMBER may be an integer or a floating point number.
       float_to_string (pigbuf, XFLOAT_DATA (number));
       return build_string (pigbuf);
     }
-#endif /* LISP_FLOAT_TYPE */
 
-  long_to_string (buffer, XINT (number));
-  return build_string (buffer);
+  {
+    char buffer[DECIMAL_PRINT_SIZE (long)];
+
+    long_to_string (buffer, XINT (number));
+    return build_string (buffer);
+  }
 }
 
 static int
@@ -1070,10 +1066,8 @@ Floating point numbers always use base 10.
   while (*p == ' ' || *p == '\t')
     p++;
 
-#ifdef LISP_FLOAT_TYPE
   if (isfloat_string (p) && b == 10)
     return make_float (atof (p));
-#endif /* LISP_FLOAT_TYPE */
 
   if (b == 10)
     {
@@ -1260,7 +1254,7 @@ With one argument, reciprocates the argument.
 
  divide_by_zero:
   Fsignal (Qarith_error, Qnil);
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("max", Fmax, 1, MANY, 0, /*
@@ -1455,7 +1449,6 @@ If either argument is a float, a float will be returned.
   number_char_or_marker_to_int_or_double (x, &iod1);
   number_char_or_marker_to_int_or_double (y, &iod2);
 
-#ifdef LISP_FLOAT_TYPE
   if (!iod1.int_p || !iod2.int_p)
     {
       double dval1 = iod1.int_p ? (double) iod1.c.ival : iod1.c.dval;
@@ -1469,7 +1462,7 @@ If either argument is a float, a float will be returned.
 
       return make_float (dval1);
     }
-#endif /* LISP_FLOAT_TYPE */
+
   {
     EMACS_INT ival;
     if (iod2.c.ival == 0) goto divide_by_zero;
@@ -1485,7 +1478,7 @@ If either argument is a float, a float will be returned.
 
  divide_by_zero:
   Fsignal (Qarith_error, Qnil);
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("ash", Fash, 2, 2, 0, /*
@@ -1529,9 +1522,7 @@ Markers and characters are converted to integers.
   if (INTP    (number)) return make_int (XINT  (number) + 1);
   if (CHARP   (number)) return make_int (XCHAR (number) + 1);
   if (MARKERP (number)) return make_int (marker_position (number) + 1);
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP  (number)) return make_float (XFLOAT_DATA (number) + 1.0);
-#endif /* LISP_FLOAT_TYPE */
 
   number = wrong_type_argument (Qnumber_char_or_marker_p, number);
   goto retry;
@@ -1548,9 +1539,7 @@ Markers and characters are converted to integers.
   if (INTP    (number)) return make_int (XINT  (number) - 1);
   if (CHARP   (number)) return make_int (XCHAR (number) - 1);
   if (MARKERP (number)) return make_int (marker_position (number) - 1);
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP  (number)) return make_float (XFLOAT_DATA (number) - 1.0);
-#endif /* LISP_FLOAT_TYPE */
 
   number = wrong_type_argument (Qnumber_char_or_marker_p, number);
   goto retry;
@@ -1582,14 +1571,11 @@ static void
 print_weak_list (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   if (print_readably)
-    error ("printing unreadable object #<weak-list>");
+    printing_unreadable_object ("#<weak-list>");
 
-  write_c_string ("#<weak-list ", printcharfun);
-  print_internal (encode_weak_list_type (XWEAK_LIST (obj)->type),
-		  printcharfun, 0);
-  write_c_string (" ", printcharfun);
-  print_internal (XWEAK_LIST (obj)->list, printcharfun, escapeflag);
-  write_c_string (">", printcharfun);
+  write_fmt_string_lisp (printcharfun, "#<weak-list %s %S>", 2,
+			 encode_weak_list_type (XWEAK_LIST (obj)->type),
+			 XWEAK_LIST (obj)->list);
 }
 
 static int
@@ -1602,12 +1588,12 @@ weak_list_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
 	  internal_equal (w1->list, w2->list, depth + 1));
 }
 
-static unsigned long
+static Hashcode
 weak_list_hash (Lisp_Object obj, int depth)
 {
   struct weak_list *w = XWEAK_LIST (obj);
 
-  return HASH2 ((unsigned long) w->type,
+  return HASH2 ((Hashcode) w->type,
 		internal_hash (w->list, depth + 1));
 }
 
@@ -1620,19 +1606,22 @@ make_weak_list (enum weak_list_type type)
 
   wl->list = Qnil;
   wl->type = type;
-  XSETWEAK_LIST (result, wl);
+  result = wrap_weak_list (wl);
   wl->next_weak = Vall_weak_lists;
   Vall_weak_lists = result;
   return result;
 }
 
-static const struct lrecord_description weak_list_description[] = {
-  { XD_LISP_OBJECT, offsetof (struct weak_list, list) },
-  { XD_LO_LINK,     offsetof (struct weak_list, next_weak) },
+static const struct memory_description weak_list_description[] = {
+  { XD_LISP_OBJECT, offsetof (struct weak_list, list), 
+  0, 0, XD_FLAG_NO_KKCC },
+  { XD_LO_LINK,     offsetof (struct weak_list, next_weak), 
+  0, 0, XD_FLAG_NO_KKCC },
   { XD_END }
 };
 
 DEFINE_LRECORD_IMPLEMENTATION ("weak-list", weak_list,
+			       1, /*dumpable-flag*/
 			       mark_weak_list, print_weak_list,
 			       0, weak_list_equal, weak_list_hash,
 			       weak_list_description,
@@ -1769,12 +1758,16 @@ finish_marking_weak_lists (void)
 	      break;
 
 	    default:
-	      ABORT ();
+	      abort ();
 	    }
 
 	  if (need_to_mark_elem && ! marked_p (elem))
 	    {
+#ifdef USE_KKCC
+	      kkcc_gc_stack_push_lisp_object (elem);
+#else /* NOT USE_KKCC */
 	      mark_object (elem);
+#endif /* NOT USE_KKCC */
 	      did_mark = 1;
 	    }
 
@@ -1798,7 +1791,11 @@ finish_marking_weak_lists (void)
          because we're not removing it */
       if (!NILP (rest2) && ! marked_p (rest2))
 	{
+#ifdef USE_KKCC
+	  kkcc_gc_stack_push_lisp_object (rest2);
+#else /* NOT USE_KKCC */
 	  mark_object (rest2);
+#endif /* NOT USE_KKCC */
 	  did_mark = 1;
 	}
     }
@@ -1910,8 +1907,8 @@ decode_weak_list_type (Lisp_Object symbol)
   if (EQ (symbol, Qvalue_assoc)) return WEAK_LIST_VALUE_ASSOC;
   if (EQ (symbol, Qfull_assoc))  return WEAK_LIST_FULL_ASSOC;
 
-  signal_simple_error ("Invalid weak list type", symbol);
-  return WEAK_LIST_SIMPLE; /* not reached */
+  invalid_constant ("Invalid weak list type", symbol);
+  RETURN_NOT_REACHED (WEAK_LIST_SIMPLE);
 }
 
 static Lisp_Object
@@ -1925,10 +1922,10 @@ encode_weak_list_type (enum weak_list_type type)
     case WEAK_LIST_VALUE_ASSOC: return Qvalue_assoc;
     case WEAK_LIST_FULL_ASSOC:  return Qfull_assoc;
     default:
-      ABORT ();
+      abort ();
     }
 
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("weak-list-p", Fweak_list_p, 1, 1, 0, /*
@@ -2001,6 +1998,376 @@ Change the list contained in a weak-list object.
   return new_list;
 }
 
+
+/************************************************************************/
+/*                              weak boxes                              */
+/************************************************************************/
+
+static Lisp_Object Vall_weak_boxes; /* Gemarke es niemals ever!!! */
+
+void
+prune_weak_boxes (void)
+{
+  Lisp_Object rest, prev = Qnil;
+  int removep = 0;
+
+  for (rest = Vall_weak_boxes;
+       !NILP(rest);
+       rest = XWEAK_BOX (rest)->next_weak_box)
+    {
+      if (! (marked_p (rest)))
+	/* This weak box itself is garbage. */
+	removep = 1;
+
+       if (! marked_p (XWEAK_BOX (rest)->value))
+	 {
+	   XSET_WEAK_BOX (rest, Qnil);
+	   removep = 1;
+	 }
+
+       if (removep)
+	 {
+	   /* Remove weak box from list. */
+	   if (NILP (prev))
+	     Vall_weak_boxes = XWEAK_BOX (rest)->next_weak_box;
+	   else
+	     XWEAK_BOX (prev)->next_weak_box = XWEAK_BOX (rest)->next_weak_box;
+	   removep = 0;
+	 }
+       else
+	 prev = rest;
+    }
+}
+
+static Lisp_Object
+mark_weak_box (Lisp_Object obj)
+{
+  return Qnil;
+}
+
+static void
+print_weak_box (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
+{
+  if (print_readably)
+    printing_unreadable_object ("#<weak_box>");
+  write_fmt_string (printcharfun, "#<weak_box>");
+}
+
+static int
+weak_box_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
+{
+  struct weak_box *wb1 = XWEAK_BOX (obj1);
+  struct weak_box *wb2 = XWEAK_BOX (obj2);
+
+  return (internal_equal (wb1->value, wb2->value, depth + 1));
+}
+
+static Hashcode
+weak_box_hash (Lisp_Object obj, int depth)
+{
+  struct weak_box *wb = XWEAK_BOX (obj);
+
+  return internal_hash (wb->value, depth + 1);
+}
+
+Lisp_Object
+make_weak_box (Lisp_Object value)
+{
+  Lisp_Object result;
+
+  struct weak_box *wb =
+    alloc_lcrecord_type (struct weak_box, &lrecord_weak_box);
+
+  wb->value = value;
+  result = wrap_weak_box (wb);
+  wb->next_weak_box = Vall_weak_boxes;
+  Vall_weak_boxes = result;
+  return result;
+}
+
+static const struct memory_description weak_box_description[] = {
+  { XD_LO_LINK, offsetof (struct weak_box, value) },
+  { XD_END}
+};
+
+DEFINE_LRECORD_IMPLEMENTATION ("weak_box", weak_box,
+			       0, /*dumpable-flag*/
+			       mark_weak_box, print_weak_box,
+			       0, weak_box_equal, weak_box_hash,
+			       weak_box_description,
+			       struct weak_box);
+
+DEFUN ("make-weak-box", Fmake_weak_box, 1, 1, 0, /*
+Return a new weak box from value CONTENTS.
+The weak box is a reference to CONTENTS which may be extracted with
+`weak-box-ref'.  However, the weak box does not contribute to the
+reachability of CONTENTS.  When CONTENTS is garbage-collected,
+`weak-box-ref' will return NIL.
+*/
+       (value))
+{
+  return make_weak_box(value);
+}
+
+DEFUN ("weak-box-ref", Fweak_box_ref, 1, 1, 0, /*
+Return the contents of weak box WEAK-BOX.
+If the contents have been GCed, return NIL.
+*/
+       (wb))
+{
+  return XWEAK_BOX (wb)->value;
+}
+
+DEFUN ("weak-box-p", Fweak_boxp, 1, 1, 0, /*
+Return non-nil if OBJECT is a weak box.
+*/
+       (object))
+{
+  return WEAK_BOXP (object) ? Qt : Qnil;
+}
+
+/************************************************************************/
+/*                              ephemerons                              */
+/************************************************************************/
+
+/* The concept of ephemerons is due to:
+ * Barry Hayes: Ephemerons: A New Finalization Mechanism. OOPSLA 1997: 176-183
+ * The original idea is due to George Bosworth of Digitalk, Inc.
+ *
+ * For a discussion of finalization and weakness that also reviews
+ * ephemerons, refer to:
+ * Simon Peyton Jones, Simon Marlow, Conal Elliot:
+ * Stretching the storage manager
+ * Implementation of Functional Languages, 1999
+ */
+
+static Lisp_Object Vall_ephemerons; /* Gemarke es niemals ever!!! */
+static Lisp_Object Vnew_all_ephemerons;
+static Lisp_Object Vfinalize_list;
+
+void
+init_marking_ephemerons(void)
+{
+  Vnew_all_ephemerons = Qnil;
+}
+
+/* Move all live ephemerons with live keys over to
+ * Vnew_all_ephemerons, marking the values and finalizers along the
+ * way. */
+
+int
+continue_marking_ephemerons(void)
+{
+  Lisp_Object rest = Vall_ephemerons, next, prev = Qnil;
+  int did_mark = 0;
+
+  while (!NILP (rest))
+    {
+      next = XEPHEMERON_NEXT (rest);
+
+      if (marked_p (rest))
+	{
+	  MARK_CONS (XCONS (XEPHEMERON (rest)->cons_chain));
+	  if (marked_p (XEPHEMERON (rest)->key))
+	    {
+#ifdef USE_KKCC
+	      kkcc_gc_stack_push_lisp_object 
+	      (XCAR (XEPHEMERON (rest)->cons_chain));
+#else /* NOT USE_KKCC */
+	      mark_object (XCAR (XEPHEMERON (rest)->cons_chain));
+#endif /* NOT USE_KKCC */
+	      did_mark = 1;
+	      XSET_EPHEMERON_NEXT (rest, Vnew_all_ephemerons);
+	      Vnew_all_ephemerons = rest;
+	      if (NILP (prev))
+		Vall_ephemerons = next;
+	      else
+		XSET_EPHEMERON_NEXT (prev, next);
+	    }
+	  else
+	    prev = rest;
+	}
+      else
+	prev = rest;
+
+      rest = next;
+    }
+
+  return did_mark;
+}
+
+/* At this point, everything that's in Vall_ephemerons is dead.
+ * Well, almost: we still need to run the finalizers, so we need to
+ * resurrect them.
+ */
+
+int
+finish_marking_ephemerons(void)
+{
+  Lisp_Object rest = Vall_ephemerons, next, prev = Qnil;
+  int did_mark = 0;
+
+  while (! NILP (rest))
+    {
+      next = XEPHEMERON_NEXT (rest);
+
+      if (marked_p (rest))
+	/* The ephemeron itself is live, but its key is garbage */
+	{
+	  /* tombstone */
+	  XSET_EPHEMERON_VALUE (rest, Qnil);
+
+	  if (! NILP (XEPHEMERON_FINALIZER (rest)))
+	    {
+	      MARK_CONS (XCONS (XEPHEMERON (rest)->cons_chain));
+#ifdef USE_KKCC
+	      kkcc_gc_stack_push_lisp_object 
+	      (XCAR (XEPHEMERON (rest)->cons_chain));
+#else /* NOT USE_KKCC */
+	      mark_object (XCAR (XEPHEMERON (rest)->cons_chain));
+#endif /* NOT USE_KKCC */
+
+	      /* Register the finalizer */
+	      XSET_EPHEMERON_NEXT (rest, Vfinalize_list);
+	      Vfinalize_list = XEPHEMERON (rest)->cons_chain;
+	      did_mark = 1;
+	    }
+
+	  /* Remove it from the list. */
+	  if (NILP (prev))
+	    Vall_ephemerons = next;
+	  else
+	    XSET_EPHEMERON_NEXT (prev, next);
+	}
+      else
+	prev = rest;
+
+      rest = next;
+    }
+  
+  return did_mark;
+}
+
+void
+prune_ephemerons(void)
+{
+  Vall_ephemerons = Vnew_all_ephemerons;
+}
+
+Lisp_Object
+zap_finalize_list(void)
+{
+  Lisp_Object finalizers = Vfinalize_list;
+
+  Vfinalize_list = Qnil;
+
+  return finalizers;
+}
+
+static Lisp_Object
+mark_ephemeron (Lisp_Object obj)
+{
+  return Qnil;
+}
+
+static void
+print_ephemeron (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
+{
+  if (print_readably)
+    printing_unreadable_object ("#<ephemeron>");
+  write_fmt_string (printcharfun, "#<ephemeron>");
+}
+
+static int
+ephemeron_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
+{
+  return
+    internal_equal (XEPHEMERON_REF (obj1), XEPHEMERON_REF(obj2), depth + 1);
+}
+
+static Hashcode
+ephemeron_hash(Lisp_Object obj, int depth)
+{
+  return internal_hash (XEPHEMERON_REF (obj), depth + 1);
+}
+
+Lisp_Object
+make_ephemeron(Lisp_Object key, Lisp_Object value, Lisp_Object finalizer)
+{
+  Lisp_Object result, temp = Qnil;
+  struct gcpro gcpro1, gcpro2;
+
+  struct ephemeron *eph =
+    alloc_lcrecord_type (struct ephemeron, &lrecord_ephemeron);
+
+  eph->key = Qnil;
+  eph->cons_chain = Qnil;
+  eph->value = Qnil;
+
+  result = wrap_ephemeron(eph);
+  GCPRO2 (result, temp);
+
+  eph->key = key;
+  temp = Fcons(value, finalizer);
+  eph->cons_chain = Fcons(temp, Vall_ephemerons);
+  eph->value = value;
+
+  Vall_ephemerons = result;
+
+  UNGCPRO;
+  return result;
+}
+
+/* Ephemerons are special cases in the KKCC mark algorithm, so nothing
+   is marked here. */
+static const struct memory_description ephemeron_description[] = {
+  { XD_LISP_OBJECT, offsetof(struct ephemeron, key),
+    0, 0, XD_FLAG_NO_KKCC },
+  { XD_LISP_OBJECT, offsetof(struct ephemeron, cons_chain),
+    0, 0, XD_FLAG_NO_KKCC },
+  { XD_LISP_OBJECT, offsetof(struct ephemeron, value),
+    0, 0, XD_FLAG_NO_KKCC },
+  { XD_END }
+};
+
+DEFINE_LRECORD_IMPLEMENTATION ("ephemeron", ephemeron,
+			       0, /*dumpable-flag*/
+			       mark_ephemeron, print_ephemeron,
+			       0, ephemeron_equal, ephemeron_hash,
+			       ephemeron_description,
+			       struct ephemeron);
+
+DEFUN ("make-ephemeron", Fmake_ephemeron, 2, 3, 0, /*
+Return a new ephemeron with key KEY, value VALUE, and finalizer FINALIZER.
+The ephemeron is a reference to VALUE which may be extracted with
+`ephemeron-ref'.  VALUE is only reachable through the ephemeron as
+long as KEY is reachable; the ephemeron does not contribute to the
+reachability of KEY.  When KEY becomes unreachable while the ephemeron
+itself is still reachable, VALUE is queued for finalization: FINALIZER
+will possibly be called on VALUE some time in the future.  Moreover,
+future calls to `ephemeron-ref' will return NIL.
+*/
+       (key, value, finalizer))
+{
+  return make_ephemeron(key, value, finalizer);
+}
+
+DEFUN ("ephemeron-ref",  Fephemeron_ref, 1, 1, 0, /*
+Return the contents of ephemeron EPHEMERON.
+If the contents have been GCed, return NIL.
+*/
+       (eph))
+{
+  return XEPHEMERON_REF (eph);
+}
+
+DEFUN ("ephemeron-p", Fephemeronp, 1, 1, 0, /*
+Return non-nil if OBJECT is an ephemeron.
+*/
+       (object))
+{
+  return EPHEMERONP (object) ? Qt : Qnil;
+}
 
 /************************************************************************/
 /*                            initialization                            */
@@ -2011,7 +2378,7 @@ arith_error (int signo)
 {
   EMACS_REESTABLISH_SIGNAL (signo, arith_error);
   EMACS_UNBLOCK_SIGNAL (signo);
-  signal_error (Qarith_error, Qnil);
+  signal_error (Qarith_error, 0, Qunbound);
 }
 
 void
@@ -2021,13 +2388,11 @@ init_data_very_early (void)
      We don't want to call `signal' in this case
      so that we don't have trouble with dumping
      signal-delivering routines in an inconsistent state.  */
-#ifndef CANNOT_DUMP
   if (!initialized)
     return;
-#endif /* CANNOT_DUMP */
-  signal (SIGFPE, arith_error);
+  EMACS_SIGNAL (SIGFPE, arith_error);
 #ifdef uts
-  signal (SIGEMT, arith_error);
+  EMACS_SIGNAL (SIGEMT, arith_error);
 #endif /* uts */
 }
 
@@ -2046,28 +2411,25 @@ init_errors_once_early (void)
   DEFERROR (Qerror, "error", Qnil);
   DEFERROR_STANDARD (Qquit, Qnil);
 
-  DEFERROR (Qunimplemented, "Feature not yet implemented", Qerror);
-  DEFERROR_STANDARD (Qsyntax_error, Qerror);
-  DEFERROR_STANDARD (Qinvalid_read_syntax, Qsyntax_error);
-  DEFERROR_STANDARD (Qlist_formation_error, Qsyntax_error);
+  DEFERROR_STANDARD (Qinvalid_argument, Qerror);
 
-  /* Generated by list traversal macros */
+  DEFERROR_STANDARD (Qsyntax_error, Qinvalid_argument);
+  DEFERROR_STANDARD (Qinvalid_read_syntax, Qsyntax_error);
+  DEFERROR_STANDARD (Qstructure_formation_error, Qsyntax_error);
+  DEFERROR_STANDARD (Qlist_formation_error, Qstructure_formation_error);
   DEFERROR_STANDARD (Qmalformed_list, Qlist_formation_error);
   DEFERROR_STANDARD (Qmalformed_property_list, Qmalformed_list);
   DEFERROR_STANDARD (Qcircular_list, Qlist_formation_error);
   DEFERROR_STANDARD (Qcircular_property_list, Qcircular_list);
 
-  DEFERROR_STANDARD (Qinvalid_argument, Qerror);
   DEFERROR_STANDARD (Qwrong_type_argument, Qinvalid_argument);
   DEFERROR_STANDARD (Qargs_out_of_range, Qinvalid_argument);
   DEFERROR_STANDARD (Qwrong_number_of_arguments, Qinvalid_argument);
   DEFERROR_STANDARD (Qinvalid_function, Qinvalid_argument);
+  DEFERROR_STANDARD (Qinvalid_constant, Qinvalid_argument);
   DEFERROR (Qno_catch, "No catch for tag", Qinvalid_argument);
 
-  DEFERROR_STANDARD (Qinternal_error, Qerror);
-
-  DEFERROR (Qinvalid_state, "Properties or values have been set incorrectly",
-	    Qerror);
+  DEFERROR_STANDARD (Qinvalid_state, Qerror);
   DEFERROR (Qvoid_function, "Symbol's function definition is void",
 	    Qinvalid_state);
   DEFERROR (Qcyclic_function_indirection,
@@ -2078,22 +2440,27 @@ init_errors_once_early (void)
   DEFERROR (Qcyclic_variable_indirection,
 	    "Symbol's chain of variable indirections contains a loop",
 	    Qinvalid_state);
+  DEFERROR_STANDARD (Qstack_overflow, Qinvalid_state);
+  DEFERROR_STANDARD (Qinternal_error, Qinvalid_state);
+  DEFERROR_STANDARD (Qout_of_memory, Qinvalid_state);
 
-  DEFERROR (Qinvalid_operation,
-	    "Operation not allowed or error during operation", Qerror);
-  DEFERROR (Qinvalid_change, "Attempt to set properties or values incorrectly",
-	    Qinvalid_operation);
+  DEFERROR_STANDARD (Qinvalid_operation, Qerror);
+  DEFERROR_STANDARD (Qinvalid_change, Qinvalid_operation);
   DEFERROR (Qsetting_constant, "Attempt to set a constant symbol",
 	    Qinvalid_change);
+  DEFERROR_STANDARD (Qprinting_unreadable_object, Qinvalid_operation);
+  DEFERROR (Qunimplemented, "Feature not yet implemented", Qinvalid_operation);
 
-  DEFERROR (Qediting_error, "Invalid operation during editing",
-	    Qinvalid_operation);
+  DEFERROR_STANDARD (Qediting_error, Qinvalid_operation);
   DEFERROR_STANDARD (Qbeginning_of_buffer, Qediting_error);
   DEFERROR_STANDARD (Qend_of_buffer, Qediting_error);
   DEFERROR (Qbuffer_read_only, "Buffer is read-only", Qediting_error);
 
   DEFERROR (Qio_error, "IO Error", Qinvalid_operation);
-  DEFERROR (Qend_of_file, "End of file or stream", Qio_error);
+  DEFERROR_STANDARD (Qfile_error, Qio_error);
+  DEFERROR (Qend_of_file, "End of file or stream", Qfile_error);
+  DEFERROR_STANDARD (Qconversion_error, Qio_error);
+  DEFERROR_STANDARD (Qtext_conversion_error, Qconversion_error);
 
   DEFERROR (Qarith_error, "Arithmetic error", Qinvalid_operation);
   DEFERROR (Qrange_error, "Arithmetic range error", Qarith_error);
@@ -2107,6 +2474,8 @@ void
 syms_of_data (void)
 {
   INIT_LRECORD_IMPLEMENTATION (weak_list);
+  INIT_LRECORD_IMPLEMENTATION (ephemeron);
+  INIT_LRECORD_IMPLEMENTATION (weak_box);
 
   DEFSYMBOL (Qquote);
   DEFSYMBOL (Qlambda);
@@ -2133,11 +2502,9 @@ syms_of_data (void)
   DEFSYMBOL (Qnumberp);
   DEFSYMBOL (Qnumber_char_or_marker_p);
   DEFSYMBOL (Qcdr);
+  DEFSYMBOL (Qerror_lacks_explanatory_string);
   DEFSYMBOL_MULTIWORD_PREDICATE (Qweak_listp);
-
-#ifdef LISP_FLOAT_TYPE
   DEFSYMBOL (Qfloatp);
-#endif /* LISP_FLOAT_TYPE */
 
   DEFSUBR (Fwrong_type_argument);
 
@@ -2163,9 +2530,7 @@ syms_of_data (void)
   DEFSUBR (Fnumberp);
   DEFSUBR (Fnumber_or_marker_p);
   DEFSUBR (Fnumber_char_or_marker_p);
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Ffloatp);
-#endif /* LISP_FLOAT_TYPE */
   DEFSUBR (Fnatnump);
   DEFSUBR (Fsymbolp);
   DEFSUBR (Fkeywordp);
@@ -2222,6 +2587,13 @@ syms_of_data (void)
   DEFSUBR (Fweak_list_type);
   DEFSUBR (Fweak_list_list);
   DEFSUBR (Fset_weak_list_list);
+
+  DEFSUBR (Fmake_ephemeron);
+  DEFSUBR (Fephemeron_ref);
+  DEFSUBR (Fephemeronp);
+  DEFSUBR (Fmake_weak_box);
+  DEFSUBR (Fweak_box_ref);
+  DEFSUBR (Fweak_boxp);
 }
 
 void
@@ -2231,6 +2603,15 @@ vars_of_data (void)
   Vall_weak_lists = Qnil;
   dump_add_weak_object_chain (&Vall_weak_lists);
 
+  Vall_ephemerons = Qnil;
+  dump_add_weak_object_chain (&Vall_ephemerons);
+
+  Vfinalize_list = Qnil;
+  staticpro (&Vfinalize_list);
+
+  Vall_weak_boxes = Qnil;
+  dump_add_weak_object_chain (&Vall_weak_boxes);
+
 #ifdef DEBUG_XEMACS
   DEFVAR_BOOL ("debug-issue-ebola-notices", &debug_issue_ebola_notices /*
 If non-zero, note when your code may be suffering from char-int confoundance.
