@@ -1,6 +1,7 @@
 /* Device functions for X windows.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -29,7 +30,23 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "console-x.h"
+#include "buffer.h"
+#include "device-impl.h"
+#include "elhash.h"
+#include "events.h"
+#include "faces.h"
+#include "frame-impl.h"
+#include "redisplay.h"
+#include "sysdep.h"
+#include "window.h"
+
+#include "console-x-impl.h"
+#include "glyphs-x.h"
+#include "objects-x.h"
+
+#include "sysfile.h"
+#include "systime.h"
+
 #include "xintrinsicp.h"	/* CoreP.h needs this */
 #include <X11/CoreP.h>		/* Numerous places access the fields of
 				   a core widget directly.  We could
@@ -37,20 +54,6 @@ Boston, MA 02111-1307, USA.  */
 #include "xgccache.h"
 #include <X11/Shell.h>
 #include "xmu.h"
-#include "glyphs-x.h"
-#include "objects-x.h"
-
-#include "buffer.h"
-#include "elhash.h"
-#include "events.h"
-#include "faces.h"
-#include "frame.h"
-#include "redisplay.h"
-#include "sysdep.h"
-#include "window.h"
-
-#include "sysfile.h"
-#include "systime.h"
 
 #if defined(HAVE_SHLIB) && defined(LWLIB_USES_ATHENA) && !defined(HAVE_ATHENA_3D)
 #include "sysdll.h"
@@ -60,9 +63,9 @@ Boston, MA 02111-1307, USA.  */
 #include "offix.h"
 #endif
 
-Lisp_Object Vdefault_x_device;
-#if defined(MULE) && (defined(LWLIB_MENUBARS_MOTIF) || defined(HAVE_XIM) || defined (USE_XFONTSET))
+#ifdef MULE
 Lisp_Object Vx_app_defaults_directory;
+Lisp_Object Qget_coding_system_from_locale;
 #endif
 
 /* Qdisplay in general.c */
@@ -99,6 +102,18 @@ static XrmOptionDescRec emacs_options[] =
   {"-fontset",  "*FontSet",                     XrmoptionSepArg, NULL},
 };
 
+static const struct memory_description x_device_data_description_1 [] = {
+  { XD_LISP_OBJECT, offsetof (struct x_device, x_keysym_map_hash_table) },
+  { XD_LISP_OBJECT, offsetof (struct x_device, WM_COMMAND_frame) },
+  { XD_END }
+};
+
+extern const struct sized_memory_description x_device_data_description;
+
+const struct sized_memory_description x_device_data_description = {
+  sizeof (struct x_device), x_device_data_description_1
+};
+
 /* Functions to synchronize mirroring resources and specifiers */
 int in_resource_setting;
 
@@ -134,15 +149,16 @@ get_device_from_display (Display *dpy)
 # define FALLBACK_RESOURCE_NAME "infodock"
 #endif
 
-  if (!d) {
-    /* This isn't one of our displays.  Let's crash? */
-    stderr_out
-      ("\n%s: Fatal X Condition.  Asked about display we don't own: \"%s\"\n",
-       (STRINGP (Vinvocation_name) ?
-	(char *) XSTRING_DATA (Vinvocation_name) : FALLBACK_RESOURCE_NAME),
-       DisplayString (dpy) ? DisplayString (dpy) : "???");
-    abort();
-  }
+  if (!d)
+    {
+      /* This isn't one of our displays.  Let's crash? */
+      stderr_out
+	("\n%s: Fatal X Condition.  Asked about display we don't own: \"%s\"\n",
+	 (STRINGP (Vinvocation_name) ?
+	  (char *) XSTRING_DATA (Vinvocation_name) : FALLBACK_RESOURCE_NAME),
+	 DisplayString (dpy) ? DisplayString (dpy) : "???");
+      abort();
+    }
 
 #undef FALLBACK_RESOURCE_NAME
 
@@ -152,7 +168,7 @@ get_device_from_display (Display *dpy)
 struct device *
 decode_x_device (Lisp_Object device)
 {
-  XSETDEVICE (device, decode_device (device));
+  device = wrap_device (decode_device (device));
   CHECK_X_DEVICE (device);
   return XDEVICE (device);
 }
@@ -163,6 +179,18 @@ get_x_display (Lisp_Object device)
   return DEVICE_X_DISPLAY (decode_x_device (device));
 }
 
+static Lisp_Object
+coding_system_of_xrm_database (XrmDatabase db)
+{
+#ifdef MULE
+  const Extbyte *locale = XrmLocaleOfDatabase (db);
+  Lisp_Object localestr = build_ext_string (locale, Qbinary);
+  return call1 (Qget_coding_system_from_locale, localestr);
+#else
+  return Qbinary;
+#endif
+}
+
 
 /************************************************************************/
 /*		      initializing an X connection			*/
@@ -191,9 +219,9 @@ Xatoms_of_device_x (struct device *d)
 static void
 sanity_check_geometry_resource (Display *dpy)
 {
-  char *app_name, *app_class, *s;
-  char buf1 [255], buf2 [255];
-  char *type;
+  Extbyte *app_name, *app_class, *s;
+  Extbyte buf1 [255], buf2 [255];
+  Extbyte *type;
   XrmValue value;
   XtGetApplicationNameAndClass (dpy, &app_name, &app_class);
   strcpy (buf1, app_name);
@@ -203,6 +231,12 @@ sanity_check_geometry_resource (Display *dpy)
   strcat (buf2, "._no_._such_._resource_.Geometry");
   if (XrmGetResource (XtDatabase (dpy), buf1, buf2, &type, &value) == True)
     {
+      Ibyte *app_name_int, *app_class_int, *value_addr_int;
+      Lisp_Object codesys = coding_system_of_xrm_database (XtDatabase (dpy));
+      EXTERNAL_TO_C_STRING (app_name, app_name_int, codesys);
+      EXTERNAL_TO_C_STRING (app_class, app_class_int, codesys);
+      EXTERNAL_TO_C_STRING (value.addr, value_addr_int, codesys);
+
       warn_when_safe (Qgeometry, Qerror,
 		      "\n"
 "Apparently \"%s*geometry: %s\" or \"%s*geometry: %s\" was\n"
@@ -210,10 +244,10 @@ sanity_check_geometry_resource (Display *dpy)
 "XEmacs (and most other X programs) malfunction in obscure ways. (i.e.\n"
 "the Xt or Xm libraries will probably crash, which is a very bad thing.)\n"
 "You should always use \".geometry\" or \"*EmacsFrame.geometry\" instead.\n",
-		  app_name, (char *) value.addr,
-		  app_class, (char *) value.addr);
+		      app_name_int, value_addr_int,
+		      app_class_int, value_addr_int);
       suppress_early_error_handler_backtrace = 1;
-      error ("Invalid geometry resource");
+      syntax_error ("Invalid geometry resource", Qunbound);
     }
 }
 
@@ -222,7 +256,7 @@ x_init_device_class (struct device *d)
 {
   if (DEVICE_X_DEPTH(d) > 2)
     {
-      switch (DEVICE_X_VISUAL(d)->class)
+      switch (DEVICE_X_VISUAL(d)->X_CLASSFIELD)
 	{
 	case StaticGray:
 	case GrayScale:
@@ -310,7 +344,7 @@ have_xemacs_resources_in_xrdb (Display *dpy)
 static char valid_resource_char_p[256];
 
 static void
-validify_resource_component (char *str, size_t len)
+validify_resource_component (Extbyte *str, Bytecount len)
 {
   for (; len; len--, str++)
     if (!valid_resource_char_p[(unsigned char) (*str)])
@@ -318,11 +352,15 @@ validify_resource_component (char *str, size_t len)
 }
 
 static void
-Dynarr_add_validified_lisp_string (char_dynarr *cda, Lisp_Object str)
+Dynarr_add_validified_lisp_string (Extbyte_dynarr *cda, Lisp_Object str)
 {
-  Bytecount len = XSTRING_LENGTH (str);
-  Dynarr_add_many (cda, (char *) XSTRING_DATA (str), len);
-  validify_resource_component (Dynarr_atp (cda, Dynarr_length (cda) - len), len);
+  Bytecount len;
+  Extbyte *data;
+
+  TO_EXTERNAL_FORMAT (LISP_STRING, str, ALLOCA, (data, len), Qbinary);
+  Dynarr_add_many (cda, data, len);
+  validify_resource_component (Dynarr_atp (cda, Dynarr_length (cda) - len),
+			       len);
 }
 
 #if 0
@@ -340,23 +378,21 @@ x_comp_visual_info (const void *elem1, const void *elem2)
   if ( right == NULL )
     return 1;
 
-  if ( left->depth > right->depth ) {
+  if ( left->depth > right->depth )
     return 1;
-  }
-  else if ( left->depth == right->depth ) {
-    if ( left->colormap_size > right->colormap_size )
-      return 1;
-    if ( left->class > right->class )
-      return 1;
-    else if ( left->class < right->class )
-      return -1;
-    else
-      return 0;
-  }
-  else {
+  else if ( left->depth == right->depth )
+    {
+      if ( left->colormap_size > right->colormap_size )
+	return 1;
+      if ( left->X_CLASSFIELD > right->X_CLASSFIELD )
+	return 1;
+      else if ( left->X_CLASSFIELD < right->X_CLASSFIELD )
+	return -1;
+      else
+	return 0;
+    }
+  else
     return -1;
-  }
-
 }
 #endif /* if 0 */
 
@@ -369,7 +405,7 @@ x_try_best_visual_class (Screen *screen, int scrnum, int visual_class)
   XVisualInfo *vi_out = NULL;
   int out_count;
 
-  vi_in.class = visual_class;
+  vi_in.X_CLASSFIELD = visual_class;
   vi_in.screen = scrnum;
   vi_out = XGetVisualInfo (dpy, (VisualClassMask | VisualScreenMask),
 			   &vi_in, &out_count);
@@ -502,7 +538,7 @@ x_init_device (struct device *d, Lisp_Object props)
 
   {
     /* Get a dll handle to the main process. */
-    dll_handle xaw_dll_handle = dll_open (NULL);
+    dll_handle xaw_dll_handle = dll_open (Qnil);
 
     /* Did that fail?  If so, continue without error.
      * We could die here but, well, that's unfriendly and all -- plus I feel
@@ -518,7 +554,7 @@ x_init_device (struct device *d, Lisp_Object props)
 	/* If we found it, warn the user in big, nasty, unfriendly letters */
 	if (xaw_function_handle != NULL)
 	  {
-	    warn_when_safe (Qdevice, Qerror, "\n"
+	    warn_when_safe (Qdevice, Qcritical, "\n"
 "It seems that XEmacs is built dynamically linked to the flat Athena widget\n"
 "library but it finds a 3D Athena variant with the same name at runtime.\n"
 "\n"
@@ -546,7 +582,7 @@ x_init_device (struct device *d, Lisp_Object props)
 #endif /* HAVE_SHLIB and LWLIB_USES_ATHENA and not HAVE_ATHENA_3D */
 
 
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
   display = DEVICE_CONNECTION (d);
 
   allocate_x_device_struct (d);
@@ -572,7 +608,7 @@ x_init_device (struct device *d, Lisp_Object props)
   if (dpy == 0)
     {
       suppress_early_error_handler_backtrace = 1;
-      signal_simple_error ("X server not responding\n", display);
+      gui_error ("X server not responding\n", display);
     }
 
   if (STRINGP (Vx_emacs_application_class) &&
@@ -601,26 +637,25 @@ x_init_device (struct device *d, Lisp_Object props)
   speed_up_interrupts ();
 
   screen = DefaultScreen (dpy);
-  if (NILP (Vdefault_x_device))
-    Vdefault_x_device = device;
 
 #ifdef MULE
-#if defined(LWLIB_MENUBARS_MOTIF) || defined(HAVE_XIM) || defined (USE_XFONTSET)
   {
     /* Read in locale-specific resources from
        data-directory/app-defaults/$LANG/Emacs.
        This is in addition to the standard app-defaults files, and
        does not override resources defined elsewhere */
-    const char *data_dir;
-    char *path;
+    const Extbyte *data_dir;
+    Extbyte *path;
     XrmDatabase db = XtDatabase (dpy); /* #### XtScreenDatabase(dpy) ? */
-    const char *locale = XrmLocaleOfDatabase (db);
+    const Extbyte *locale = XrmLocaleOfDatabase (db);
 
     if (STRINGP (Vx_app_defaults_directory) &&
 	XSTRING_LENGTH (Vx_app_defaults_directory) > 0)
       {
-	LISP_STRING_TO_EXTERNAL (Vx_app_defaults_directory, data_dir, Qfile_name);
-	path = (char *)alloca (strlen (data_dir) + strlen (locale) + 7);
+	LISP_STRING_TO_EXTERNAL (Vx_app_defaults_directory, data_dir,
+				 Qfile_name);
+	path = (Extbyte *) ALLOCA (strlen (data_dir) + strlen (locale) +
+				   7);
 	sprintf (path, "%s%s/Emacs", data_dir, locale);
 	if (!access (path, R_OK))
 	  XrmCombineFileDatabase (path, &db, False);
@@ -628,13 +663,13 @@ x_init_device (struct device *d, Lisp_Object props)
     else if (STRINGP (Vdata_directory) && XSTRING_LENGTH (Vdata_directory) > 0)
       {
 	LISP_STRING_TO_EXTERNAL (Vdata_directory, data_dir, Qfile_name);
-	path = (char *)alloca (strlen (data_dir) + 13 + strlen (locale) + 7);
+	path = (Extbyte *) ALLOCA (strlen (data_dir) + 13 + strlen (locale) +
+				   7);
 	sprintf (path, "%sapp-defaults/%s/Emacs", data_dir, locale);
 	if (!access (path, R_OK))
 	  XrmCombineFileDatabase (path, &db, False);
       }
  }
-#endif /* LWLIB_MENUBARS_MOTIF or HAVE_XIM USE_XFONTSET */
 #endif /* MULE */
 
   if (NILP (DEVICE_NAME (d)))
@@ -682,7 +717,8 @@ x_init_device (struct device *d, Lisp_Object props)
 	    depth = atoi (str + cnt);
 	    if (depth == 0)
 	      {
-		stderr_out ("Invalid Depth specification in %s... ignoring...\n", str);
+		stderr_out ("Invalid Depth specification in %s... "
+			    "ignoring...\n", str);
 	      }
 	    else
 	      {
@@ -692,13 +728,15 @@ x_init_device (struct device *d, Lisp_Object props)
 		  }
 		else
 		  {
-		    stderr_out ("Can't match the requested visual %s... using defaults\n", str);
+		    stderr_out ("Can't match the requested visual %s... "
+				"using defaults\n", str);
 		  }
 	      }
 	  }
 	else
 	  {
-	    stderr_out( "Invalid Visual specification in %s... ignoring.\n", str);
+	    stderr_out ("Invalid Visual specification in %s... "
+			"ignoring.\n", str);
 	  }
       }
     if (visual == NULL)
@@ -718,28 +756,26 @@ x_init_device (struct device *d, Lisp_Object props)
       {
 	sprintf (buf1, "%s.privateColormap", app_name);
 	sprintf (buf2, "%s.PrivateColormap", app_class);
-	if ((visual->class == PseudoColor) &&
-	    (XrmGetResource (XtDatabase (dpy), buf1, buf2, &type, &value) == True))
-	  {
-	     cmap = XCopyColormapAndFree (dpy, DefaultColormap (dpy, screen));
-	  }
+	if ((visual->X_CLASSFIELD == PseudoColor) &&
+	    (XrmGetResource (XtDatabase (dpy), buf1, buf2, &type, &value)
+	     == True))
+	  cmap = XCopyColormapAndFree (dpy, DefaultColormap (dpy, screen));
 	else
-	  {
-	    cmap = DefaultColormap (dpy, screen);
-	  }
+	  cmap = DefaultColormap (dpy, screen);
       }
     else
       {
 	if ( best_visual_found )
-	  {
-	    cmap = XCreateColormap (dpy,  RootWindow (dpy, screen), visual, AllocNone);
-	  }
+	  cmap = XCreateColormap (dpy,  RootWindow (dpy, screen), visual,
+				  AllocNone);
 	else
 	  {
-	    /* We have to create a matching colormap anyway...
-	       #### think about using standard colormaps (need the Xmu libs?) */
-	    cmap = XCreateColormap(dpy, RootWindow(dpy, screen), visual, AllocNone);
-	    XInstallColormap(dpy, cmap);
+	    /* We have to create a matching colormap anyway...  ####
+	       think about using standard colormaps (need the Xmu
+	       libs?) */
+	    cmap = XCreateColormap (dpy, RootWindow (dpy, screen), visual,
+				    AllocNone);
+	    XInstallColormap (dpy, cmap);
 	  }
       }
   }
@@ -793,7 +829,7 @@ x_init_device (struct device *d, Lisp_Object props)
 
 
 #ifdef HAVE_OFFIX_DND
-  DndInitialize ( app_shell );
+  DndInitialize (app_shell);
 #endif
 
   Vx_initial_argv_list = make_arg_list (argc, argv);
@@ -810,7 +846,7 @@ x_init_device (struct device *d, Lisp_Object props)
   init_baud_rate (d);
   init_one_device (d);
 
-  DEVICE_X_GC_CACHE (d) = make_gc_cache (dpy, XtWindow(app_shell));
+  DEVICE_X_GC_CACHE (d) = make_gc_cache (dpy, XtWindow (app_shell));
   DEVICE_X_GRAY_PIXMAP (d) = None;
   Xatoms_of_device_x (d);
   Xatoms_of_select_x (d);
@@ -855,7 +891,7 @@ x_delete_device (struct device *d)
   int checking_free;
 #endif
 
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
   display = DEVICE_X_DISPLAY (d);
 
   if (display)
@@ -888,22 +924,6 @@ x_delete_device (struct device *d)
 #endif
     }
 
-  if (EQ (device, Vdefault_x_device))
-    {
-      Lisp_Object devcons, concons;
-      /* #### handle deleting last X device */
-      Vdefault_x_device = Qnil;
-      DEVICE_LOOP_NO_BREAK (devcons, concons)
-	{
-	  if (DEVICE_X_P (XDEVICE (XCAR (devcons))) &&
-	      !EQ (device, XCAR (devcons)))
-	    {
-	      Vdefault_x_device = XCAR (devcons);
-	      goto double_break;
-	    }
-	}
-    }
- double_break:
   free_x_device_struct (d);
 }
 
@@ -1022,6 +1042,8 @@ x_error_handler (Display *disp, XErrorEvent *event)
     }
   else
     {
+      int depth;
+
 #ifdef EXTERNAL_WIDGET
       struct frame *f;
       struct device *d = get_device_from_display (disp);
@@ -1046,7 +1068,7 @@ x_error_handler (Display *disp, XErrorEvent *event)
 
 	if (f->being_deleted)
 	  return 0;
-	XSETFRAME (frame, f);
+	frame = wrap_frame (f);
 	if (!NILP (condition_case_1 (Qerror, x_error_handler_do_enqueue,
 				     frame, x_error_handler_error, Qnil)))
 	  {
@@ -1057,11 +1079,14 @@ x_error_handler (Display *disp, XErrorEvent *event)
       }
 #endif /* EXTERNAL_WIDGET */
 
+      /* #### this should issue a warning instead of outputting to stderr */
+      depth = begin_dont_check_for_quit ();
       stderr_out ("\n%s: ",
 		  (STRINGP (Vinvocation_name)
 		   ? (char *) XSTRING_DATA (Vinvocation_name)
 		   : "xemacs"));
       XmuPrintDefaultErrorMessage (disp, event, stderr);
+      unbind_to (depth);
     }
   return 0;
 }
@@ -1089,24 +1114,26 @@ x_error_occurred_p (Display *dpy)
 int
 signal_if_x_error (Display *dpy, int resumable_p)
 {
-  char buf[1024];
+  Extbyte buf[1024];
+  Ibyte num[100];
   Lisp_Object data;
   if (! x_error_occurred_p (dpy))
     return 0;
   data = Qnil;
-  sprintf (buf, "0x%X", (unsigned int) last_error.resourceid);
-  data = Fcons (build_string (buf), data);
-  {
-    char num [32];
-    sprintf (num, "%d", last_error.request_code);
-    XGetErrorDatabaseText (last_error.display, "XRequest", num, "",
-			   buf, sizeof (buf));
-    if (! *buf)
-      sprintf (buf, "Request-%d", last_error.request_code);
-    data = Fcons (build_string (buf), data);
-  }
+  qxesprintf (num, "0x%X", (unsigned int) last_error.resourceid);
+  data = Fcons (build_intstring (num), data);
+  qxesprintf (num, "%d", last_error.request_code);
+  XGetErrorDatabaseText (last_error.display, "XRequest", (char *) num, "",
+			 buf, sizeof (buf));
+  if (*buf)
+    data = Fcons (build_ext_string (buf, Qnative), data);
+  else
+    {
+      qxesprintf (num, "Request-%d", last_error.request_code);
+      data = Fcons (build_intstring (num), data);
+    }
   XGetErrorText (last_error.display, last_error.error_code, buf, sizeof (buf));
-  data = Fcons (build_string (buf), data);
+  data = Fcons (build_ext_string (buf, Qnative), data);
  again:
   Fsignal (Qx_error, data);
   if (! resumable_p) goto again;
@@ -1124,31 +1151,37 @@ x_IO_error_handler (Display *disp)
     d = device_being_initialized;
 
   assert (d != NULL);
-  XSETDEVICE (dev, d);
+  dev = wrap_device (d);
 
   if (NILP (find_nonminibuffer_frame_not_on_device (dev)))
     {
+      int depth = begin_dont_check_for_quit ();
       /* We're going down. */
-      stderr_out
-	("\n%s: Fatal I/O Error %d (%s) on display connection \"%s\"\n",
-         (STRINGP (Vinvocation_name) ?
-	  (char *) XSTRING_DATA (Vinvocation_name) : "xemacs"),
-	 errno, strerror (errno), DisplayString (disp));
-      stderr_out
-        ("  after %lu requests (%lu known processed) with %d events remaining.\n",
-         NextRequest (disp) - 1, LastKnownRequestProcessed (disp),
-         QLength (disp));
+      Ibyte *errmess;
+      GET_STRERROR (errmess, errno);
+      stderr_out ("\n%s: Fatal I/O Error %d (%s) on display "
+		  "connection \"%s\"\n",
+		  (STRINGP (Vinvocation_name) ?
+		   (char *) XSTRING_DATA (Vinvocation_name) : "xemacs"),
+		  errno, errmess, DisplayString (disp));
+      stderr_out ("  after %lu requests (%lu known processed) with %d "
+		  "events remaining.\n",
+		  NextRequest (disp) - 1, LastKnownRequestProcessed (disp),
+		  QLength (disp));
       /* assert (!_Xdebug); */
+      unbind_to (depth);
     }
   else
     {
+      Ibyte *errmess;
+      GET_STRERROR (errmess, errno);
       warn_when_safe
 	(Qx, Qcritical,
 	 "I/O Error %d (%s) on display connection\n"
 	 "  \"%s\" after after %lu requests (%lu known processed)\n"
 	 "  with %d events remaining.\n"
 	 "  Throwing to top level.\n",
-	 errno, strerror (errno), DisplayString (disp),
+	 errno, errmess, DisplayString (disp),
          NextRequest (disp) - 1, LastKnownRequestProcessed (disp),
          QLength (disp));
     }
@@ -1202,7 +1235,7 @@ or starting the program with the `-sync' command line argument.
 
 static void
 construct_name_list (Display *display, Widget widget, char *fake_name,
-		     char *fake_class, char *name, char *class)
+		     char *fake_class, char *name, char *class_)
 {
   char *stack [100][2];
   Widget this;
@@ -1239,10 +1272,10 @@ construct_name_list (Display *display, Widget widget, char *fake_name,
 				&stack [count][1]);
 
   name [0] = 0;
-  class [0] = 0;
+  class_ [0] = 0;
 
   name_tail  = name;
-  class_tail = class;
+  class_tail = class_;
   for (; count >= 0; count--)
     {
       strcat (name_tail,  stack [count][0]);
@@ -1261,24 +1294,8 @@ construct_name_list (Display *display, Widget widget, char *fake_name,
 
 #endif /* 0 */
 
-/* strcasecmp() is not sufficiently portable or standard,
-   and it's easier just to write our own. */
-static int
-ascii_strcasecmp (const char *s1, const char *s2)
-{
-  while (1)
-    {
-      char c1 = *s1++;
-      char c2 = *s2++;
-      if (c1 >= 'A' && c1 <= 'Z') c1 += 'a' - 'A';
-      if (c2 >= 'A' && c2 <= 'Z') c2 += 'a' - 'A';
-      if (c1 != c2) return c1 - c2;
-      if (c1 == '\0') return 0;
-    }
-}
-
-static char_dynarr *name_char_dynarr;
-static char_dynarr *class_char_dynarr;
+static Extbyte_dynarr *name_Extbyte_dynarr;
+static Extbyte_dynarr *class_Extbyte_dynarr;
 
 /* Given a locale and device specification from x-get-resource or
 x-get-resource-prefix, return the resource prefix and display to
@@ -1286,19 +1303,20 @@ fetch the resource on. */
 
 static void
 x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
-		       Display **display_out, char_dynarr *name,
-		       char_dynarr *class)
+		       Display **display_out, Extbyte_dynarr *name,
+		       Extbyte_dynarr *class_)
 {
   if (NILP (locale))
     locale = Qglobal;
   if (NILP (Fvalid_specifier_locale_p (locale)))
-    signal_simple_error ("Invalid locale", locale);
+    invalid_argument ("Invalid locale", locale);
   if (WINDOWP (locale))
     /* #### I can't come up with any coherent way of naming windows.
        By relative position?  That seems tricky because windows
        can change position, be split, etc.  By order of creation?
        That seems less than useful. */
-    signal_simple_error ("Windows currently can't be resourced", locale);
+    signal_error (Qunimplemented,
+		  "Windows currently can't be resourced", locale);
 
   if (!NILP (device) && !DEVICEP (device))
     CHECK_DEVICE (device);
@@ -1310,7 +1328,7 @@ x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
       if (DEVICEP (device) && !DEVICE_X_P (XDEVICE (device)))
 	device = Qnil;
       if (NILP (device))
-	device = Vdefault_x_device;
+	device = get_default_device (Qx);
       if (NILP (device))
 	{
 	  *display_out = 0;
@@ -1321,15 +1339,15 @@ x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
   *display_out = DEVICE_X_DISPLAY (XDEVICE (device));
 
   {
-    char *appname, *appclass;
+    Extbyte *appname, *appclass;
     int name_len, class_len;
     XtGetApplicationNameAndClass (*display_out, &appname, &appclass);
     name_len  = strlen (appname);
     class_len = strlen (appclass);
-    Dynarr_add_many (name , appname,  name_len);
-    Dynarr_add_many (class, appclass, class_len);
+    Dynarr_add_many (name,  appname,  name_len);
+    Dynarr_add_many (class_, appclass, class_len);
     validify_resource_component (Dynarr_atp (name,  0), name_len);
-    validify_resource_component (Dynarr_atp (class, 0), class_len);
+    validify_resource_component (Dynarr_atp (class_, 0), class_len);
   }
 
   if (EQ (locale, Qglobal))
@@ -1339,14 +1357,14 @@ x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
       Dynarr_add_literal_string (name, ".buffer.");
       /* we know buffer is live; otherwise we got an error above. */
       Dynarr_add_validified_lisp_string (name, Fbuffer_name (locale));
-      Dynarr_add_literal_string (class, ".EmacsLocaleType.EmacsBuffer");
+      Dynarr_add_literal_string (class_, ".EmacsLocaleType.EmacsBuffer");
     }
   else if (FRAMEP (locale))
     {
       Dynarr_add_literal_string (name, ".frame.");
       /* we know frame is live; otherwise we got an error above. */
       Dynarr_add_validified_lisp_string (name, Fframe_name (locale));
-      Dynarr_add_literal_string (class, ".EmacsLocaleType.EmacsFrame");
+      Dynarr_add_literal_string (class_, ".EmacsLocaleType.EmacsFrame");
     }
   else
     {
@@ -1354,7 +1372,7 @@ x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
       Dynarr_add_literal_string (name, ".device.");
       /* we know device is live; otherwise we got an error above. */
       Dynarr_add_validified_lisp_string (name, Fdevice_name (locale));
-      Dynarr_add_literal_string (class, ".EmacsLocaleType.EmacsDevice");
+      Dynarr_add_literal_string (class_, ".EmacsLocaleType.EmacsDevice");
     }
   return;
 }
@@ -1441,37 +1459,38 @@ found.  If the third arg is `string', a string is returned, and if it is
 returned value is the list (t) for true, (nil) for false, and is nil to
 mean ``unspecified''.
 */
-       (name, class, type, locale, device, noerror))
+       (name, class_, type, locale, device, noerror))
 {
-  char* name_string, *class_string;
-  char *raw_result;
+  Extbyte *name_string, *class_string;
+  Extbyte *raw_result;
   XrmDatabase db;
   Display *display;
-  Error_behavior errb = decode_error_behavior_flag (noerror);
+  Error_Behavior errb = decode_error_behavior_flag (noerror);
+  Lisp_Object codesys;
 
   CHECK_STRING (name);
-  CHECK_STRING (class);
+  CHECK_STRING (class_);
   CHECK_SYMBOL (type);
 
-  Dynarr_reset (name_char_dynarr);
-  Dynarr_reset (class_char_dynarr);
+  Dynarr_reset (name_Extbyte_dynarr);
+  Dynarr_reset (class_Extbyte_dynarr);
 
   x_get_resource_prefix (locale, device, &display,
-			 name_char_dynarr, class_char_dynarr);
+			 name_Extbyte_dynarr, class_Extbyte_dynarr);
   if (!display)
     return Qnil;
 
   db = XtDatabase (display);
+  codesys = coding_system_of_xrm_database (db);
+  Dynarr_add (name_Extbyte_dynarr, '.');
+  Dynarr_add_lisp_string (name_Extbyte_dynarr, name, Qbinary);
+  Dynarr_add (class_Extbyte_dynarr, '.');
+  Dynarr_add_lisp_string (class_Extbyte_dynarr, class_, Qbinary);
+  Dynarr_add (name_Extbyte_dynarr,  '\0');
+  Dynarr_add (class_Extbyte_dynarr, '\0');
 
-  Dynarr_add (name_char_dynarr, '.');
-  Dynarr_add_lisp_string (name_char_dynarr, name);
-  Dynarr_add (class_char_dynarr, '.');
-  Dynarr_add_lisp_string (class_char_dynarr, class);
-  Dynarr_add (name_char_dynarr,  '\0');
-  Dynarr_add (class_char_dynarr, '\0');
-
-  name_string  = Dynarr_atp (name_char_dynarr,  0);
-  class_string = Dynarr_atp (class_char_dynarr, 0);
+  name_string  = Dynarr_atp (name_Extbyte_dynarr,  0);
+  class_string = Dynarr_atp (class_Extbyte_dynarr, 0);
 
   {
     XrmValue xrm_value;
@@ -1490,54 +1509,61 @@ mean ``unspecified''.
     while (namerest[0] && classrest[0])
       namerest++, classrest++;
     if (namerest[0] || classrest[0])
-      signal_simple_error_2
-	("class list and name list must be the same length", name, class);
+      {
+	maybe_signal_error_2
+	  (Qstructure_formation_error,
+	   "class list and name list must be the same length", name, class_,
+	   Qresource, errb);
+	return Qnil;
+      }
     result = XrmQGetResource (db, namelist, classlist, &xrm_type, &xrm_value);
 
     if (result != True || xrm_type != string_quark)
       return Qnil;
-    raw_result = (char *) xrm_value.addr;
+    raw_result = (Extbyte *) xrm_value.addr;
   }
 
   if (EQ (type, Qstring))
-    return build_string (raw_result);
+    return build_ext_string (raw_result, codesys);
   else if (EQ (type, Qboolean))
     {
-      if (!ascii_strcasecmp (raw_result, "off")   ||
-	  !ascii_strcasecmp (raw_result, "false") ||
-	  !ascii_strcasecmp (raw_result, "no"))
+      if (!strcasecmp (raw_result, "off")   ||
+	  !strcasecmp (raw_result, "false") ||
+	  !strcasecmp (raw_result, "no"))
 	return Fcons (Qnil, Qnil);
-      if (!ascii_strcasecmp (raw_result, "on")   ||
-	  !ascii_strcasecmp (raw_result, "true") ||
-	  !ascii_strcasecmp (raw_result, "yes"))
+      if (!strcasecmp (raw_result, "on")   ||
+	  !strcasecmp (raw_result, "true") ||
+	  !strcasecmp (raw_result, "yes"))
 	return Fcons (Qt, Qnil);
-      return maybe_continuable_error
-	(Qresource, errb,
-	 "can't convert %s: %s to a Boolean", name_string, raw_result);
+      return maybe_signal_continuable_error_2
+	(Qinvalid_operation, "Can't convert to a Boolean",
+	 build_ext_string (name_string, Qbinary),
+	 build_ext_string (raw_result, codesys), Qresource,
+	 errb);
     }
   else if (EQ (type, Qinteger) || EQ (type, Qnatnum))
     {
       int i;
       char c;
       if (1 != sscanf (raw_result, "%d%c", &i, &c))
-	return maybe_continuable_error
-	  (Qresource, errb,
-	   "can't convert %s: %s to an integer", name_string, raw_result);
+      return maybe_signal_continuable_error_2
+	(Qinvalid_operation, "Can't convert to an integer",
+	 build_ext_string (name_string, Qbinary),
+	 build_ext_string (raw_result, codesys), Qresource,
+	 errb);
       else if (EQ (type, Qnatnum) && i < 0)
-	return maybe_continuable_error
-	  (Qresource, errb,
-	   "invalid numerical value %d for resource %s", i, name_string);
+	return maybe_signal_continuable_error_2
+	  (Qinvalid_argument, "Invalid numerical value for resource",
+	   make_int (i), build_ext_string (name_string, Qbinary),
+	   Qresource, errb);
       else
 	return make_int (i);
     }
   else
     {
       return maybe_signal_continuable_error
-	(Qwrong_type_argument,
-	 list2 (build_translated_string
-		("should be string, integer, natnum or boolean"),
-		type),
-	 Qresource, errb);
+	(Qwrong_type_argument, "Should be string, integer, natnum or boolean",
+	 type, Qresource, errb);
     }
 }
 
@@ -1555,18 +1581,18 @@ returns nil. (In such a case, `x-get-resource' would always return nil.)
 {
   Display *display;
 
-  Dynarr_reset (name_char_dynarr );
-  Dynarr_reset (class_char_dynarr);
+  Dynarr_reset (name_Extbyte_dynarr );
+  Dynarr_reset (class_Extbyte_dynarr);
 
   x_get_resource_prefix (locale, device, &display,
-			 name_char_dynarr, class_char_dynarr);
+			 name_Extbyte_dynarr, class_Extbyte_dynarr);
   if (!display)
     return Qnil;
 
-  return Fcons (make_string ((Bufbyte *) Dynarr_atp (name_char_dynarr, 0),
-			     Dynarr_length (name_char_dynarr)),
-		make_string ((Bufbyte *) Dynarr_atp (class_char_dynarr, 0),
-			     Dynarr_length (class_char_dynarr)));
+  return Fcons (make_string ((Ibyte *) Dynarr_atp (name_Extbyte_dynarr, 0),
+			     Dynarr_length (name_Extbyte_dynarr)),
+		make_string ((Ibyte *) Dynarr_atp (class_Extbyte_dynarr, 0),
+			     Dynarr_length (class_Extbyte_dynarr)));
 }
 
 DEFUN ("x-put-resource", Fx_put_resource, 1, 2, 0, /*
@@ -1577,23 +1603,26 @@ standard resource specification.
        (resource_line, device))
 {
   struct device *d = decode_device (device);
-  char *str, *colon_pos;
-
-  CHECK_STRING (resource_line);
-  str = (char *) XSTRING_DATA (resource_line);
-  if (!(colon_pos = strchr (str, ':')) || strchr (str, '\n'))
-  invalid:
-    signal_simple_error ("Invalid resource line", resource_line);
-  if (strspn (str,
-	      /* Only the following chars are allowed before the colon */
-	      " \t.*?abcdefghijklmnopqrstuvwxyz"
-	      "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-")
-      != (size_t) (colon_pos - str))
-    goto invalid;
 
   if (DEVICE_X_P (d))
     {
       XrmDatabase db = XtDatabase (DEVICE_X_DISPLAY (d));
+      Extbyte *str, *colon_pos;
+
+      CHECK_STRING (resource_line);
+      LISP_STRING_TO_EXTERNAL (resource_line, str,
+			       coding_system_of_xrm_database (db));
+      if (!(colon_pos = strchr (str, ':')) || strchr (str, '\n'))
+      invalid:
+	syntax_error ("Invalid resource line", resource_line);
+      if ((int)
+	  strspn (str,
+		  /* Only the following chars are allowed before the colon */
+		  " \t.*?abcdefghijklmnopqrstuvwxyz"
+		  "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-")
+	  != colon_pos - str)
+	goto invalid;
+
       XrmPutLineResource (&db, str);
     }
 
@@ -1608,10 +1637,11 @@ standard resource specification.
 DEFUN ("default-x-device", Fdefault_x_device, 0, 0, 0, /*
 Return the default X device for resourcing.
 This is the first-created X device that still exists.
+See also `default-device'.
 */
        ())
 {
-  return Vdefault_x_device;
+  return get_default_device (Qx);
 }
 
 DEFUN ("x-display-visual-class", Fx_display_visual_class, 0, 1, 0, /*
@@ -1623,7 +1653,7 @@ The returned value will be one of the symbols `static-gray', `gray-scale',
        (device))
 {
   Visual *vis = DEVICE_X_VISUAL (decode_x_device (device));
-  switch (vis->class)
+  switch (vis->X_CLASSFIELD)
     {
     case StaticGray:  return intern ("static-gray");
     case GrayScale:   return intern ("gray-scale");
@@ -1632,7 +1662,7 @@ The returned value will be one of the symbols `static-gray', `gray-scale',
     case TrueColor:   return intern ("true-color");
     case DirectColor: return intern ("direct-color");
     default:
-      error ("display has an unknown visual class");
+      invalid_state ("display has an unknown visual class", Qunbound);
       return Qnil;	/* suppress compiler warning */
     }
 }
@@ -1718,7 +1748,7 @@ DEVICE must be an X11 display device.  See `x-keysym-on-keyboard-p'.
 {
   struct device *d = decode_device (device);
   if (!DEVICE_X_P (d))
-    signal_simple_error ("Not an X device", device);
+    gui_error ("Not an X device", device);
 
   return DEVICE_X_DATA (d)->x_keysym_map_hash_table;
 }
@@ -1739,7 +1769,7 @@ The two names differ in capitalization and underscoring.
 {
   struct device *d = decode_device (device);
   if (!DEVICE_X_P (d))
-    signal_simple_error ("Not an X device", device);
+    gui_error ("Not an X device", device);
 
   return (EQ (Qsans_modifiers,
 	      Fgethash (keysym, DEVICE_X_KEYSYM_MAP_HASH_TABLE (d), Qnil)) ?
@@ -1762,7 +1792,7 @@ The two names differ in capitalization and underscoring.
 {
   struct device *d = decode_device (device);
   if (!DEVICE_X_P (d))
-    signal_simple_error ("Not an X device", device);
+    gui_error ("Not an X device", device);
 
   return (NILP (Fgethash (keysym, DEVICE_X_KEYSYM_MAP_HASH_TABLE (d), Qnil)) ?
 	  Qnil : Qt);
@@ -1910,7 +1940,7 @@ See also `x-set-font-path'.
   Lisp_Object font_path = Qnil;
 
   if (!directories)
-    signal_simple_error ("Can't get X font path", device);
+    gui_error ("Can't get X font path", device);
 
   while (ndirs_return--)
       font_path = Fcons (build_ext_string (directories[ndirs_return],
@@ -1992,22 +2022,26 @@ syms_of_device_x (void)
   DEFSUBR (Fx_get_font_path);
   DEFSUBR (Fx_set_font_path);
 
-  defsymbol (&Qx_error, "x-error");
-  defsymbol (&Qinit_pre_x_win, "init-pre-x-win");
-  defsymbol (&Qinit_post_x_win, "init-post-x-win");
+  DEFSYMBOL (Qx_error);
+  DEFSYMBOL (Qinit_pre_x_win);
+  DEFSYMBOL (Qinit_post_x_win);
+
+#ifdef MULE
+  DEFSYMBOL (Qget_coding_system_from_locale);
+#endif
 }
 
 void
 reinit_console_type_create_device_x (void)
 {
   /* Initialize variables to speed up X resource interactions */
-  const char *valid_resource_chars =
+  const Char_ASCII *valid_resource_chars =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
   while (*valid_resource_chars)
     valid_resource_char_p[(unsigned int) (*valid_resource_chars++)] = 1;
 
-  name_char_dynarr  = Dynarr_new (char);
-  class_char_dynarr = Dynarr_new (char);
+  name_Extbyte_dynarr  = Dynarr_new (Extbyte);
+  class_Extbyte_dynarr = Dynarr_new (Extbyte);
 }
 
 void
@@ -2065,7 +2099,7 @@ just reside in C.
 */ );
   Vx_initial_argv_list = Qnil;
 
-#if defined(MULE) && (defined(LWLIB_MENUBARS_MOTIF) || defined(HAVE_XIM) || defined (USE_XFONTSET))
+#ifdef MULE
   DEFVAR_LISP ("x-app-defaults-directory", &Vx_app_defaults_directory /*
 Used by the Lisp code to communicate to the low level X initialization
 where the localized init files are.
@@ -2074,7 +2108,4 @@ where the localized init files are.
 #endif
 
   Fprovide (Qx);
-
-  staticpro (&Vdefault_x_device);
-  Vdefault_x_device = Qnil;
 }
