@@ -1,7 +1,7 @@
 /* Implements elisp-programmable dialog boxes -- X interface.
    Copyright (C) 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
-   Copyright (C) 2000 Ben Wing.
+   Copyright (C) 2000, 2002, 2003 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -27,18 +27,17 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "console-x.h"
-#include "EmacsFrame.h"
-#include "gui-x.h"
-
 #include "buffer.h"
 #include "commands.h"           /* zmacs_regions */
 #include "events.h"
-#include "frame.h"
+#include "frame-impl.h"
 #include "gui.h"
 #include "opaque.h"
 #include "window.h"
 
+#include "console-x-impl.h"
+
+#include "EmacsFrame.h"
 
 static void
 maybe_run_dbox_text_callback (LWLIB_ID id)
@@ -52,7 +51,7 @@ maybe_run_dbox_text_callback (LWLIB_ID id)
     {
       Lisp_Object text_field_callback;
       Extbyte *text_field_value = wv->value;
-      VOID_TO_LISP (text_field_callback, wv->call_data);
+      text_field_callback = VOID_TO_LISP (wv->call_data);
       text_field_callback = XCAR (XCDR (text_field_callback));
       if (text_field_value)
 	{
@@ -105,7 +104,7 @@ dbox_selection_callback (Widget widget, LWLIB_ID id, XtPointer client_data)
      ourselves. */
 #ifdef EXTERNAL_WIDGET
   /* #### Not sure if this special case is necessary. */
-  if (f && !FRAME_X_EXTERNAL_WINDOW_P (f))
+  if (!FRAME_X_EXTERNAL_WINDOW_P (f) && f)
 #else
   if (f)
 #endif
@@ -126,7 +125,7 @@ dbox_descriptor_to_widget_value (Lisp_Object keys)
   int allow_text_p = 1;
   widget_value *prev = 0, *kids = 0;
   int n = 0;
-  int count = specpdl_depth ();
+  int count;
   Lisp_Object wv_closure, gui_item;
   Lisp_Object question = Qnil;
   Lisp_Object title    = Qnil;  /* #### currently unused */
@@ -151,20 +150,18 @@ dbox_descriptor_to_widget_value (Lisp_Object keys)
 	    buttons = value;
 	  }
 	else
-	  syntax_error ("Unrecognized question-dialog keyword", key);
+	  invalid_constant ("Unrecognized question-dialog keyword", key);
       }
   }
 
   if (NILP (question))
-    syntax_error ("Dialog descriptor provides no question", keys);
+    sferror ("Dialog descriptor provides no question", keys);
 
   /* Inhibit GC during this conversion.  The reasons for this are
      the same as in menu_item_descriptor_to_widget_value(); see
      the large comment above that function. */
 
-  record_unwind_protect (restore_gc_inhibit,
-			 make_int (gc_currently_forbidden));
-  gc_currently_forbidden = 1;
+  count = begin_gc_forbidden ();
 
   kids = prev = xmalloc_widget_value ();
 
@@ -185,7 +182,7 @@ dbox_descriptor_to_widget_value (Lisp_Object keys)
 	if (NILP (button))
 	  {
 	    if (partition_seen)
-	      syntax_error ("More than one partition (nil) seen in dbox spec",
+	      sferror ("More than one partition (nil) seen in dbox spec",
 			    keys);
 	    partition_seen = 1;
 	    continue;
@@ -221,7 +218,7 @@ dbox_descriptor_to_widget_value (Lisp_Object keys)
 	    n++;
 
 	    if (lbuttons > 9 || rbuttons > 9)
-	      syntax_error ("Too many buttons (9)",
+	      sferror ("Too many buttons (9)",
 			    keys); /* #### this leaks */
 	  }
 
@@ -231,7 +228,7 @@ dbox_descriptor_to_widget_value (Lisp_Object keys)
   }
 
   if (n == 0)
-    syntax_error ("Dialog boxes must have some buttons", keys);
+    sferror ("Dialog boxes must have some buttons", keys);
 
   {
     Extbyte type = (text_field_p ? 'P' : 'Q');
@@ -245,7 +242,7 @@ dbox_descriptor_to_widget_value (Lisp_Object keys)
 
     /* No more need to free the half-filled-in structures. */
     set_opaque_ptr (wv_closure, 0);
-    unbind_to (count, Qnil);
+    unbind_to (count);
     return dbox;
   }
 }
@@ -259,7 +256,7 @@ x_make_dialog_box_internal (struct frame* f, Lisp_Object type,
   Widget parent, dbox;
 
   if (!EQ (type, Qquestion))
-    signal_type_error (Qunimplemented, "Dialog box type", type);
+    signal_error (Qunimplemented, "Dialog box type", type);
 
   data = dbox_descriptor_to_widget_value (keys);
 
