@@ -48,11 +48,6 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 #include "syssignal.h"
-
-#ifdef LISP_FLOAT_TYPE
-
-/* Need to define a differentiating symbol -- see sysfloat.h */
-#define THIS_FILENAME floatfns
 #include "sysfloat.h"
 
 /* The code uses emacs_rint, so that it works to undefine HAVE_RINT
@@ -112,15 +107,15 @@ static const char *float_error_fn_name;
 
 
 #define arith_error(op,arg) \
-  Fsignal (Qarith_error, list2 (build_string (op), arg))
+  Fsignal (Qarith_error, list2 (build_msg_string (op), arg))
 #define range_error(op,arg) \
-  Fsignal (Qrange_error, list2 (build_string (op), arg))
+  Fsignal (Qrange_error, list2 (build_msg_string (op), arg))
 #define range_error2(op,a1,a2) \
-  Fsignal (Qrange_error, list3 (build_string (op), a1, a2))
+  Fsignal (Qrange_error, list3 (build_msg_string (op), a1, a2))
 #define domain_error(op,arg) \
-  Fsignal (Qdomain_error, list2 (build_string (op), arg))
+  Fsignal (Qdomain_error, list2 (build_msg_string (op), arg))
 #define domain_error2(op,a1,a2) \
-  Fsignal (Qdomain_error, list3 (build_string (op), a1, a2))
+  Fsignal (Qdomain_error, list3 (build_msg_string (op), a1, a2))
 
 
 /* Convert float to Lisp Integer if it fits, else signal a range
@@ -128,16 +123,15 @@ static const char *float_error_fn_name;
 static Lisp_Object
 float_to_int (double x, const char *name, Lisp_Object num, Lisp_Object num2)
 {
-  REGISTER EMACS_INT result = (EMACS_INT) x;
-
-  if (result > EMACS_INT_MAX || result < EMACS_INT_MIN)
-    {
-      if (!UNBOUNDP (num2))
-	range_error2 (name, num, num2);
-      else
-	range_error (name, num);
-    }
-  return make_int (result);
+  if (x >= ((EMACS_INT) 1 << (VALBITS-1))
+      || x <= - ((EMACS_INT) 1 << (VALBITS-1)) - (EMACS_INT) 1)
+  {
+    if (!UNBOUNDP (num2))
+      range_error2 (name, num, num2);
+    else
+      range_error (name, num);
+  }
+  return (make_int ((EMACS_INT) x));
 }
 
 
@@ -176,7 +170,7 @@ float_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
   return (extract_float (obj1) == extract_float (obj2));
 }
 
-static unsigned long
+static Hashcode
 float_hash (Lisp_Object obj, int depth)
 {
   /* mod the value down to 32-bit range */
@@ -184,11 +178,12 @@ float_hash (Lisp_Object obj, int depth)
   return (unsigned long) fmod (extract_float (obj), 4e9);
 }
 
-static const struct lrecord_description float_description[] = {
+static const struct memory_description float_description[] = {
   { XD_END }
 };
 
 DEFINE_BASIC_LRECORD_IMPLEMENTATION ("float", float,
+				     1, /*dumpable-flag*/
 				     mark_float, print_float, 0, float_equal,
 				     float_hash, float_description,
 				     Lisp_Float);
@@ -206,11 +201,8 @@ extract_float (Lisp_Object num)
 
   return extract_float (wrong_type_argument (Qnumberp, num));
 }
-#endif /* LISP_FLOAT_TYPE */
-
 
 /* Trig functions.  */
-#ifdef LISP_FLOAT_TYPE
 
 DEFUN ("acos", Facos, 1, 1, 0, /*
 Return the inverse cosine of NUMBER.
@@ -297,12 +289,9 @@ Return the tangent of NUMBER.
   IN_FLOAT (d = (sin (d) / c), "tan", number);
   return make_float (d);
 }
-#endif /* LISP_FLOAT_TYPE (trig functions) */
-
 
 /* Bessel functions */
 #if 0 /* Leave these out unless we find there's a reason for them.  */
-/* #ifdef LISP_FLOAT_TYPE */
 
 DEFUN ("bessel-j0", Fbessel_j0, 1, 1, 0, /*
 Return the bessel function j0 of NUMBER.
@@ -374,7 +363,6 @@ The first number (the order) is truncated to an integer.
 
 /* Error functions. */
 #if 0 /* Leave these out unless we see they are worth having.  */
-/* #ifdef LISP_FLOAT_TYPE */
 
 DEFUN ("erf", Ferf, 1, 1, 0, /*
 Return the mathematical error function of NUMBER.
@@ -411,7 +399,6 @@ Return the log gamma of NUMBER.
 
 /* Root and Log functions. */
 
-#ifdef LISP_FLOAT_TYPE
 DEFUN ("exp", Fexp, 1, 1, 0, /*
 Return the exponential base e of NUMBER.
 */
@@ -428,8 +415,6 @@ Return the exponential base e of NUMBER.
     IN_FLOAT (d = exp (d), "exp", number);
   return make_float (d);
 }
-#endif /* LISP_FLOAT_TYPE */
-
 
 DEFUN ("expt", Fexpt, 2, 2, 0, /*
 Return the exponential NUMBER1 ** NUMBER2.
@@ -466,7 +451,6 @@ Return the exponential NUMBER1 ** NUMBER2.
       return make_int (retval);
     }
 
-#ifdef LISP_FLOAT_TYPE
   {
     double f1 = extract_float (number1);
     double f2 = extract_float (number2);
@@ -480,14 +464,8 @@ Return the exponential NUMBER1 ** NUMBER2.
     IN_FLOAT2 (f1 = pow (f1, f2), "expt", number1, number2);
     return make_float (f1);
   }
-#else
-  CHECK_INT_OR_FLOAT (number1);
-  CHECK_INT_OR_FLOAT (number2);
-  return Fexpt (number1, number2);
-#endif /* LISP_FLOAT_TYPE */
 }
 
-#ifdef LISP_FLOAT_TYPE
 DEFUN ("log", Flog, 1, 2, 0, /*
 Return the natural logarithm of NUMBER.
 If second optional argument BASE is given, return the logarithm of
@@ -564,12 +542,8 @@ Return the cube root of NUMBER.
 #endif
   return make_float (d);
 }
-#endif /* LISP_FLOAT_TYPE */
-
 
 /* Inverse trig functions. */
-#ifdef LISP_FLOAT_TYPE
-/* #if 0  Not clearly worth adding...  */
 
 DEFUN ("acosh", Facosh, 1, 1, 0, /*
 Return the inverse hyperbolic cosine of NUMBER.
@@ -658,7 +632,6 @@ Return the hyperbolic tangent of NUMBER.
   IN_FLOAT (d = tanh (d), "tanh", number);
   return make_float (d);
 }
-#endif /* LISP_FLOAT_TYPE (inverse trig functions) */
 
 /* Rounding functions */
 
@@ -667,14 +640,12 @@ Return the absolute value of NUMBER.
 */
        (number))
 {
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP (number))
     {
       IN_FLOAT (number = make_float (fabs (XFLOAT_DATA (number))),
 		"abs", number);
       return number;
     }
-#endif /* LISP_FLOAT_TYPE */
 
   if (INTP (number))
     return (XINT (number) >= 0) ? number : make_int (- XINT (number));
@@ -682,7 +653,6 @@ Return the absolute value of NUMBER.
   return Fabs (wrong_type_argument (Qnumberp, number));
 }
 
-#ifdef LISP_FLOAT_TYPE
 DEFUN ("float", Ffloat, 1, 1, 0, /*
 Return the floating point number numerically equal to NUMBER.
 */
@@ -696,10 +666,7 @@ Return the floating point number numerically equal to NUMBER.
 
   return Ffloat (wrong_type_argument (Qnumberp, number));
 }
-#endif /* LISP_FLOAT_TYPE */
 
-
-#ifdef LISP_FLOAT_TYPE
 DEFUN ("logb", Flogb, 1, 1, 0, /*
 Return largest integer <= the base 2 log of the magnitude of NUMBER.
 This is the same as the exponent of a float.
@@ -709,7 +676,7 @@ This is the same as the exponent of a float.
   double f = extract_float (number);
 
   if (f == 0.0)
-    return make_int (EMACS_INT_MIN);
+    return make_int (- (EMACS_INT)(((EMACS_UINT) 1) << (VALBITS - 1))); /* most-negative-fixnum */
 #ifdef HAVE_LOGB
   {
     Lisp_Object val;
@@ -750,22 +717,18 @@ This is the same as the exponent of a float.
 #endif /* ! HAVE_FREXP */
 #endif /* ! HAVE_LOGB */
 }
-#endif /* LISP_FLOAT_TYPE */
-
 
 DEFUN ("ceiling", Fceiling, 1, 1, 0, /*
 Return the smallest integer no less than NUMBER.  (Round toward +inf.)
 */
        (number))
 {
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP (number))
     {
       double d;
       IN_FLOAT ((d = ceil (XFLOAT_DATA (number))), "ceiling", number);
       return (float_to_int (d, "ceiling", number, Qunbound));
     }
-#endif /* LISP_FLOAT_TYPE */
 
   if (INTP (number))
     return number;
@@ -789,7 +752,6 @@ greater than NUMBER/DIVISOR.
 
       CHECK_INT_OR_FLOAT (divisor);
 
-#ifdef LISP_FLOAT_TYPE
       if (FLOATP (number) || FLOATP (divisor))
 	{
 	  double f1 = extract_float (number);
@@ -801,7 +763,6 @@ greater than NUMBER/DIVISOR.
 	  IN_FLOAT2 (f1 = floor (f1 / f2), "floor", number, divisor);
 	  return float_to_int (f1, "floor", number, divisor);
 	}
-#endif /* LISP_FLOAT_TYPE */
 
       i1 = XINT (number);
       i2 = XINT (divisor);
@@ -818,14 +779,12 @@ greater than NUMBER/DIVISOR.
       return (make_int (i1));
     }
 
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP (number))
     {
       double d;
       IN_FLOAT ((d = floor (XFLOAT_DATA (number))), "floor", number);
       return (float_to_int (d, "floor", number, Qunbound));
     }
-#endif /* LISP_FLOAT_TYPE */
 
   return number;
 }
@@ -835,7 +794,6 @@ Return the nearest integer to NUMBER.
 */
        (number))
 {
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP (number))
     {
       double d;
@@ -843,7 +801,6 @@ Return the nearest integer to NUMBER.
       IN_FLOAT ((d = emacs_rint (XFLOAT_DATA (number))), "round", number);
       return (float_to_int (d, "round", number, Qunbound));
     }
-#endif /* LISP_FLOAT_TYPE */
 
   if (INTP (number))
     return number;
@@ -857,10 +814,8 @@ Rounds the value toward zero.
 */
        (number))
 {
-#ifdef LISP_FLOAT_TYPE
   if (FLOATP (number))
     return float_to_int (XFLOAT_DATA (number), "truncate", number, Qunbound);
-#endif /* LISP_FLOAT_TYPE */
 
   if (INTP (number))
     return number;
@@ -869,8 +824,6 @@ Rounds the value toward zero.
 }
 
 /* Float-rounding functions. */
-#ifdef LISP_FLOAT_TYPE
-/* #if 1  It's not clear these are worth adding... */
 
 DEFUN ("fceiling", Ffceiling, 1, 1, 0, /*
 Return the smallest integer no less than NUMBER, as a float.
@@ -917,11 +870,7 @@ Rounds the value toward zero.
     IN_FLOAT (d = ceil (d), "ftruncate", number);
   return make_float (d);
 }
-
-#endif /* LISP_FLOAT_TYPE (float-rounding functions) */
-
 
-#ifdef LISP_FLOAT_TYPE
 #ifdef FLOAT_CATCH_SIGILL
 static SIGTYPE
 float_error (int signo)
@@ -938,7 +887,7 @@ float_error (int signo)
      occurring inside a signal handler to be restartable, considering
      that anything could happen when the error is signaled and trapped
      and considering the asynchronous nature of signal handlers. */
-  signal_error (Qarith_error, list1 (float_error_arg));
+  signal_error (Qarith_error, 0, float_error_arg);
 }
 
 /* Another idea was to replace the library function `infnan'
@@ -976,18 +925,14 @@ matherr (struct exception *x)
   return 1;	/* don't set errno or print a message */
 }
 #endif /* HAVE_MATHERR */
-#endif /* LISP_FLOAT_TYPE */
-
 
 void
 init_floatfns_very_early (void)
 {
-#ifdef LISP_FLOAT_TYPE
 # ifdef FLOAT_CATCH_SIGILL
-  signal (SIGILL, float_error);
+  EMACS_SIGNAL (SIGILL, float_error);
 # endif
   in_float = 0;
-#endif /* LISP_FLOAT_TYPE */
 }
 
 void
@@ -997,14 +942,12 @@ syms_of_floatfns (void)
 
   /* Trig functions.  */
 
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Facos);
   DEFSUBR (Fasin);
   DEFSUBR (Fatan);
   DEFSUBR (Fcos);
   DEFSUBR (Fsin);
   DEFSUBR (Ftan);
-#endif /* LISP_FLOAT_TYPE */
 
   /* Bessel functions */
 
@@ -1027,35 +970,27 @@ syms_of_floatfns (void)
 
   /* Root and Log functions. */
 
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Fexp);
-#endif /* LISP_FLOAT_TYPE */
   DEFSUBR (Fexpt);
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Flog);
   DEFSUBR (Flog10);
   DEFSUBR (Fsqrt);
   DEFSUBR (Fcube_root);
-#endif /* LISP_FLOAT_TYPE */
 
   /* Inverse trig functions. */
 
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Facosh);
   DEFSUBR (Fasinh);
   DEFSUBR (Fatanh);
   DEFSUBR (Fcosh);
   DEFSUBR (Fsinh);
   DEFSUBR (Ftanh);
-#endif /* LISP_FLOAT_TYPE */
 
   /* Rounding functions */
 
   DEFSUBR (Fabs);
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Ffloat);
   DEFSUBR (Flogb);
-#endif /* LISP_FLOAT_TYPE */
   DEFSUBR (Fceiling);
   DEFSUBR (Ffloor);
   DEFSUBR (Fround);
@@ -1063,18 +998,14 @@ syms_of_floatfns (void)
 
   /* Float-rounding functions. */
 
-#ifdef LISP_FLOAT_TYPE
   DEFSUBR (Ffceiling);
   DEFSUBR (Fffloor);
   DEFSUBR (Ffround);
   DEFSUBR (Fftruncate);
-#endif /* LISP_FLOAT_TYPE */
 }
 
 void
 vars_of_floatfns (void)
 {
-#ifdef LISP_FLOAT_TYPE
   Fprovide (intern ("lisp-float-type"));
-#endif
 }
