@@ -1,6 +1,6 @@
 /* Functions for the X window system.
    Copyright (C) 1989, 1992-5, 1997 Free Software Foundation, Inc.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -28,7 +28,20 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "console-x.h"
+#include "buffer.h"
+#include "device-impl.h"
+#include "events.h"
+#include "extents.h"
+#include "faces.h"
+#include "frame-impl.h"
+#include "gutter.h"
+#include "window.h"
+
+#include "console-x-impl.h"
+#include "glyphs-x.h"
+#include "objects-x-impl.h"
+#include "scrollbar-x.h"
+
 #include "xintrinsicp.h"	/* CoreP.h needs this */
 #include <X11/CoreP.h>		/* Numerous places access the fields of
 				   a core widget directly.  We could
@@ -42,17 +55,6 @@ Boston, MA 02111-1307, USA.  */
 #ifdef EXTERNAL_WIDGET
 #include "ExternalShell.h"
 #endif
-#include "glyphs-x.h"
-#include "objects-x.h"
-#include "scrollbar-x.h"
-
-#include "buffer.h"
-#include "events.h"
-#include "extents.h"
-#include "faces.h"
-#include "frame.h"
-#include "window.h"
-#include "gutter.h"
 
 #ifdef HAVE_DRAGNDROP
 #include "dragdrop.h"
@@ -61,16 +63,25 @@ Boston, MA 02111-1307, USA.  */
 #ifdef HAVE_OFFIX_DND
 #include "offix.h"
 #endif
-#if defined (HAVE_OFFIX_DND) || defined (HAVE_CDE)
-#include "events-mod.h"
-#endif
 
 /* Default properties to use when creating frames.  */
 Lisp_Object Vdefault_x_frame_plist;
 
-Lisp_Object Qwindow_id;
 Lisp_Object Qx_resource_name;
 
+static const struct memory_description x_frame_data_description_1 [] = {
+  { XD_LISP_OBJECT, offsetof (struct x_frame, last_menubar_buffer) },
+  { XD_LISP_OBJECT, offsetof (struct x_frame, icon_pixmap) },
+  { XD_LISP_OBJECT, offsetof (struct x_frame, icon_pixmap_mask) },
+  { XD_END }
+};
+
+extern const struct sized_memory_description x_frame_data_description;
+
+const struct sized_memory_description x_frame_data_description = {
+  sizeof (struct x_frame), x_frame_data_description_1
+};
+
 EXFUN (Fx_window_id, 1);
 
 
@@ -191,7 +202,7 @@ struct frame *
 decode_x_frame (Lisp_Object frame)
 {
   if (NILP (frame))
-    XSETFRAME (frame, selected_frame ());
+    frame = wrap_frame (selected_frame ());
   CHECK_LIVE_FRAME (frame);
   /* this will also catch dead frames, but putting in the above check
      results in a more useful error */
@@ -363,7 +374,7 @@ x_wm_maybe_store_wm_command (struct frame *f)
       make_argc_argv (Vcommand_line_args, &argc, &argv);
       XSetCommand (XtDisplay (w), XtWindow (w), argv, argc);
       free_argc_argv (argv);
-      XSETFRAME (DEVICE_X_WM_COMMAND_FRAME (d), f);
+      DEVICE_X_WM_COMMAND_FRAME (d) = wrap_frame (f);
     }
 }
 
@@ -399,15 +410,15 @@ x_wm_maybe_move_wm_command (struct frame *f)
 }
 #endif /* !HAVE_WMCOMMAND */
 
-static int
-x_frame_iconified_p (struct frame *f)
+int
+x_frame_window_state (struct frame *f)
 {
   Atom actual_type;
   int actual_format;
   unsigned long nitems, bytesafter;
   unsigned long *datap = 0;
   Widget widget;
-  int result = 0;
+  int result = -1;
   struct device *d = XDEVICE (FRAME_DEVICE (f));
 
   widget = FRAME_X_SHELL_WIDGET (f);
@@ -418,14 +429,20 @@ x_frame_iconified_p (struct frame *f)
 				     (unsigned char **) &datap)
       && datap)
     {
-      if (nitems <= 2	/* "suggested" by ICCCM version 1 */
-	  && datap[0] == IconicState)
-	result = 1;
+      if (nitems <= 2)	/* "suggested" by ICCCM version 1 */
+	result = (int) datap[0];
       XFree ((char *) datap);
     }
+
   return result;
 }
 
+static int
+x_frame_iconified_p (struct frame *f)
+{
+  return x_frame_window_state (f) == IconicState;
+}
+
 
 /************************************************************************/
 /*                          frame properties                            */
@@ -433,7 +450,7 @@ x_frame_iconified_p (struct frame *f)
 
 /* Connect the frame-property names (symbols) to the corresponding
    X Resource Manager names.  The name of a property, as a Lisp symbol,
-   has an `x-resource-name' property which is a Lisp_String. */
+   has an `x-resource-name' property which is a Lisp string. */
 
 static void
 init_x_prop_symbols (void)
@@ -448,14 +465,6 @@ init_x_prop_symbols (void)
   def (Qunsplittable, XtNunsplittable);
 #endif
   defi(Qinternal_border_width, XtNinternalBorderWidth);
-#ifdef HAVE_TOOLBARS
-  def (Qtop_toolbar_shadow_color, XtNtopToolBarShadowColor);
-  def (Qbottom_toolbar_shadow_color, XtNbottomToolBarShadowColor);
-  def (Qbackground_toolbar_color, XtNbackgroundToolBarColor);
-  def (Qtop_toolbar_shadow_pixmap, XtNtopToolBarShadowPixmap);
-  def (Qbottom_toolbar_shadow_pixmap, XtNbottomToolBarShadowPixmap);
-  defi(Qtoolbar_shadow_thickness, XtNtoolBarShadowThickness);
-#endif
   def (Qscrollbar_placement, XtNscrollBarPlacement);
   defi(Qinter_line_space, XtNinterline);
   /* font, foreground */
@@ -489,13 +498,13 @@ init_x_prop_symbols (void)
 static Lisp_Object
 color_to_string (Widget w, unsigned long pixel)
 {
-  char buf[255];
+  Ibyte buf[255];
 
   XColor color;
   color.pixel = pixel;
   XQueryColor (XtDisplay (w), w->core.colormap, &color);
-  sprintf (buf, "#%04x%04x%04x", color.red, color.green, color.blue);
-  return build_string (buf);
+  qxesprintf (buf, "#%04x%04x%04x", color.red, color.green, color.blue);
+  return build_intstring (buf);
 }
 
 static void
@@ -562,20 +571,10 @@ x_frame_property (struct frame *f, Lisp_Object property)
     return make_int (w->emacs_frame.internal_border_width);
   if (EQ (Qborder_color, property))
     return color_to_string (gw, w->core.border_pixel);
-#ifdef HAVE_TOOLBARS
-  if (EQ (Qtop_toolbar_shadow_color, property))
-    return color_to_string (gw, w->emacs_frame.top_toolbar_shadow_pixel);
-  if (EQ (Qbottom_toolbar_shadow_color, property))
-    return color_to_string (gw, w->emacs_frame.bottom_toolbar_shadow_pixel);
-  if (EQ (Qbackground_toolbar_color, property))
-    return color_to_string (gw, w->emacs_frame.background_toolbar_pixel);
-  if (EQ (Qtoolbar_shadow_thickness, property))
-    return make_int (w->emacs_frame.toolbar_shadow_thickness);
-#endif /* HAVE_TOOLBARS */
   if (EQ (Qinter_line_space, property))
     return make_int (w->emacs_frame.interline);
   if (EQ (Qwindow_id, property))
-    return Fx_window_id (make_frame (f));
+    return Fx_window_id (wrap_frame (f));
 
   return Qunbound;
 }
@@ -588,12 +587,6 @@ x_internal_frame_property_p (struct frame *f, Lisp_Object property)
     || EQ (property, Qborder_width)
     || EQ (property, Qinternal_border_width)
     || EQ (property, Qborder_color)
-#ifdef HAVE_TOOLBARS
-    || EQ (property, Qtop_toolbar_shadow_color)
-    || EQ (property, Qbottom_toolbar_shadow_color)
-    || EQ (property, Qbackground_toolbar_color)
-    || EQ (property, Qtoolbar_shadow_thickness)
-#endif /* HAVE_TOOLBARS */
     || EQ (property, Qinter_line_space)
     || EQ (property, Qwindow_id)
     || STRINGP (property);
@@ -608,24 +601,9 @@ x_frame_properties (struct frame *f)
   Widget gw = (Widget) w;
   Position x, y;
 
-  props = cons3 (Qwindow_id, Fx_window_id (make_frame (f)), props);
+  props = cons3 (Qwindow_id, Fx_window_id (wrap_frame (f)), props);
   props = cons3 (Qinter_line_space, make_int (w->emacs_frame.interline), props);
 
-#ifdef HAVE_TOOLBARS
-  props = cons3 (Qtoolbar_shadow_thickness,
-		 make_int (w->emacs_frame.toolbar_shadow_thickness),
-		 props);
-  props = cons3 (Qbackground_toolbar_color,
-		 color_to_string (gw, w->emacs_frame.background_toolbar_pixel),
-		 props);
-  props = cons3 (Qbottom_toolbar_shadow_color,
-		 color_to_string (gw, w->emacs_frame.bottom_toolbar_shadow_pixel),
-		 props);
-  props = cons3 (Qtop_toolbar_shadow_color,
-		 color_to_string (gw, w->emacs_frame.top_toolbar_shadow_pixel),
-		 props);
-#endif /* HAVE_TOOLBARS */
-
   props = cons3 (Qborder_color,
 		 color_to_string (gw, w->core.border_pixel), props);
   props = cons3 (Qinternal_border_width,
@@ -648,7 +626,7 @@ x_frame_properties (struct frame *f)
    individual properties. */
 
 static void
-x_set_frame_text_value (struct frame *f, Bufbyte *value,
+x_set_frame_text_value (struct frame *f, Ibyte *value,
 			String Xt_resource_name,
 			String Xt_resource_encoding_name)
 {
@@ -657,10 +635,10 @@ x_set_frame_text_value (struct frame *f, Bufbyte *value,
   String old_XtValue = NULL;
 
 #ifdef MULE
-  Bufbyte *ptr;
+  Ibyte *ptr;
   /* Optimize for common ASCII case */
   for (ptr = value; *ptr; ptr++)
-    if (!BYTE_ASCII_P (*ptr))
+    if (!byte_ascii_p (*ptr))
       {
         const char * tmp;
         encoding = DEVICE_XATOM_COMPOUND_TEXT (XDEVICE (FRAME_DEVICE (f)));
@@ -682,13 +660,13 @@ x_set_frame_text_value (struct frame *f, Bufbyte *value,
 }
 
 static void
-x_set_title_from_bufbyte (struct frame *f, Bufbyte *name)
+x_set_title_from_ibyte (struct frame *f, Ibyte *name)
 {
   x_set_frame_text_value (f, name, XtNtitle, XtNtitleEncoding);
 }
 
 static void
-x_set_icon_name_from_bufbyte (struct frame *f, Bufbyte *name)
+x_set_icon_name_from_ibyte (struct frame *f, Ibyte *name)
 {
   x_set_frame_text_value (f, name, XtNiconName, XtNiconNameEncoding);
 }
@@ -702,7 +680,7 @@ x_set_icon_name_from_bufbyte (struct frame *f, Bufbyte *name)
 
 static void
 x_set_initial_frame_size (struct frame *f, int flags, int x, int y,
-			  unsigned int w, unsigned int h)
+			  int w, int h)
 {
   char shell_geom [255];
   int xval, yval;
@@ -770,7 +748,7 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
 	  if (STRINGP (val))
 	    {
 	      const Extbyte *extval;
-	      Extcount extvallen;
+	      Bytecount extvallen;
 
 	      TO_EXTERNAL_FORMAT (LISP_STRING, val,
 				  ALLOCA, (extval, extvallen),
@@ -799,7 +777,7 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
 		    {
 		      Lisp_Object frm, font_spec;
 
-		      XSETFRAME (frm, f);
+		      frm = wrap_frame (f);
 		      font_spec = Fget (Fget_face (Qdefault), Qfont, Qnil);
 
 		      Fadd_spec_to_specifier (font_spec, val, frm, Qnil, Qnil);
@@ -930,15 +908,15 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
       {
 	if (size_specified_p || internal_border_width_specified)
 	  {
-	    Lisp_Object frame;
-	    XSETFRAME (frame, f);
+	    Lisp_Object frame = wrap_frame (f);
+
 	    Fset_frame_size (frame, make_int (width),
 			      make_int (height), Qnil);
 	  }
 	if (position_specified_p)
 	  {
-	    Lisp_Object frame;
-	    XSETFRAME (frame, f);
+	    Lisp_Object frame = wrap_frame (f);
+
 	    Fset_frame_position (frame, make_int (x), make_int (y));
 	  }
       }
@@ -1105,7 +1083,7 @@ WARNING: can only handle plain/text and file: transfers!
       struct device *d    = get_device_from_display (display);
       struct x_device *xd = DEVICE_X_DATA (d);
       XWindowAttributes win_attrib;
-      unsigned int modifier = 0, state = 0;
+      int modifier = 0, state = 0;
       char *Ctext;
       int numItems = 0, textlen = 0, pos = 0;
       Lisp_Event *lisp_event = XEVENT (event);
@@ -1130,31 +1108,31 @@ WARNING: can only handle plain/text and file: transfers!
       x_event.xbutton.root = XRootWindow(x_event.xbutton.display, 0);
       x_event.xbutton.subwindow = 0;
       x_event.xbutton.time = lisp_event->timestamp;
-      x_event.xbutton.x = lisp_event->event.button.x;
-      x_event.xbutton.y = lisp_event->event.button.y;
+      x_event.xbutton.x = EVENT_BUTTON_X (lisp_event);
+      x_event.xbutton.y = EVENT_BUTTON_Y (lisp_event);
       if (Success == XGetWindowAttributes (x_event.xbutton.display,
 					   x_event.xbutton.window,
 					   &win_attrib))
 	{
-	  x_event.xbutton.x_root = win_attrib.x + lisp_event->event.button.x;
-	  x_event.xbutton.y_root = win_attrib.y + lisp_event->event.button.y;
+	  x_event.xbutton.x_root = win_attrib.x + EVENT_BUTTON_X (lisp_event);
+	  x_event.xbutton.y_root = win_attrib.y + EVENT_BUTTON_Y (lisp_event);
 	}
       else
 	{
-	  x_event.xbutton.x_root = lisp_event->event.button.x; /* this is wrong */
-	  x_event.xbutton.y_root = lisp_event->event.button.y;
+	  x_event.xbutton.x_root = EVENT_BUTTON_X (lisp_event); /* this is wrong */
+	  x_event.xbutton.y_root = EVENT_BUTTON_Y (lisp_event);
 	}
-      modifier = lisp_event->event.button.modifiers;
+      modifier = EVENT_BUTTON_MODIFIERS (lisp_event);
       if (modifier & XEMACS_MOD_SHIFT)   state |= ShiftMask;
       if (modifier & XEMACS_MOD_CONTROL) state |= ControlMask;
       if (modifier & XEMACS_MOD_META)    state |= xd->MetaMask;
       if (modifier & XEMACS_MOD_SUPER)   state |= xd->SuperMask;
       if (modifier & XEMACS_MOD_HYPER)   state |= xd->HyperMask;
       if (modifier & XEMACS_MOD_ALT)     state |= xd->AltMask;
-      state |= Button1Mask << (lisp_event->event.button.button-1);
+      state |= Button1Mask << (EVENT_BUTTON_BUTTON (lisp_event)-1);
 
       x_event.xbutton.state = state;
-      x_event.xbutton.button = lisp_event->event.button.button;
+      x_event.xbutton.button = EVENT_BUTTON_BUTTON (lisp_event);
       x_event.xkey.same_screen = True;
 
       /* convert data strings into a big string */
@@ -1250,7 +1228,7 @@ x_cde_transfer_callback (Widget widget, XtPointer clientData,
 
   GCPRO3 (frame, l_type, l_data);
 
-  frame = make_frame ((struct frame *) clientData);
+  frame = wrap_frame ((struct frame *) clientData);
 
   if (transferInfo->dropData->protocol == DtDND_FILENAME_TRANSFER)
     {
@@ -1261,7 +1239,7 @@ x_cde_transfer_callback (Widget widget, XtPointer clientData,
 	  filePath = transferInfo->dropData->data.files[ii];
 	  hurl = dnd_url_hexify_string ((char *)filePath, "file:");
           /* #### Mule-izing required */
-	  l_data = Fcons (make_string ((Bufbyte* )hurl,
+	  l_data = Fcons (make_string ((Ibyte* )hurl,
 				       strlen (hurl)),
 			  l_data);
 	  xfree (hurl);
@@ -1283,16 +1261,16 @@ x_cde_transfer_callback (Widget widget, XtPointer clientData,
 	  /* let us forget this name thing for now... */
  	  /* filePath = transferInfo->dropData->data.buffers[ii].name;
 	     path = (filePath == NULL) ? Qnil
-	     : make_string ((Bufbyte *)filePath, strlen (filePath)); */
+	     : make_string ((Ibyte *)filePath, strlen (filePath)); */
 	  /* what, if the data is no text, and how can I tell it? */
-	  l_data = Fcons ( list3 ( list1 ( make_string ((Bufbyte *)"text/plain", 10) ),
-				   make_string ((Bufbyte *)"8bit", 4),
-				   make_string ((Bufbyte *)transferInfo->dropData->data.buffers[ii].bp,
+	  l_data = Fcons ( list3 ( list1 ( make_string ((Ibyte *)"text/plain", 10) ),
+				   make_string ((Ibyte *)"8bit", 4),
+				   make_string ((Ibyte *)transferInfo->dropData->data.buffers[ii].bp,
 						transferInfo->dropData->data.buffers[ii].size) ),
 			   l_data );
  	}
       drag_not_done = 0;
-      unbind_to(speccount, Qnil);
+      unbind_to (speccount);
     }
   else /* the other cases: NOOP_TRANSFER */
     enqueue=0;
@@ -1331,9 +1309,9 @@ The type defaults to DndText (4).
       struct device *d    = get_device_from_display (display);
       struct x_device *xd = DEVICE_X_DATA (d);
       XWindowAttributes win_attrib;
-      unsigned int modifier = 0, state = 0;
+      int modifier = 0, state = 0;
       char *dnd_data = NULL;
-      unsigned long dnd_len = 0;
+      long dnd_len = 0;
       int dnd_typ = DndText, dnd_dealloc = 0;
       Lisp_Event *lisp_event = XEVENT (event);
 
@@ -1395,32 +1373,32 @@ The type defaults to DndText (4).
       x_event.xbutton.root = XRootWindow(x_event.xkey.display, 0);
       x_event.xbutton.subwindow = 0;
       x_event.xbutton.time = lisp_event->timestamp;
-      x_event.xbutton.x = lisp_event->event.button.x;
-      x_event.xbutton.y = lisp_event->event.button.y;
+      x_event.xbutton.x = EVENT_BUTTON_X (lisp_event);
+      x_event.xbutton.y = EVENT_BUTTON_Y (lisp_event);
       if (Success == XGetWindowAttributes (x_event.xbutton.display,
 					   x_event.xbutton.window,
 					   &win_attrib))
 	{
-	  x_event.xbutton.x_root = win_attrib.x + lisp_event->event.button.x;
-	  x_event.xbutton.y_root = win_attrib.y + lisp_event->event.button.y;
+	  x_event.xbutton.x_root = win_attrib.x + EVENT_BUTTON_X (lisp_event);
+	  x_event.xbutton.y_root = win_attrib.y + EVENT_BUTTON_Y (lisp_event);
 	}
       else
 	{
-	  x_event.xbutton.x_root = lisp_event->event.button.x; /* this is wrong */
-	  x_event.xbutton.y_root = lisp_event->event.button.y;
+	  x_event.xbutton.x_root = EVENT_BUTTON_X (lisp_event); /* this is wrong */
+	  x_event.xbutton.y_root = EVENT_BUTTON_Y (lisp_event);
 	}
 
-      modifier = lisp_event->event.button.modifiers;
+      modifier = EVENT_BUTTON_MODIFIERS (lisp_event);
       if (modifier & XEMACS_MOD_SHIFT)   state |= ShiftMask;
       if (modifier & XEMACS_MOD_CONTROL) state |= ControlMask;
       if (modifier & XEMACS_MOD_META)    state |= xd->MetaMask;
       if (modifier & XEMACS_MOD_SUPER)   state |= xd->SuperMask;
       if (modifier & XEMACS_MOD_HYPER)   state |= xd->HyperMask;
       if (modifier & XEMACS_MOD_ALT)     state |= xd->AltMask;
-      state |= Button1Mask << (lisp_event->event.button.button-1);
+      state |= Button1Mask << (EVENT_BUTTON_BUTTON (lisp_event)-1);
 
       x_event.xbutton.state = state;
-      x_event.xbutton.button = lisp_event->event.button.button;
+      x_event.xbutton.button = EVENT_BUTTON_BUTTON (lisp_event);
       x_event.xkey.same_screen = True;
 
       DndSetData(dnd_typ, (unsigned char *)dnd_data, dnd_len);
@@ -1896,7 +1874,7 @@ x_create_widgets (struct frame *f, Lisp_Object lisp_window_id,
 
 #ifndef EXTERNAL_WIDGET
   if (!NILP (lisp_window_id))
-    error ("support for external widgets was not enabled at compile-time");
+    signal_error (Qunimplemented, "support for external widgets was not enabled at compile-time", Qunbound);
 #else
   if (!NILP (lisp_window_id))
     {
@@ -1917,7 +1895,8 @@ x_create_widgets (struct frame *f, Lisp_Object lisp_window_id,
       else
 	sscanf (string, "%lu", &window_id);
       if (!is_valid_window (window_id, d))
-	error ("Invalid window %lu", (unsigned long) window_id);
+	signal_ferror (Qinvalid_argument, "Invalid window %lu",
+		       (unsigned long) window_id);
       FRAME_X_EXTERNAL_WINDOW_P (f) = 1;
     } else
 #endif /* EXTERNAL_WIDGET */
@@ -2141,6 +2120,7 @@ allocate_x_frame_struct (struct frame *f)
   f->frame_data = xnew_and_zero (struct x_frame);
 
   /* yeah, except the lisp ones */
+  FRAME_X_LAST_MENUBAR_BUFFER (f) = Qnil;
   FRAME_X_ICON_PIXMAP (f) = Qnil;
   FRAME_X_ICON_PIXMAP_MASK (f) = Qnil;
 }
@@ -2151,7 +2131,8 @@ allocate_x_frame_struct (struct frame *f)
 /************************************************************************/
 
 static void
-x_init_frame_1 (struct frame *f, Lisp_Object props)
+x_init_frame_1 (struct frame *f, Lisp_Object props,
+		int frame_name_is_defaulted)
 {
   /* This function can GC */
   Lisp_Object device = FRAME_DEVICE (f);
@@ -2164,8 +2145,8 @@ x_init_frame_1 (struct frame *f, Lisp_Object props)
 	popup = Fselected_frame (device);
       CHECK_LIVE_FRAME (popup);
       if (!EQ (device, FRAME_DEVICE (XFRAME (popup))))
-	signal_simple_error_2 ("Parent must be on same device as frame",
-			       device, popup);
+	invalid_argument_2 ("Parent must be on same device as frame",
+			    device, popup);
     }
 
   /*
@@ -2215,6 +2196,7 @@ x_init_frame_3 (struct frame *f)
 static void
 x_mark_frame (struct frame *f)
 {
+  mark_object (FRAME_X_LAST_MENUBAR_BUFFER (f));
   mark_object (FRAME_X_ICON_PIXMAP (f));
   mark_object (FRAME_X_ICON_PIXMAP_MASK (f));
 }
@@ -2282,11 +2264,11 @@ a string.
 */
        (frame))
 {
-  char str[255];
+  Ibyte str[255];
   struct frame *f = decode_x_frame (frame);
 
-  sprintf (str, "%lu", XtWindow (FRAME_X_TEXT_WIDGET (f)));
-  return build_string (str);
+  qxesprintf (str, "%lu", XtWindow (FRAME_X_TEXT_WIDGET (f)));
+  return build_intstring (str);
 }
 
 
@@ -2412,7 +2394,7 @@ x_get_mouse_position (struct device *d, Lisp_Object *frame, int *x, int *y)
   f = x_any_window_to_frame (d, win);
   if (!f)
     return 0;
-  XSETFRAME (*frame, f);
+  *frame = wrap_frame (f);
 
   if (XTranslateCoordinates (display, win,
 			     XtWindow (FRAME_X_TEXT_WIDGET (f)),
@@ -2426,7 +2408,7 @@ x_get_mouse_position (struct device *d, Lisp_Object *frame, int *x, int *y)
 static void
 x_cant_notify_wm_error (void)
 {
-  error ("Can't notify window manager of iconification.");
+  signal_error (Qgui_error, "Can't notify window manager of iconification", Qunbound);
 }
 
 /* Raise frame F.  */
@@ -2529,6 +2511,9 @@ static int
 x_frame_visible_p (struct frame *f)
 {
 #if 0
+
+  /* #### Ben suggests using x_frame_window_state (f) == NormalState. */
+
   Display *display = DEVICE_X_DISPLAY (XDEVICE (f->device));
   XWindowAttributes xwa;
   int result;
@@ -2708,9 +2693,8 @@ x_update_frame_external_traits (struct frame* frm, Lisp_Object name)
 {
   Arg al[10];
   int ac = 0;
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (frm);
 
-  XSETFRAME(frame, frm);
 
   if (EQ (name, Qforeground))
    {
@@ -2775,8 +2759,7 @@ x_update_frame_external_traits (struct frame* frm, Lisp_Object name)
 void
 syms_of_frame_x (void)
 {
-  defsymbol (&Qwindow_id, "window-id");
-  defsymbol (&Qx_resource_name, "x-resource-name");
+  DEFSYMBOL (Qx_resource_name);
 
   DEFSUBR (Fx_window_id);
 #ifdef HAVE_CDE
@@ -2812,8 +2795,8 @@ console_type_create_frame_x (void)
   CONSOLE_HAS_METHOD (x, internal_frame_property_p);
   CONSOLE_HAS_METHOD (x, frame_properties);
   CONSOLE_HAS_METHOD (x, set_frame_properties);
-  CONSOLE_HAS_METHOD (x, set_title_from_bufbyte);
-  CONSOLE_HAS_METHOD (x, set_icon_name_from_bufbyte);
+  CONSOLE_HAS_METHOD (x, set_title_from_ibyte);
+  CONSOLE_HAS_METHOD (x, set_icon_name_from_ibyte);
   CONSOLE_HAS_METHOD (x, frame_visible_p);
   CONSOLE_HAS_METHOD (x, frame_totally_visible_p);
   CONSOLE_HAS_METHOD (x, frame_iconified_p);
