@@ -1,6 +1,6 @@
 /* Generic GUI code. (menubars, scrollbars, toolbars, dialogs)
    Copyright (C) 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002, 2003 Ben Wing.
    Copyright (C) 1995 Sun Microsystems, Inc.
    Copyright (C) 1998 Free Software Foundation, Inc.
 
@@ -23,15 +23,17 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: Not in FSF. */
 
-/* This file not quite Mule-ized yet but will be when merged with my
-   Mule workspace. --ben */
+/* This file Mule-ized by Ben Wing, 3-24-02. */
 
 #include <config.h>
 #include "lisp.h"
-#include "gui.h"
-#include "elhash.h"
+
 #include "buffer.h"
 #include "bytecode.h"
+#include "elhash.h"
+#include "gui.h"
+#include "menubar.h"
+#include "redisplay.h"
 
 Lisp_Object Qmenu_no_selection_hook;
 Lisp_Object Vmenu_no_selection_hook;
@@ -39,6 +41,8 @@ Lisp_Object Vmenu_no_selection_hook;
 static Lisp_Object parse_gui_item_tree_list (Lisp_Object list);
 Lisp_Object find_keyword_in_vector (Lisp_Object vector, Lisp_Object keyword);
 
+Lisp_Object Qgui_error;
+
 #ifdef HAVE_POPUPS
 
 /* count of menus/dboxes currently up */
@@ -55,10 +59,10 @@ See `popup-menu' and `popup-dialog-box'.
 #endif /* HAVE_POPUPS */
 
 int
-separator_string_p (const Bufbyte *s)
+separator_string_p (const Ibyte *s)
 {
-  const Bufbyte *p;
-  Bufbyte first;
+  const Ibyte *p;
+  Ibyte first;
 
   if (!s || s[0] == '\0')
     return 0;
@@ -101,7 +105,7 @@ get_gui_callback (Lisp_Object data, Lisp_Object *fn, Lisp_Object *arg)
       *fn = Qeval;
       *arg = list3 (Qsignal,
 		    list2 (Qquote, Qerror),
-		    list2 (Qquote, list2 (build_translated_string
+		    list2 (Qquote, list2 (build_msg_string
 					  ("illegal callback"),
 					  data)));
     }
@@ -115,13 +119,13 @@ get_gui_callback (Lisp_Object data, Lisp_Object *fn, Lisp_Object *arg)
 int
 gui_item_add_keyval_pair (Lisp_Object gui_item,
 			  Lisp_Object key, Lisp_Object val,
-			  Error_behavior errb)
+			  Error_Behavior errb)
 {
   Lisp_Gui_Item *pgui_item = XGUI_ITEM (gui_item);
   int retval = 0;
 
   if (!KEYWORDP (key))
-    syntax_error_2 ("Non-keyword in gui item", key, pgui_item->name);
+    sferror_2 ("Non-keyword in gui item", key, pgui_item->name);
 
   if (EQ (key, Q_descriptor))
     {
@@ -131,13 +135,13 @@ gui_item_add_keyval_pair (Lisp_Object gui_item,
 	  pgui_item->name   = val;
 	}
     }
-#define FROB(slot) \
+#define FROB(slot)				\
   else if (EQ (key, Q_##slot))			\
   {						\
-    if (!EQ (pgui_item->slot, val))			\
+    if (!EQ (pgui_item->slot, val))		\
       {						\
 	retval = 1;				\
-	pgui_item->slot   = val;			\
+	pgui_item->slot   = val;		\
       }						\
   }
   FROB (suffix)
@@ -162,12 +166,11 @@ gui_item_add_keyval_pair (Lisp_Object gui_item,
 	  if (SYMBOLP (val) || CHARP (val))
 	    pgui_item->accelerator = val;
 	  else if (ERRB_EQ (errb, ERROR_ME))
-	    syntax_error ("Bad keyboard accelerator", val);
+	    invalid_argument ("Bad keyboard accelerator", val);
 	}
     }
   else if (ERRB_EQ (errb, ERROR_ME))
-    syntax_error_2 ("Unknown keyword in gui item", key,
-			   pgui_item->name);
+    invalid_argument_2 ("Unknown keyword in gui item", key, pgui_item->name);
   return retval;
 }
 
@@ -197,8 +200,7 @@ allocate_gui_item (void)
   Lisp_Gui_Item *lp = alloc_lcrecord_type (Lisp_Gui_Item, &lrecord_gui_item);
   Lisp_Object val;
 
-  zero_lcrecord (lp);
-  XSETGUI_ITEM (val, lp);
+  val = wrap_gui_item (lp);
 
   gui_item_init (val);
 
@@ -212,7 +214,7 @@ allocate_gui_item (void)
  */
 static Lisp_Object
 make_gui_item_from_keywords_internal (Lisp_Object item,
-				      Error_behavior errb)
+				      Error_Behavior errb)
 {
   int length, plist_p, start;
   Lisp_Object *contents;
@@ -224,7 +226,7 @@ make_gui_item_from_keywords_internal (Lisp_Object item,
   contents = XVECTOR_DATA (item);
 
   if (length < 1)
-    syntax_error ("GUI item descriptors must be at least 1 elts long", item);
+    sferror ("GUI item descriptors must be at least 1 elts long", item);
 
   /* length 1:     		[ "name" ]
      length 2:		[ "name" callback ]
@@ -259,9 +261,9 @@ make_gui_item_from_keywords_internal (Lisp_Object item,
     {
       int i;
       if ((length - start) & 1)
-	syntax_error (
+	sferror (
 		"GUI item descriptor has an odd number of keywords and values",
-			     item);
+		 item);
 
       for (i = start; i < length;)
 	{
@@ -287,12 +289,12 @@ widget_gui_parse_item_keywords (Lisp_Object item)
   contents = XVECTOR_DATA (item);
 
   if (!NILP (desc) && !STRINGP (desc) && !VECTORP (desc))
-    syntax_error ("Invalid GUI item descriptor", item);
+    sferror ("Invalid GUI item descriptor", item);
 
   if (length & 1)
     {
       if (!SYMBOLP (contents [0]))
-	syntax_error ("Invalid GUI item descriptor", item);
+	sferror ("Invalid GUI item descriptor", item);
       contents++;			/* Ignore the leading symbol. */
       length--;
     }
@@ -321,7 +323,7 @@ update_gui_item_keywords (Lisp_Object gui_item, Lisp_Object item)
  if (length & 1)
     {
       if (!SYMBOLP (contents [0]))
-	syntax_error ("Invalid GUI item descriptor", item);
+	sferror ("Invalid GUI item descriptor", item);
       contents++;			/* Ignore the leading symbol. */
       length--;
     }
@@ -330,7 +332,7 @@ update_gui_item_keywords (Lisp_Object gui_item, Lisp_Object item)
     {
       Lisp_Object key = contents [i++];
       Lisp_Object val = contents [i++];
-      if (gui_item_add_keyval_pair (gui_item, key, val, ERROR_ME_NOT))
+      if (gui_item_add_keyval_pair (gui_item, key, val, ERROR_ME_DEBUG_WARN))
 	retval = 1;
     }
   return retval;
@@ -345,7 +347,7 @@ gui_parse_item_keywords (Lisp_Object item)
 Lisp_Object
 gui_parse_item_keywords_no_errors (Lisp_Object item)
 {
-  return make_gui_item_from_keywords_internal (item, ERROR_ME_NOT);
+  return make_gui_item_from_keywords_internal (item, ERROR_ME_DEBUG_WARN);
 }
 
 /* convert a gui item into plist properties */
@@ -380,18 +382,33 @@ gui_add_item_keywords_to_plist (Lisp_Object plist, Lisp_Object gui_item)
     Fplist_put (plist, Q_value, pgui_item->value);
 }
 
+static int
+gui_item_value (Lisp_Object form, int in_redisplay)
+{
+  /* This function can call Lisp. */
+
+#ifndef ERROR_CHECK_DISPLAY
+  /* Shortcut to avoid evaluating Qt/Qnil each time; but don't do it when
+     error-checking so we catch unprotected eval within redisplay quicker */
+  if (NILP (form))
+    return 0;
+  if (EQ (form, Qt))
+    return 1;
+#endif
+  if (in_redisplay)
+    return !NILP (eval_within_redisplay (form));
+  else
+    return !NILP (Feval (form));
+}
+
 /*
  * Decide whether a GUI item is active by evaluating its :active form
  * if any
  */
 int
-gui_item_active_p (Lisp_Object gui_item)
+gui_item_active_p (Lisp_Object gui_item, int in_redisplay)
 {
-  /* This function can call lisp */
-
-  /* Shortcut to avoid evaluating Qt each time */
-  return (EQ (XGUI_ITEM (gui_item)->active, Qt)
-	  || !NILP (Feval (XGUI_ITEM (gui_item)->active)));
+  return gui_item_value (XGUI_ITEM (gui_item)->active, in_redisplay);
 }
 
 /* set menu accelerator key to first underlined character in menu name */
@@ -410,7 +427,7 @@ gui_item_accelerator (Lisp_Object gui_item)
 Lisp_Object
 gui_name_accelerator (Lisp_Object nm)
 {
-  Bufbyte *name = XSTRING_DATA (nm);
+  Ibyte *name = XSTRING_DATA (nm);
 
   while (*name)
     {
@@ -421,15 +438,13 @@ gui_name_accelerator (Lisp_Object nm)
 	    return Qnil;
 	  if (*name == '_' && *(name + 1))
 	    {
-	      Emchar accelerator = charptr_emchar (name + 1);
-	      /* #### bogus current_buffer dependency */
-	      return make_char (DOWNCASE (current_buffer, accelerator));
+	      Ichar accelerator = itext_ichar (name + 1);
+	      return make_char (DOWNCASE (0, accelerator));
 	    }
 	}
-	INC_CHARPTR (name);
+	INC_IBYTEPTR (name);
     }
-  return make_char (DOWNCASE (current_buffer,
-			      charptr_emchar (XSTRING_DATA (nm))));
+  return make_char (DOWNCASE (0, itext_ichar (XSTRING_DATA (nm))));
 }
 
 /*
@@ -437,23 +452,20 @@ gui_name_accelerator (Lisp_Object nm)
  * if any
  */
 int
-gui_item_selected_p (Lisp_Object gui_item)
+gui_item_selected_p (Lisp_Object gui_item, int in_redisplay)
 {
-  /* This function can call lisp */
-
-  /* Shortcut to avoid evaluating Qt each time */
-  return (EQ (XGUI_ITEM (gui_item)->selected, Qt)
-	  || !NILP (Feval (XGUI_ITEM (gui_item)->selected)));
+  return gui_item_value (XGUI_ITEM (gui_item)->selected, in_redisplay);
 }
 
 Lisp_Object
 gui_item_list_find_selected (Lisp_Object gui_item_list)
 {
-  /* This function can GC. */
+  /* This function can call Lisp but cannot GC because it is called within
+     redisplay, and redisplay disables GC. */
   Lisp_Object rest;
   LIST_LOOP (rest, gui_item_list)
     {
-      if (gui_item_selected_p (XCAR (rest)))
+      if (gui_item_selected_p (XCAR (rest), 1))
 	return XCAR (rest);
     }
   return XCAR (gui_item_list);
@@ -471,8 +483,7 @@ gui_item_included_p (Lisp_Object gui_item, Lisp_Object conflist)
   Lisp_Gui_Item *pgui_item = XGUI_ITEM (gui_item);
 
   /* Evaluate :included first. Shortcut to avoid evaluating Qt each time */
-  if (!EQ (pgui_item->included, Qt)
-      && NILP (Feval (pgui_item->included)))
+  if (!gui_item_value (pgui_item->included, 0))
     return 0;
 
   /* Do :config if conflist is given */
@@ -483,41 +494,19 @@ gui_item_included_p (Lisp_Object gui_item, Lisp_Object conflist)
   return 1;
 }
 
-static DOESNT_RETURN
-signal_too_long_error (Lisp_Object name)
-{
-  syntax_error ("GUI item produces too long displayable string", name);
-}
-
-#ifdef HAVE_WINDOW_SYSTEM
 /*
- * Format "left flush" display portion of an item into BUF, guarded by
- * maximum buffer size BUF_LEN. BUF_LEN does not count for terminating
- * null character, so actual maximum size of buffer consumed is
- * BUF_LEN + 1 bytes. If buffer is not big enough, then error is
- * signaled.
- * Return value is the offset to the terminating null character into the
- * buffer.
+ * Format "left flush" display portion of an item.
  */
-unsigned int
-gui_item_display_flush_left (Lisp_Object gui_item,
-			     char *buf, Bytecount buf_len)
+Lisp_Object
+gui_item_display_flush_left (Lisp_Object gui_item)
 {
   /* This function can call lisp */
-  char *p = buf;
-  Bytecount len;
   Lisp_Gui_Item *pgui_item = XGUI_ITEM (gui_item);
+  Lisp_Object retval;
 
-  /* Copy item name first */
   CHECK_STRING (pgui_item->name);
-  len = XSTRING_LENGTH (pgui_item->name);
-  if (len > buf_len)
-    signal_too_long_error (pgui_item->name);
-  memcpy (p, XSTRING_DATA (pgui_item->name), len);
-  p += len;
+  retval = pgui_item->name;
 
-  /* Add space and suffix, if there is a suffix.
-   * If suffix is not string evaluate it */
   if (!NILP (pgui_item->suffix))
     {
       Lisp_Object suffix = pgui_item->suffix;
@@ -528,68 +517,65 @@ gui_item_display_flush_left (Lisp_Object gui_item,
 	  CHECK_STRING (suffix);
 	}
 
-      len = XSTRING_LENGTH (suffix);
-      if (p + len + 1 > buf + buf_len)
-	signal_too_long_error (pgui_item->name);
-      *(p++) = ' ';
-      memcpy (p, XSTRING_DATA (suffix), len);
-      p += len;
+      retval = concat3 (pgui_item->name, build_string (" "), suffix);
     }
-  *p = '\0';
-  return p - buf;
+
+  return retval;
 }
 
 /*
- * Format "right flush" display portion of an item into BUF, guarded by
- * maximum buffer size BUF_LEN. BUF_LEN does not count for terminating
- * null character, so actual maximum size of buffer consumed is
- * BUF_LEN + 1 bytes. If buffer is not big enough, then error is
- * signaled.
- * Return value is the offset to the terminating null character into the
- * buffer.
+ * Format "right flush" display portion of an item into BUF.
  */
-unsigned int
-gui_item_display_flush_right (Lisp_Object gui_item,
-			      char *buf, Bytecount buf_len)
+Lisp_Object
+gui_item_display_flush_right (Lisp_Object gui_item)
 {
   Lisp_Gui_Item *pgui_item = XGUI_ITEM (gui_item);
-  *buf = 0;
 
 #ifdef HAVE_MENUBARS
   /* Have keys? */
   if (!menubar_show_keybindings)
-    return 0;
+    return Qnil;
 #endif
 
   /* Try :keys first */
   if (!NILP (pgui_item->keys))
     {
       CHECK_STRING (pgui_item->keys);
-      if (XSTRING_LENGTH (pgui_item->keys) + 1 > buf_len)
-	signal_too_long_error (pgui_item->name);
-      memcpy (buf, XSTRING_DATA (pgui_item->keys),
-	      XSTRING_LENGTH (pgui_item->keys) + 1);
-      return XSTRING_LENGTH (pgui_item->keys);
+      return pgui_item->keys;
     }
 
   /* See if we can derive keys out of callback symbol */
   if (SYMBOLP (pgui_item->callback))
     {
-      char buf2[1024]; /* #### */
-      Bytecount len;
-
-      where_is_to_char (pgui_item->callback, buf2);
-      len = strlen (buf2);
-      if (len > buf_len)
-	signal_too_long_error (pgui_item->name);
-      strcpy (buf, buf2);
-      return len;
+      DECLARE_EISTRING_MALLOC (buf);
+      Lisp_Object str;
+      
+      where_is_to_char (pgui_item->callback, buf);
+      str = eimake_string (buf);
+      eifree (buf);
+      return str;
     }
 
   /* No keys - no right flush display */
-  return 0;
+  return Qnil;
 }
-#endif /* HAVE_WINDOW_SYSTEM */
+
+static const struct memory_description gui_item_description [] = {
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, name) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, callback) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, callback_ex) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, suffix) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, active) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, included) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, config) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, filter) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, style) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, selected) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, keys) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, accelerator) },
+  { XD_LISP_OBJECT, offsetof (struct Lisp_Gui_Item, value) },
+  { XD_END }
+};
 
 static Lisp_Object
 mark_gui_item (Lisp_Object obj)
@@ -614,7 +600,7 @@ mark_gui_item (Lisp_Object obj)
   return Qnil;
 }
 
-static unsigned long
+static Hashcode
 gui_item_hash (Lisp_Object obj, int depth)
 {
   Lisp_Gui_Item *p = XGUI_ITEM (obj);
@@ -638,25 +624,40 @@ gui_item_id_hash (Lisp_Object hashtable, Lisp_Object gitem, int slot)
 {
   int hashid = gui_item_hash (gitem, 0);
   int id = GUI_ITEM_ID_BITS (hashid, slot);
-  while (!NILP (Fgethash (make_int (id),
-			  hashtable, Qnil)))
+  while (!UNBOUNDP (Fgethash (make_int (id), hashtable, Qunbound)))
     {
       id = GUI_ITEM_ID_BITS (id + 1, slot);
     }
   return id;
 }
 
+static int
+gui_value_equal (Lisp_Object a, Lisp_Object b, int depth, int in_redisplay)
+{
+  if (in_redisplay)
+    return internal_equal_trapping_problems
+      (Qredisplay, "Error calling function within redisplay", 0, 0,
+       /* say they're not equal in case of error; code calling
+	  gui_item_equal_sans_selected() in redisplay does extra stuff
+	  only when equal */
+       0, a, b, depth);
+  else
+    return internal_equal (a, b, depth);
+}
+
 int
-gui_item_equal_sans_selected (Lisp_Object obj1, Lisp_Object obj2, int depth)
+gui_item_equal_sans_selected (Lisp_Object obj1, Lisp_Object obj2, int depth,
+			      int in_redisplay)
 {
   Lisp_Gui_Item *p1 = XGUI_ITEM (obj1);
   Lisp_Gui_Item *p2 = XGUI_ITEM (obj2);
 
-  if (!(internal_equal (p1->name, p2->name, depth + 1)
+  if (!(gui_value_equal (p1->name, p2->name, depth + 1, in_redisplay)
 	&&
-	internal_equal (p1->callback, p2->callback, depth + 1)
+	gui_value_equal (p1->callback, p2->callback, depth + 1, in_redisplay)
 	&&
-	internal_equal (p1->callback_ex, p2->callback_ex, depth + 1)
+	gui_value_equal (p1->callback_ex, p2->callback_ex, depth + 1,
+			 in_redisplay)
 	&&
 	EQ (p1->suffix, p2->suffix)
 	&&
@@ -685,8 +686,7 @@ gui_item_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
   Lisp_Gui_Item *p1 = XGUI_ITEM (obj1);
   Lisp_Gui_Item *p2 = XGUI_ITEM (obj2);
 
-  if (!(gui_item_equal_sans_selected (obj1, obj2, depth)
-	&&
+  if (!(gui_item_equal_sans_selected (obj1, obj2, depth, 0) &&
 	EQ (p1->selected, p2->selected)))
     return 0;
   return 1;
@@ -696,14 +696,11 @@ static void
 print_gui_item (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Gui_Item *g = XGUI_ITEM (obj);
-  char buf[20];
 
   if (print_readably)
-    error ("printing unreadable object #<gui-item 0x%x>", g->header.uid);
+    printing_unreadable_object ("#<gui-item 0x%x>", g->header.uid);
 
-  write_c_string ("#<gui-item ", printcharfun);
-  sprintf (buf, "0x%x>", g->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<gui-item 0x%x>", g->header.uid);
 }
 
 Lisp_Object
@@ -771,7 +768,7 @@ parse_gui_item_tree_item (Lisp_Object entry)
       CHECK_STRING (entry);
     }
   else
-    syntax_error ("item must be a vector or a string", entry);
+    sferror ("item must be a vector or a string", entry);
 
   RETURN_UNGCPRO (ret);
 }
@@ -812,17 +809,30 @@ parse_gui_item_tree_list (Lisp_Object list)
 }
 
 static void
-finalize_gui_item (void* header, int for_disksave)
+finalize_gui_item (void *header, int for_disksave)
 {
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("gui-item", gui_item,
+			       0, /*dumpable-flag*/
 			       mark_gui_item, print_gui_item,
 			       finalize_gui_item, gui_item_equal,
 			       gui_item_hash,
-			       0,
+			       gui_item_description,
 			       Lisp_Gui_Item);
 
+DOESNT_RETURN
+gui_error (const Char_ASCII *reason, Lisp_Object frob)
+{
+  signal_error (Qgui_error, reason, frob);
+}
+
+DOESNT_RETURN
+gui_error_2 (const Char_ASCII *reason, Lisp_Object frob0, Lisp_Object frob1)
+{
+  signal_error_2 (Qgui_error, reason, frob0, frob1);
+}
+
 void
 syms_of_gui (void)
 {
@@ -830,6 +840,8 @@ syms_of_gui (void)
 
   DEFSYMBOL (Qmenu_no_selection_hook);
 
+  DEFERROR_STANDARD (Qgui_error, Qio_error);
+
 #ifdef HAVE_POPUPS
   DEFSUBR (Fpopup_up_p);
 #endif
