@@ -26,6 +26,8 @@
 
 /* Synched up with: Not in FSF. */
 
+/* This file Mule-ized by Ben Wing, 5-15-01. */
+
 /* There are four compile-time options.
  *
  * XTOOLKIT	This will be part of an Xt program.
@@ -49,70 +51,66 @@
  *			correct error facilities.
  *      4/11/94, rjc    Added wait_for_sounds to be called when user wants to
  *			be sure all play has finished.
+ *      1998-10-01 rlt  Added support for WAVE files.
+ *      2002-10-16      Jon Trulson modifed this to work with NAS releases
+ *                      1.5f and higher.  We were using the private variable
+ *                      SoundFileInfo that doesn't exist anymore.  But preserve
+ *                      backward compatibility.  This will not work for some
+ *                      versions of NAS around 1.5b to 1.5f or so.  Known to
+ *                      work on 1.2p5 and 1.6.
  */
 
-#ifdef emacs
 #include <config.h>
 #include "lisp.h"
-#endif
 
-#if __STDC__ || defined (STDC_HEADERS)
-#    include <stdlib.h>
-#    include <stdarg.h>
-#    include <string.h>
-#endif
+#include "sound.h"
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <stdio.h>
+#include "sysdep.h"
 #include "syssignal.h"
 
+/* NAS <= 1.2p5 defines {BIG,LITTLE}_ENDIAN in <audio/fileutil.h>,
+   conflicting with GNU libc (at least); newer versions avoid this
+   name space pollution.
+
+   DO NOT USE THOSE MACROS in this file.  Use NAS_{BIG,LITTLE}_ENDIAN.
+
+   It would be slightly more reliable to do this via configure, but that
+   seems unnecessarily complex.
+*/
 #undef LITTLE_ENDIAN
 #undef BIG_ENDIAN
+
 #include <audio/audiolib.h>
 #include <audio/soundlib.h>
 #include <audio/snd.h>
+#include <audio/wave.h>
 #include <audio/fileutil.h>
 
-#ifdef emacs
+/* NAS <= 1.2p5 <audio/fileutil.h> doesn't define the NAS_ versions */
+#ifndef NAS_LITTLE_ENDIAN
+# define NAS_LITTLE_ENDIAN LITTLE_ENDIAN
+# define NAS_BIG_ENDIAN BIG_ENDIAN
+#endif
 
-#    define XTOOLKIT
-#    define XTEVENTS
-#    define ROBUST_PLAY
-#    define CACHE_SOUNDS
+#define XTOOLKIT
+#define XTEVENTS
+#define ROBUST_PLAY
+#define CACHE_SOUNDS
 
     /*
      * For old NAS libraries, force playing to be synchronous
      * and declare the long jump point locally.
      */
 
-#    if defined (NAS_NO_ERROR_JUMP)
-
-#	undef XTEVENTS
-
-#	include <setjmp.h>
-	jmp_buf AuXtErrorJump;
-#    endif
-
-     /* The GETTEXT is correct. --ben */
-#    define warn(str) warn_when_safe (Qnas, Qwarning, "nas: %s ", GETTEXT (str))
-
-#    define play_sound_file nas_play_sound_file
-#    define play_sound_data nas_play_sound_data
-#    define wait_for_sounds nas_wait_for_sounds
-#    define init_play       nas_init_play
-#    define close_down_play nas_close_down_play
-
-#else /* !emacs */
-#    define warn(str) fprintf (stderr, "%s\n", (str))
-#    define CONST const
-#endif /* emacs */
+#if defined (NAS_NO_ERROR_JUMP)
+# undef XTEVENTS
+# include <setjmp.h>
+jmp_buf AuXtErrorJump;
+#endif
 
 #ifdef XTOOLKIT
-#    include <X11/Intrinsic.h>
-#    include <audio/Xtutil.h>
+# include <X11/Intrinsic.h>
+# include <audio/Xtutil.h>
 #endif
 
 #if defined (ROBUST_PLAY)
@@ -123,7 +121,7 @@ SIGTYPE sigpipe_handle (int signo);
 
 extern Lisp_Object Vsynchronous_sounds;
 
-static Sound SoundOpenDataForReading (unsigned char *data, int length);
+static Sound SoundOpenDataForReading (UChar_Binary *data, int length);
 
 static AuServer       *aud;
 
@@ -135,23 +133,31 @@ static int sounds_in_play;
 static Display *aud_server;
 static XtInputId input_id;
 #else
-static char *aud_server;
+static Extbyte *aud_server;
 #endif /* XTOOLKIT */
 
-char *
-init_play (
+Extbyte *
+nas_init_play (
 #ifdef XTOOLKIT
 	   Display *display
 #else
-	   char *server
+	   Extbyte *server
+#endif
+	   );
+Extbyte *
+nas_init_play (
+#ifdef XTOOLKIT
+	   Display *display
+#else
+	   Extbyte *server
 #endif
 	   )
 {
-  char *err_message;
-  SIGTYPE (*old_sigpipe) ();
+  Extbyte *err_message;
+  SIGTYPE (*old_sigpipe) (int);
 
 #ifdef XTOOLKIT
-  char * server = DisplayString (display);
+  Extbyte * server = DisplayString (display);
   XtAppContext app_context = XtDisplayToApplicationContext (display);
 
   aud_server = display;
@@ -161,13 +167,11 @@ init_play (
 #endif
 
 #ifdef ROBUST_PLAY
-  old_sigpipe = signal (SIGPIPE, sigpipe_handle);
+  old_sigpipe = EMACS_SIGNAL (SIGPIPE, sigpipe_handle);
   if (setjmp (AuXtErrorJump))
     {
-      signal (SIGPIPE, old_sigpipe);
-#ifdef emacs
+      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
       start_interrupts ();
-#endif  
       return "error in NAS";
     }
 #endif
@@ -177,17 +181,13 @@ init_play (
   AuDefaultErrorHandler = CatchErrorAndJump;
 #endif
 
-#ifdef emacs
   stop_interrupts ();
-#endif  
   aud = AuOpenServer (server, 0, NULL, 0, NULL, &err_message);
-#ifdef emacs
   start_interrupts ();
-#endif  
   if (!aud)
     {
 #ifdef ROBUST_PLAY
-      signal (SIGPIPE, old_sigpipe);
+      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
       if (err_message == NULL)
 	return "Can't connect to audio server";
@@ -214,7 +214,7 @@ init_play (
 #endif
 
 #ifdef ROBUST_PLAY
-  signal (SIGPIPE, old_sigpipe);
+  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
 
   sounds_in_play = 0;
@@ -222,12 +222,12 @@ init_play (
   return NULL;
 }
 
-void
-close_down_play (void)
+static void
+nas_close_down_play (void)
 
 {
   AuCloseServer (aud);
-  warn ("disconnected from audio server");
+  sound_warn ("disconnected from audio server");
 }
 
  /********************************************************************\
@@ -237,7 +237,7 @@ close_down_play (void)
  \********************************************************************/
 
 static void
-doneCB (AuServer       *aud,
+doneCB (AuServer       *auserver,
 	AuEventHandlerRec *handler,
 	AuEvent        *ev,
 	AuPointer       data)
@@ -259,7 +259,7 @@ doneCB (AuServer       *aud,
 static void
 do_caching_play (Sound s,
 		 int volume,
-		 unsigned char *buf)
+		 UChar_Binary *buf)
 
 {
   AuBucketAttributes *list, b;
@@ -273,23 +273,23 @@ do_caching_play (Sound s,
 
   if (list == NULL)
     {
-      unsigned char *my_buf;
+      AuPointer my_buf;
 
       if (buf==NULL)
 	{
-	  if ((my_buf=malloc (SoundNumBytes (s)))==NULL)
+	  if ((my_buf= (AuPointer) malloc (SoundNumBytes (s)))==NULL)
 	    {
 	      return;
 	    }
 
-	  if (SoundReadFile (my_buf, SoundNumBytes (s), s) != SoundNumBytes (s))
+	  if (SoundReadFile ((Extbyte *) my_buf, SoundNumBytes (s), s) != SoundNumBytes (s))
 	    {
 	      free (my_buf);
 	      return;
 	    }
 	}
       else
-	my_buf=buf;
+	my_buf = (AuPointer) buf;
 
       id = AuSoundCreateBucketFromData (aud, 
 					s,
@@ -321,8 +321,9 @@ do_caching_play (Sound s,
 #endif /* CACHE_SOUNDS */
 
 
+void nas_wait_for_sounds (void);
 void 
-wait_for_sounds (void)
+nas_wait_for_sounds (void)
 
 {
   AuEvent         ev;
@@ -334,44 +335,46 @@ wait_for_sounds (void)
     }
 }
 
+int nas_play_sound_file (Extbyte *sound_file, int volume);
 int
-play_sound_file (char *sound_file,
-		 int volume)
+nas_play_sound_file (Extbyte *sound_file,
+		     int volume)
 {
-  SIGTYPE (*old_sigpipe) ();
+  SIGTYPE (*old_sigpipe) (int);
 
 #ifdef ROBUST_PLAY
-  old_sigpipe=signal (SIGPIPE, sigpipe_handle);
+  old_sigpipe = EMACS_SIGNAL (SIGPIPE, sigpipe_handle);
   if (setjmp (AuXtErrorJump))
     {
-      signal (SIGPIPE, old_sigpipe);
+      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
       return 0;
     }
 #endif
 
-  if (aud==NULL) {
-    if (aud_server != NULL)
-      {
-	char *m;
-	/* attempt to reconect */
-	if ((m=init_play (aud_server))!= NULL)
-	  {
-
+  if (aud==NULL)
+    {
+      if (aud_server != NULL)
+	{
+	  Extbyte *m;
+	  /* attempt to reconect */
+	  if ((m = nas_init_play (aud_server)) != NULL)
+	    {
+	      
 #ifdef ROBUST_PLAY
-	    signal (SIGPIPE, old_sigpipe);
+	      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
-	    return 0;
-	  }
-      }
-    else
-      {
-	warn ("Attempt to play with no audio init\n");
+	      return 0;
+	    }
+	}
+      else
+	{
+	  sound_warn ("Attempt to play with no audio init\n");
 #ifdef ROBUST_PLAY
-	signal (SIGPIPE, old_sigpipe);
+	  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
-	return 0;
-      }
-  }
+	  return 0;
+	}
+    }
 
 #ifndef CACHE_SOUNDS
   sounds_in_play++;
@@ -393,7 +396,7 @@ play_sound_file (char *sound_file,
     if ((s = SoundOpenFileForReading (sound_file))==NULL)
       {
 #ifdef ROBUST_PLAY
-	signal (SIGPIPE, old_sigpipe);
+	EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
 	return 0;
       }
@@ -411,39 +414,36 @@ play_sound_file (char *sound_file,
 #endif /* CACHE_SOUNDS */
 
 #ifndef XTEVENTS
-  wait_for_sounds ();
+  nas_wait_for_sounds ();
 #else
   if (!NILP (Vsynchronous_sounds))
-    {
-      wait_for_sounds ();
-    }
+    nas_wait_for_sounds ();
 #endif
 
 #ifdef ROBUST_PLAY
-  signal (SIGPIPE, old_sigpipe);
+  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
 
   return 1;
 }
 
+int nas_play_sound_data (UChar_Binary *data, int length, int volume);
 int
-play_sound_data (unsigned char *data,
-		 int length, 
-		 int volume)
+nas_play_sound_data (UChar_Binary *data, int length, int volume)
 {
   Sound s;
   int offset;
-  SIGTYPE (*old_sigpipe) ();
+  SIGTYPE (*old_sigpipe) (int);
 
 #if !defined (XTEVENTS)
   AuEvent         ev;
 #endif
 
 #ifdef ROBUST_PLAY
-  old_sigpipe = signal (SIGPIPE, sigpipe_handle);
-  if (setjmp (AuXtErrorJump) !=0)
+  old_sigpipe = EMACS_SIGNAL (SIGPIPE, sigpipe_handle);
+  if (setjmp (AuXtErrorJump) != 0)
     {
-      signal (SIGPIPE, old_sigpipe);
+      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
       return 0;
     }
 #endif
@@ -452,21 +452,21 @@ play_sound_data (unsigned char *data,
   if (aud == NULL) {
     if (aud_server != NULL)
       {
-	char *m;
+	Extbyte *m;
 	/* attempt to reconect */
-	if ((m = init_play (aud_server)) != NULL)
+	if ((m = nas_init_play (aud_server)) != NULL)
 	  {
 #ifdef ROBUST_PLAY
-	    signal (SIGPIPE, old_sigpipe);
+	    EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
 	    return 0;
 	  }
       }
     else
       {
-	warn ("Attempt to play with no audio init\n");
+	sound_warn ("Attempt to play with no audio init\n");
 #ifdef ROBUST_PLAY
-	signal (SIGPIPE, old_sigpipe);
+	EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
 	return 0;
       }
@@ -474,9 +474,9 @@ play_sound_data (unsigned char *data,
 
   if ((s=SoundOpenDataForReading (data, length))==NULL)
     {
-      warn ("unknown sound type");
+      sound_warn ("unknown sound type");
 #ifdef ROBUST_PLAY
-      signal (SIGPIPE, old_sigpipe);
+      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
       return 0;
     }
@@ -486,12 +486,16 @@ play_sound_data (unsigned char *data,
       /* hack, hack */
       offset = ((SndInfo *) (s->formatInfo))->h.dataOffset;
     }
+  else if (SoundFileFormat (s) == SoundFileFormatWave)
+    {
+      offset = ((WaveInfo *) (s->formatInfo))->dataOffset;
+    }
   else
     {
-      warn ("only understand snd files at the moment");
+      sound_warn ("only understand snd and wave files at the moment");
       SoundCloseFile (s);
 #ifdef ROBUST_PLAY
-      signal (SIGPIPE, old_sigpipe);
+      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
       return 0;
     }
@@ -511,25 +515,21 @@ play_sound_data (unsigned char *data,
 #else
   /* Cache the sounds in buckets on the server */
 
-  {
-    do_caching_play (s, volume, data+offset);
-  }
+  do_caching_play (s, volume, data+offset);
 #endif /* CACHE_SOUNDS */
 
 
 #ifndef XTEVENTS
-  wait_for_sounds ();
+  nas_wait_for_sounds ();
 #else
   if (!NILP (Vsynchronous_sounds))
-    {
-      wait_for_sounds ();
-    }
+    nas_wait_for_sounds ();
 #endif
 
   SoundCloseFile (s); 
 
 #ifdef ROBUST_PLAY
-  signal (SIGPIPE, old_sigpipe);
+  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
 #endif
 
   return 1;
@@ -549,15 +549,13 @@ static AuBool
 CatchIoErrorAndJump (AuServer *old_aud)
 {
   if (old_aud)
-    warn ("Audio Server connection broken"); 
+    sound_warn ("Audio Server connection broken"); 
   else
-    warn ("Audio Server connection broken because of signal");
+    sound_warn ("Audio Server connection broken because of signal");
 
 #ifdef XTEVENTS
 #ifdef XTOOLKIT
-  {
-    AuXtAppRemoveAudioHandler (aud, input_id); 
-  }
+  AuXtAppRemoveAudioHandler (aud, input_id); 
 #endif
 
   if (aud)
@@ -576,6 +574,7 @@ CatchIoErrorAndJump (AuServer *old_aud)
   longjmp (AuXtErrorJump, 1);
  
 #endif /* XTEVENTS */
+  return 0;
 }
 
 SIGTYPE
@@ -606,14 +605,14 @@ CatchErrorAndJump (AuServer *old_aud,
 
 /* Create a name from the sound. */
 
-static char *
-NameFromData (CONST unsigned char *buf,
+static Extbyte *
+NameFromData (const Char_Binary *buf,
 	      int len)
 
 {
-  unsigned char name[9];
+  Extbyte name[9];
   int i;
-  char *s;
+  Extbyte *s;
 
   buf+=len/2;
   len -= len/2;
@@ -636,11 +635,11 @@ NameFromData (CONST unsigned char *buf,
 
   if (i==8)
     {
-      strcpy (s=malloc (10), name);
+      strcpy (s = (Extbyte *) malloc (10), name);
     }
   else 
     {
-      strcpy (s=malloc (15), "short sound");
+      strcpy (s = (Extbyte *) malloc (15), "short sound");
     }
 
   return s;
@@ -651,7 +650,7 @@ NameFromData (CONST unsigned char *buf,
  */
 
 static SndInfo *
-SndOpenDataForReading (CONST char *data,
+SndOpenDataForReading (const Char_Binary *data,
 		       int length)
 
 {
@@ -666,9 +665,9 @@ SndOpenDataForReading (CONST char *data,
 
   memcpy (&si->h, data, sizeof (SndHeader));
 
-  if (LITTLE_ENDIAN)
+  if (NAS_LITTLE_ENDIAN)
     {
-      char            n;
+      Char_Binary            n;
     
       swapl (&si->h.magic, n);
       swapl (&si->h.dataOffset, n);
@@ -688,7 +687,7 @@ SndOpenDataForReading (CONST char *data,
 
   if (size)
     {
-      if (!(si->comment = (char *) malloc (size + 1)))
+      if (!(si->comment = (Extbyte *) malloc (size + 1)))
 	{
 	  free (si);
 	  return NULL;
@@ -711,30 +710,311 @@ SndOpenDataForReading (CONST char *data,
   return si;
 }
 
+/* Stuff taken from wave.c from NAS.  Just like snd files, NAS can't
+   read wave data from memory, so these functions do that for us. */
+
+#define Err()		{ return NULL; }
+#define readFourcc(_f)	dread(_f, sizeof(RIFF_FOURCC), 1)
+#define cmpID(_x, _y)							      \
+    strncmp((Char_Binary *) (_x), (Char_Binary *) (_y), sizeof(RIFF_FOURCC))
+#define PAD2(_x)	(((_x) + 1) & ~1)
+
+/* These functions here are for faking file I/O from buffer. */
+
+/* The "file" position */
+static size_t file_posn;
+/* The length of the "file" */
+static size_t file_len;
+/* The actual "file" data. */
+static const void* file_data;
+
+/* Like fopen, but for a buffer in memory */
+static void
+dopen (const void* data, size_t length)
+{
+   file_data = data;
+   file_len = length;
+   file_posn = 0;
+}
+
+/* Like fread, but for a buffer in memory */
+static int
+dread (void* buf, size_t size, size_t nitems)
+{
+  size_t nread = size * nitems;
+  
+  if (file_posn + nread <= file_len)
+    {
+      memcpy(buf, (Char_Binary *) file_data + file_posn, size * nitems);
+      file_posn += nread;
+      return nitems;
+    }
+  else
+    {
+      return EOF;
+    }
+}
+
+/* Like fgetc, but for a buffer in memory */
+static int
+dgetc (void)
+{
+  if (file_posn < file_len)
+    return ((Char_Binary *)file_data)[file_posn++];
+  else
+    return -1;
+}
+
+/* Like fseek, but for a buffer in memory */
+static int
+dseek (long offset, int from)
+{
+  if (from == 0)
+    file_posn = offset;
+  else if (from == 1)
+    file_posn += offset;
+  else if (from == 2)
+    file_posn = file_len + offset;
+
+  return 0;
+}
+
+/* Like ftell, but for a buffer in memory */
+static long
+dtell (void)
+{
+  return file_posn;
+}
+
+/* Data buffer analogs for FileReadS and FileReadL in NAS. */
+
+static unsigned short
+DataReadS (int swapit)
+{
+  unsigned short us;
+
+  dread(&us, 2, 1);
+  if (swapit)
+    us = FileSwapS(us);
+  return us;
+}
+
+static AuUint32
+DataReadL (int swapit)
+{
+  AuUint32 ul;
+
+  dread(&ul, 4, 1);
+  if (swapit)
+    ul = FileSwapL(ul);
+  return ul;
+}
+
+static int
+readChunk (RiffChunk *c)
+{
+  int             status;
+  Char_Binary            n;
+
+  if ((status = dread(c, sizeof(RiffChunk), 1)))
+    if (NAS_BIG_ENDIAN)
+      swapl(&c->ckSize, n);
+
+  return status;
+}
+
+/* A very straight-forward translation of WaveOpenFileForReading to
+   read the wave data from a buffer in memory. */
+
+static WaveInfo *
+WaveOpenDataForReading (const Char_Binary *data,
+			int length)
+{
+  RiffChunk       ck;
+  RIFF_FOURCC     fourcc;
+  AuInt32            fileSize;
+  WaveInfo       *wi;
+
+    
+  if (!(wi = (WaveInfo *) malloc(sizeof(WaveInfo))))
+    return NULL;
+
+  wi->comment = NULL;
+  wi->dataOffset = wi->format = wi->writing = 0;
+
+  dopen(data, length);
+    
+  if (!readChunk(&ck) ||
+      cmpID(&ck.ckID, RIFF_RiffID) ||
+      !readFourcc(&fourcc) ||
+      cmpID(&fourcc, RIFF_WaveID))
+    Err();
+
+  fileSize = PAD2(ck.ckSize) - sizeof(RIFF_FOURCC);
+
+  while (fileSize >= (AuInt32) sizeof(RiffChunk))
+    {
+      if (!readChunk(&ck))
+	Err();
+
+      fileSize -= sizeof(RiffChunk) + PAD2(ck.ckSize);
+
+      /* LIST chunk */
+      if (!cmpID(&ck.ckID, RIFF_ListID))
+	{
+	  if (!readFourcc(&fourcc))
+	    Err();
+
+	  /* INFO chunk */
+	  if (!cmpID(&fourcc, RIFF_ListInfoID))
+	    {
+	      ck.ckSize -= sizeof(RIFF_FOURCC);
+
+	      while (ck.ckSize)
+		{
+		  RiffChunk       c;
+
+		  if (!readChunk(&c))
+		    Err();
+
+		  /* ICMT chunk */
+		  if (!cmpID(&c.ckID, RIFF_InfoIcmtID))
+		    {
+		      if (!(wi->comment = (Extbyte *) malloc(c.ckSize)) ||
+			  !dread(wi->comment, c.ckSize, 1))
+			Err();
+
+		      if (c.ckSize & 1)
+			dgetc(); /* eat the pad byte */
+		    }
+		  else
+		    /* skip unknown chunk */
+		    dseek(PAD2(c.ckSize), 1);
+
+		  ck.ckSize -= sizeof(RiffChunk) + PAD2(c.ckSize);
+		}
+	    }
+	  else
+	    /* skip unknown chunk */
+	    dseek(PAD2(ck.ckSize) - sizeof(RIFF_FOURCC), 1);
+	}
+      /* wave format chunk */
+      else if (!cmpID(&ck.ckID, RIFF_WaveFmtID) && !wi->format)
+	{
+	  AuInt32            dummy;
+
+	  wi->format = DataReadS(NAS_BIG_ENDIAN);
+	  wi->channels = DataReadS(NAS_BIG_ENDIAN);
+	  wi->sampleRate = DataReadL(NAS_BIG_ENDIAN);
+
+	  /* we don't care about the next two fields */
+	  dummy = DataReadL(NAS_BIG_ENDIAN);
+	  dummy = DataReadS(NAS_BIG_ENDIAN);
+
+	  if (wi->format != RIFF_WAVE_FORMAT_PCM)
+	    Err();
+
+	  wi->bitsPerSample = DataReadS(NAS_BIG_ENDIAN);
+
+	  /* skip any other format specific fields */
+	  dseek(PAD2(ck.ckSize - 16), 1);
+	}
+      /* wave data chunk */
+      else if (!cmpID(&ck.ckID, RIFF_WaveDataID) && !wi->dataOffset)
+	{
+	  long endOfFile;
+
+	  wi->dataOffset = dtell();
+	  wi->dataSize = ck.ckSize;
+	  dseek(0, 2);
+	  endOfFile = dtell();
+
+	  /* seek past the data */
+	  if (dseek(wi->dataOffset + PAD2(ck.ckSize), 0) ||
+	      dtell() > endOfFile)
+	    {
+	      /* the seek failed, assume the size is bogus */
+	      dseek(0, 2);
+	      wi->dataSize = dtell() - wi->dataOffset;
+	    }
+
+	  wi->dataOffset -= sizeof(long);
+	}
+      else
+	/* skip unknown chunk */
+	dseek(PAD2(ck.ckSize), 1);
+    }
+
+  if (!wi->dataOffset)
+    Err();
+
+  wi->numSamples = wi->dataSize / wi->channels / (wi->bitsPerSample >> 3);
+
+  if (!wi->comment)
+    wi->comment = NameFromData (data + wi->dataOffset,
+				length - wi->dataOffset);
+
+  wi->fp = NULL;
+    
+  return wi;
+}
+
+
 static Sound
-SoundOpenDataForReading (unsigned char *data,
+SoundOpenDataForReading (UChar_Binary *data,
 			 int length)
 
 {
   Sound s;
+#if (AudioLibraryVersionMajor >= 2 ) && (AudioLibraryVersionMinor >= 3)
+  SoundFileInfoProc toProc;
+#endif
 
   if (!(s = (Sound) malloc (sizeof (SoundRec))))
     return NULL;
 
-  if ((s->formatInfo = SndOpenDataForReading (data, length))==NULL)
+  if ((s->formatInfo = SndOpenDataForReading ((Char_Binary *) data, length)) != NULL)
     {
-      free (s);
-      return NULL;
-    }
-    
+#if (AudioLibraryVersionMajor >= 2 ) && (AudioLibraryVersionMinor >= 3)
+      if ((toProc = SoundFileGetProc(SoundFileFormatSnd, 
+				     SoundFileInfoProcTo)) == NULL)
+	{
+	  SndCloseFile ((SndInfo *) (s->formatInfo));
+	  free (s);
 
-  if (!(SoundFileInfo[SoundFileFormatSnd].toSound) (s))
+	  return NULL;
+	}
+      if (!((*toProc)(s)))
+#else
+      if (!((int(*)(Sound))(SoundFileInfo[SoundFileFormatSnd].toSound)) (s))
+#endif
+	{
+	  SndCloseFile ((SndInfo *) (s->formatInfo));
+	  free (s);
+	  return NULL;
+	}
+    }
+  else if ((s->formatInfo = WaveOpenDataForReading ((Char_Binary *) data, length)) != NULL)
     {
-      SndCloseFile (s->formatInfo);
-      free (s);
-      return NULL;
+#if (AudioLibraryVersionMajor >= 2 ) && (AudioLibraryVersionMinor >= 3)
+      if ((toProc = SoundFileGetProc(SoundFileFormatWave, 
+				     SoundFileInfoProcTo)) == NULL)
+	{
+	  WaveCloseFile ((WaveInfo *) (s->formatInfo));
+	  free (s);
+
+	  return NULL;
+	}
+      if (!((*toProc)(s)))
+#else
+      if (!((int(*)(Sound))(SoundFileInfo[SoundFileFormatWave].toSound)) (s))
+#endif
+	{
+	  WaveCloseFile ((WaveInfo *) (s->formatInfo));
+	  free (s);
+	  return NULL;
+	}
     }
 
   return s;
 }
-
