@@ -21,32 +21,12 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with:  Not in FSF. */
 
-#ifndef _XEMACS_OBJECTS_TTY_H_
-#define _XEMACS_OBJECTS_TTY_H_
+#ifndef INCLUDED_objects_tty_h_
+#define INCLUDED_objects_tty_h_
 
 #include "objects.h"
 
-struct tty_color_instance_data
-{
-  Lisp_Object symbol; /* so we don't have to constantly call Fintern() */
-};
-
-#define TTY_COLOR_INSTANCE_DATA(c) 				\
-  ((struct tty_color_instance_data *) (c)->data)
-
-#define COLOR_INSTANCE_TTY_SYMBOL(c) (TTY_COLOR_INSTANCE_DATA (c)->symbol)
-
-struct tty_font_instance_data
-{
-  Lisp_Object charset;
-};
-
-#define TTY_FONT_INSTANCE_DATA(c) 				\
-  ((struct tty_font_instance_data *) (c)->data)
-
-#define FONT_INSTANCE_TTY_CHARSET(c) (TTY_FONT_INSTANCE_DATA (c)->charset)
-
 extern Lisp_Object Vtty_color_alist, Vtty_dynamic_color_bg;
 extern Lisp_Object Vtty_dynamic_color_fg;
 
-#endif /* _XEMACS_OBJECTS_TTY_H_ */
+#endif /* INCLUDED_objects_tty_h_ */
