@@ -2,7 +2,7 @@
    Copyright (C) 1985, 1986, 1987, 1988, 1992, 1993, 1994, 1995
    Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995, 1996, 2000 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -23,24 +23,20 @@ Boston, MA 02111-1307, USA.  */
 
 /* Written by Kirill M. Katsnelson <kkm@kis.ru>, April 1998 */
 
+/* Mule-ized as of 8-6-00 */
+
 #include <config.h>
 #include "lisp.h"
 
-#include "buffer.h"
 #include "console-msw.h"
 #include "hash.h"
 #include "lstream.h"
-#include "nt.h"
 #include "process.h"
 #include "procimpl.h"
-#include "sysdep.h"
 
-#include <shellapi.h>
-#include <errno.h>
-#include <signal.h>
-#ifdef HAVE_SOCKETS
-#include <winsock.h>
-#endif
+#include "sysfile.h"
+#include "sysproc.h"
+#include "syssignal.h"
 
 /* Bound by win32-native.el */
 Lisp_Object Qmswindows_construct_process_command_line;
@@ -54,6 +50,7 @@ struct nt_process_data
   HANDLE h_process;
   DWORD dwProcessId;
   HWND hwnd; /* console window */
+  int selected_for_exit_notify;
 };
 
 /* Control whether create_child causes the process to inherit Emacs'
@@ -67,14 +64,29 @@ Lisp_Object Vmswindows_start_process_share_console;
    subprocesses blocking when accessing unmounted drives.  */
 Lisp_Object Vmswindows_start_process_inherit_error_mode;
 
-#define NT_DATA(p) ((struct nt_process_data*)((p)->process_data))
+#define NT_DATA(p) ((struct nt_process_data *)((p)->process_data))
 
 /*-----------------------------------------------------------------------*/
 /* Process helpers							 */
 /*-----------------------------------------------------------------------*/
 
-/* This one breaks process abstraction. Prototype is in console-msw.h,
-   used by select_process method in event-msw.c */
+/* These break process abstraction. Prototypes in console-msw.h,
+   used by select_process method in event-msw.c.
+
+   If called the first time on a process, return the process handle, so we
+   can select on it and receive exit notification.  "First time only" so we
+   don't select the same process multiple times if someone turns off and on
+   the receipt of process data. */
+
+HANDLE
+get_nt_process_handle_only_first_time (Lisp_Process *p)
+{
+  if (NT_DATA (p)->selected_for_exit_notify)
+    return INVALID_HANDLE_VALUE;
+  NT_DATA (p)->selected_for_exit_notify = 1;
+  return (NT_DATA (p)->h_process);
+}
+
 HANDLE
 get_nt_process_handle (Lisp_Process *p)
 {
@@ -115,25 +127,25 @@ typedef struct
  * The memory in other process is allocated by creating a suspended
  * thread. Initial stack of that thread is used as the memory
  * block. The thread entry point is the routine ExitThread in
- * kernel32.dll, so the allocated memory is freed just by resuming the 
+ * kernel32.dll, so the allocated memory is freed just by resuming the
  * thread, which immediately terminates after that.
  */
 
-static int 
-alloc_process_memory (HANDLE h_process, size_t size,
-		      process_memory* pmc)
+static int
+alloc_process_memory (HANDLE h_process, Bytecount size,
+		      process_memory *pmc)
 {
   LPTHREAD_START_ROUTINE adr_ExitThread =
     (LPTHREAD_START_ROUTINE)
-    GetProcAddress (GetModuleHandle ("kernel32"), "ExitThread");
+    GetProcAddress (qxeGetModuleHandle (XETEXT ("kernel32")), "ExitThread");
   DWORD dw_unused;
   CONTEXT context;
   MEMORY_BASIC_INFORMATION mbi;
 
   pmc->h_process = h_process;
   pmc->h_thread = CreateRemoteThread (h_process, NULL, size,
-				     adr_ExitThread, NULL,
-				     CREATE_SUSPENDED, &dw_unused);
+				      adr_ExitThread, NULL,
+				      CREATE_SUSPENDED, &dw_unused);
   if (pmc->h_thread == NULL)
     return 0;
 
@@ -170,7 +182,7 @@ alloc_process_memory (HANDLE h_process, size_t size,
 }
 
 static void
-free_process_memory (process_memory* pmc)
+free_process_memory (process_memory *pmc)
 {
   ResumeThread (pmc->h_thread);
 }
@@ -192,12 +204,12 @@ free_process_memory (process_memory* pmc)
 static DWORD
 run_in_other_process (HANDLE h_process,
 		      LPTHREAD_START_ROUTINE routine,
-		      LPVOID data, size_t data_size)
+		      LPVOID data, Bytecount data_size)
 {
   process_memory pm;
-  const size_t code_size = FRAGMENT_CODE_SIZE;
+  const Bytecount code_size = FRAGMENT_CODE_SIZE;
   /* Need at most 3 extra bytes of memory, for data alignment */
-  size_t total_size = code_size + data_size + 3;
+  Bytecount total_size = code_size + data_size + 3;
   LPVOID remote_data;
   HANDLE h_thread;
   DWORD dw_unused;
@@ -275,7 +287,7 @@ typedef struct
 } sigkill_data;
 
 static DWORD WINAPI
-sigkill_proc (sigkill_data* data)
+sigkill_proc (sigkill_data *data)
 {
   (*data->adr_ExitProcess)(255);
   return 1;
@@ -291,7 +303,7 @@ typedef struct
 } sigint_data;
 
 static DWORD WINAPI
-sigint_proc (sigint_data* data)
+sigint_proc (sigint_data *data)
 {
   return (*data->adr_GenerateConsoleCtrlEvent) (data->event, 0);
 }
@@ -305,7 +317,7 @@ typedef struct
 } sig_enable_data;
 
 static DWORD WINAPI
-sig_enable_proc (sig_enable_data* data)
+sig_enable_proc (sig_enable_data *data)
 {
   (*data->adr_SetConsoleCtrlHandler) (NULL, FALSE);
   return 1;
@@ -320,12 +332,12 @@ static int
 send_signal_the_nt_way (struct nt_process_data *cp, int pid, int signo)
 {
   HANDLE h_process;
-  HMODULE h_kernel = GetModuleHandle ("kernel32");
+  HMODULE h_kernel = qxeGetModuleHandle (XETEXT ("kernel32"));
   int close_process = 0;
   DWORD retval;
-  
+
   assert (h_kernel != NULL);
-  
+
   if (cp)
     {
       pid = cp->dwProcessId;
@@ -336,7 +348,7 @@ send_signal_the_nt_way (struct nt_process_data *cp, int pid, int signo)
       close_process = 1;
       /* Try to open the process with required privileges */
       h_process = OpenProcess (PROCESS_CREATE_THREAD
-			       | PROCESS_QUERY_INFORMATION 
+			       | PROCESS_QUERY_INFORMATION
 			       | PROCESS_VM_OPERATION
 			       | PROCESS_VM_WRITE,
 			       FALSE, pid);
@@ -356,8 +368,8 @@ send_signal_the_nt_way (struct nt_process_data *cp, int pid, int signo)
 	d.adr_ExitProcess =
 	  (void (WINAPI *) (UINT)) GetProcAddress (h_kernel, "ExitProcess");
 	assert (d.adr_ExitProcess);
-	retval = run_in_other_process (h_process, 
-				       (LPTHREAD_START_ROUTINE)sigkill_proc,
+	retval = run_in_other_process (h_process,
+				       (LPTHREAD_START_ROUTINE) sigkill_proc,
 				       &d, sizeof (d));
 	break;
       }
@@ -369,8 +381,8 @@ send_signal_the_nt_way (struct nt_process_data *cp, int pid, int signo)
 	  GetProcAddress (h_kernel, "GenerateConsoleCtrlEvent");
 	assert (d.adr_GenerateConsoleCtrlEvent);
 	d.event = CTRL_C_EVENT;
-	retval = run_in_other_process (h_process, 
-				       (LPTHREAD_START_ROUTINE)sigint_proc,
+	retval = run_in_other_process (h_process,
+				       (LPTHREAD_START_ROUTINE) sigint_proc,
 				       &d, sizeof (d));
 	break;
       }
@@ -389,9 +401,9 @@ send_signal_the_nt_way (struct nt_process_data *cp, int pid, int signo)
 static void
 enable_child_signals (HANDLE h_process)
 {
-  HMODULE h_kernel = GetModuleHandle ("kernel32");
+  HMODULE h_kernel = qxeGetModuleHandle (XETEXT ("kernel32"));
   sig_enable_data d;
-  
+
   assert (h_kernel != NULL);
   d.adr_SetConsoleCtrlHandler =
     (BOOL (WINAPI *) (LPVOID, BOOL))
@@ -400,8 +412,6 @@ enable_child_signals (HANDLE h_process)
   run_in_other_process (h_process, (LPTHREAD_START_ROUTINE)sig_enable_proc,
 			&d, sizeof (d));
 }
-  
-#pragma warning (default : 4113)
 
 /* ---------------------------- the 95 way ------------------------------- */
 
@@ -415,11 +425,12 @@ find_child_console (HWND hwnd, long putada)
   thread_id = GetWindowThreadProcessId (hwnd, &process_id);
   if (process_id == cp->dwProcessId)
     {
-      char window_class[32];
+      Extbyte window_class[32];
 
-      GetClassName (hwnd, window_class, sizeof (window_class));
+      /* GetClassNameA to avoid problems with Unicode return values */
+      GetClassNameA (hwnd, window_class, sizeof (window_class));
       if (strcmp (window_class,
-		  mswindows_windows9x_p ()
+		  mswindows_windows9x_p
 		  ? "tty"
 		  : "ConsoleWindowClass") == 0)
 	{
@@ -437,7 +448,7 @@ send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
   HANDLE h_process;
   int close_process = 0;
   int rc = 1;
-  
+
   if (cp)
     {
       pid = cp->dwProcessId;
@@ -454,21 +465,21 @@ send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
       if (!h_process)
 	return 0;
     }
-    
+
   if (signo == SIGINT)
     {
       if (NILP (Vmswindows_start_process_share_console) && cp && cp->hwnd)
 	{
-	  BYTE control_scan_code = (BYTE) MapVirtualKey (VK_CONTROL, 0);
+	  BYTE control_scan_code = (BYTE) MapVirtualKeyA (VK_CONTROL, 0);
 	  BYTE vk_break_code = VK_CANCEL;
-	  BYTE break_scan_code = (BYTE) MapVirtualKey (vk_break_code, 0);
+	  BYTE break_scan_code = (BYTE) MapVirtualKeyA (vk_break_code, 0);
 	  HWND foreground_window;
 
 	  if (break_scan_code == 0)
 	    {
 	      /* Fake Ctrl-C if we can't manage Ctrl-Break. */
 	      vk_break_code = 'C';
-	      break_scan_code = (BYTE) MapVirtualKey (vk_break_code, 0);
+	      break_scan_code = (BYTE) MapVirtualKeyA (vk_break_code, 0);
 	    }
 
 	  foreground_window = GetForegroundWindow ();
@@ -540,7 +551,7 @@ send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
       if (NILP (Vmswindows_start_process_share_console) && cp && cp->hwnd)
 	{
 #if 1
-	  if (mswindows_windows9x_p ())
+	  if (mswindows_windows9x_p)
 	    {
 /*
    Another possibility is to try terminating the VDM out-right by
@@ -567,7 +578,7 @@ send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
 		 than ideal in that subprocesses tend to stick around
 		 until the machine is shutdown, but at least it
 		 doesn't freeze the 16-bit subsystem.  */
-	      PostMessage (cp->hwnd, WM_QUIT, 0xff, 0);
+	      qxePostMessage (cp->hwnd, WM_QUIT, 0xff, 0);
 #endif
 	      if (!TerminateProcess (h_process, 0xff))
 		{
@@ -581,7 +592,7 @@ send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
 	    }
 	  else
 #endif
-	    PostMessage (cp->hwnd, WM_CLOSE, 0, 0);
+	    qxePostMessage (cp->hwnd, WM_CLOSE, 0, 0);
 	}
       /* Kill the process.  On W32 this doesn't kill child processes
 	 so it doesn't work very well for shells which is why it's not
@@ -608,7 +619,7 @@ send_signal_the_95_way (struct nt_process_data *cp, int pid, int signo)
 static int
 send_signal (struct nt_process_data *cp, int pid, int signo)
 {
-  return (!mswindows_windows9x_p () && send_signal_the_nt_way (cp, pid, signo))
+  return (!mswindows_windows9x_p && send_signal_the_nt_way (cp, pid, signo))
     || send_signal_the_95_way (cp, pid, signo);
 }
 
@@ -621,9 +632,9 @@ validate_signal_number (int signo)
   if (signo != SIGKILL && signo != SIGTERM
       && signo != SIGQUIT && signo != SIGINT
       && signo != SIGHUP)
-    invalid_argument ("Signal number not supported", make_int (signo));
+    invalid_constant ("Signal number not supported", make_int (signo));
 }
-  
+
 /*-----------------------------------------------------------------------*/
 /* Process methods							 */
 /*-----------------------------------------------------------------------*/
@@ -671,24 +682,26 @@ nt_init_process (void)
  * must signal an error instead.
  */
 
-static void
-signal_cannot_launch (Lisp_Object image_file, DWORD err)
+static DOESNT_RETURN
+mswindows_report_winsock_error (const char *string, Lisp_Object data,
+				int errnum)
 {
-  mswindows_set_errno (err);
-  report_file_error ("Error starting", image_file);
+  report_file_type_error (Qnetwork_error, mswindows_lisp_error (errnum),
+			  string, data);
 }
 
 static void
 ensure_console_window_exists (void)
 {
-  if (mswindows_windows9x_p ())
+  if (mswindows_windows9x_p)
     mswindows_hide_console ();
 }
 
 int
-compare_env (const void *strp1, const void *strp2)
+mswindows_compare_env (const void *strp1, const void *strp2)
 {
-  const char *str1 = *(const char**)strp1, *str2 = *(const char**)strp2;
+  const Ibyte *str1 = *(const Ibyte **)strp1,
+    *str2 = *(const Ibyte **)strp2;
 
   while (*str1 && *str2 && *str1 != '=' && *str2 != '=')
     {
@@ -707,73 +720,87 @@ compare_env (const void *strp1, const void *strp2)
     return 1;
 }
 
+/*
+ * Fork off a subprocess. P is a pointer to newly created subprocess
+ * object. If this function signals, the caller is responsible for
+ * deleting (and finalizing) the process object.
+ *
+ * The method must return PID of the new process, a (positive??? ####) number
+ * which fits into Lisp_Int. No return value indicates an error, the method
+ * must signal an error instead.
+ */
+
 static int
 nt_create_process (Lisp_Process *p,
 		   Lisp_Object *argv, int nargv,
-		   Lisp_Object program, Lisp_Object cur_dir)
+		   Lisp_Object program, Lisp_Object cur_dir,
+		   int separate_err)
 {
   /* Synched up with sys_spawnve in FSF 20.6.  Significantly different
      but still synchable. */
-  HANDLE hmyshove, hmyslurp, hprocin, hprocout, hprocerr;
+  HANDLE hmyshove, hmyslurp, hmyslurp_err, hprocin, hprocout, hprocerr;
   Extbyte *command_line;
   BOOL do_io, windowed;
-  char *proc_env;
+  Extbyte *proc_env;
 
   /* No need to DOS-ize the filename; expand-file-name (called prior)
      already does this. */
 
   /* Find out whether the application is windowed or not */
-  if (xSHGetFileInfoA)
-    {
-      /* SHGetFileInfo tends to return ERROR_FILE_NOT_FOUND on most
-	 errors. This leads to bogus error message. */
-      DWORD image_type;
-      char *p = strrchr ((char *)XSTRING_DATA (program), '.');
-      if (p != NULL &&
-	  (stricmp (p, ".exe") == 0 ||
-	   stricmp (p, ".com") == 0 ||
-	   stricmp (p, ".bat") == 0 ||
-	   stricmp (p, ".cmd") == 0))
-	{
-	  image_type = xSHGetFileInfoA ((char *)XSTRING_DATA (program), 0,NULL,
-					0, SHGFI_EXETYPE);
-	}
-      else
-	{
-	  char progname[MAX_PATH];
-	  sprintf (progname, "%s.exe", (char *)XSTRING_DATA (program));
-	  image_type = xSHGetFileInfoA (progname, 0, NULL, 0, SHGFI_EXETYPE);
-	}
-      if (image_type == 0)
-	signal_cannot_launch (program, (GetLastError () == ERROR_FILE_NOT_FOUND
-					? ERROR_BAD_FORMAT : GetLastError ()));
-      windowed = HIWORD (image_type) != 0;
-    }
-  else /* NT 3.5; we have no idea so just guess. */
-    windowed = 0;
+  {
+    /* SHGetFileInfo tends to return ERROR_FILE_NOT_FOUND on most
+       errors. This leads to bogus error message. */
+    DWORD image_type;
+    if (mswindows_is_executable (XSTRING_DATA (program)))
+      {
+	Extbyte *progext;
+	LISP_STRING_TO_TSTR (program, progext);
+	image_type = qxeSHGetFileInfo (progext, 0, NULL, 0, SHGFI_EXETYPE);
+      }
+    else
+      {
+	DECLARE_EISTRING (progext);
+	eicpy_lstr (progext, program);
+	eicat_c (progext, ".exe");
+	eito_external (progext, Qmswindows_tstr);
+	image_type = qxeSHGetFileInfo (eiextdata (progext), 0, NULL, 0,
+				       SHGFI_EXETYPE);
+      }
+    if (image_type == 0)
+      mswindows_report_process_error
+	("Determining executable file type",
+	 program,
+	 GetLastError () == ERROR_FILE_NOT_FOUND
+	 ? ERROR_BAD_FORMAT : GetLastError ());
+    windowed = HIWORD (image_type) != 0;
+  }
+
 
   /* Decide whether to do I/O on process handles, or just mark the
      process exited immediately upon successful launching. We do I/O if the
      process is a console one, or if it is windowed but windowed_process_io
      is non-zero */
   do_io = !windowed || windowed_process_io ;
-  
+
   if (do_io)
     {
       /* Create two unidirectional named pipes */
       HANDLE htmp;
       SECURITY_ATTRIBUTES sa;
 
-      sa.nLength = sizeof(sa);
+      sa.nLength = sizeof (sa);
       sa.bInheritHandle = TRUE;
       sa.lpSecurityDescriptor = NULL;
 
       CreatePipe (&hprocin, &hmyshove, &sa, 0);
       CreatePipe (&hmyslurp, &hprocout, &sa, 0);
 
-      /* Duplicate the stdout handle for use as stderr */
-      DuplicateHandle(GetCurrentProcess(), hprocout, GetCurrentProcess(),
-                      &hprocerr, 0, TRUE, DUPLICATE_SAME_ACCESS);
+      if (separate_err)
+	CreatePipe (&hmyslurp_err, &hprocerr, &sa, 0);
+      else
+	/* Duplicate the stdout handle for use as stderr */
+	DuplicateHandle(GetCurrentProcess(), hprocout, GetCurrentProcess(),
+			&hprocerr, 0, TRUE, DUPLICATE_SAME_ACCESS);
 
       /* Stupid Win32 allows to create a pipe with *both* ends either
 	 inheritable or not. We need process ends inheritable, and local
@@ -786,6 +813,13 @@ nt_create_process (Lisp_Process *p,
 		       &htmp, 0, FALSE,
 		       DUPLICATE_CLOSE_SOURCE | DUPLICATE_SAME_ACCESS);
       hmyslurp = htmp;
+      if (separate_err)
+	{
+	  DuplicateHandle (GetCurrentProcess(), hmyslurp_err,
+			   GetCurrentProcess(), &htmp, 0, FALSE,
+			   DUPLICATE_CLOSE_SOURCE | DUPLICATE_SAME_ACCESS);
+	  hmyslurp_err = htmp;
+	}
     }
 
   /* Convert an argv vector into Win32 style command line by a call to
@@ -812,21 +846,20 @@ nt_create_process (Lisp_Process *p,
 	("Bogus return value from `mswindows-construct-process-command-line'",
 	 args_or_ret);
 
-    LISP_STRING_TO_EXTERNAL (args_or_ret, command_line, Qmswindows_tstr);
+    LISP_STRING_TO_TSTR (args_or_ret, command_line);
 
     UNGCPRO; /* args_or_ret */
   }
 
   /* Set `proc_env' to a nul-separated array of the strings in
      Vprocess_environment terminated by 2 nuls.  */
- 
+
   {
-    char **env;
+    Ibyte **env;
     REGISTER Lisp_Object tem;
-    REGISTER char **new_env;
-    REGISTER int new_length = 0, i, new_space;
-    char *penv;
-    
+    REGISTER Ibyte **new_env;
+    REGISTER int new_length = 0, i;
+
     for (tem = Vprocess_environment;
  	 (CONSP (tem)
  	  && STRINGP (XCAR (tem)));
@@ -835,33 +868,33 @@ nt_create_process (Lisp_Process *p,
 
     /* FSF adds an extra env var to hold the current process ID of the
        Emacs process.  Apparently this is used only by emacsserver.c,
-       which we have superseded to gnuserv.c. (#### Does it work under
+       which we have superseded by gnuserv.c. (#### Does it work under
        MS Windows?)
 
-       sprintf (ppid_env_var_buffer, "EM_PARENT_PROCESS_ID=%d", 
+       sprintf (ppid_env_var_buffer, "EM_PARENT_PROCESS_ID=%d",
          GetCurrentProcessId ());
        arglen += strlen (ppid_env_var_buffer) + 1;
        numenv++;
     */
-    
+
     /* new_length + 1 to include terminating 0.  */
-    env = new_env = alloca_array (char *, new_length + 1);
- 
+    env = new_env = alloca_array (Ibyte *, new_length + 1);
+
     /* Copy the Vprocess_environment strings into new_env.  */
     for (tem = Vprocess_environment;
  	 (CONSP (tem)
  	  && STRINGP (XCAR (tem)));
  	 tem = XCDR (tem))
       {
-	char **ep = env;
-	char *string = (char *) XSTRING_DATA (XCAR (tem));
+	Ibyte **ep = env;
+	Ibyte *string = XSTRING_DATA (XCAR (tem));
 	/* See if this string duplicates any string already in the env.
 	   If so, don't put it in.
 	   When an env var has multiple definitions,
 	   we keep the definition that comes first in process-environment.  */
 	for (; ep != new_env; ep++)
 	  {
-	    char *p = *ep, *q = string;
+	    Ibyte *p = *ep, *q = string;
 	    while (1)
 	      {
 		if (*q == 0)
@@ -878,27 +911,24 @@ nt_create_process (Lisp_Process *p,
       duplicate: ;
       }
     *new_env = 0;
-    
+
     /* Sort the environment variables */
     new_length = new_env - env;
-    qsort (env, new_length, sizeof (char *), compare_env);
-    
-    /* Work out how much space to allocate */
-    new_space = 0;
-    for (i = 0; i < new_length; i++)
-      {
- 	new_space += strlen(env[i]) + 1;
-      }
-    new_space++;
-    
-    /* Allocate space and copy variables into it */
-    penv = proc_env = (char*) alloca(new_space);
-    for (i = 0; i < new_length; i++)
-      {
- 	strcpy(penv, env[i]);
- 	penv += strlen(env[i]) + 1;
-      }
-    *penv = 0;
+    qsort (env, new_length, sizeof (Ibyte *), mswindows_compare_env);
+
+    {
+      DECLARE_EISTRING (envout);
+
+      for (i = 0; i < new_length; i++)
+	{
+	  eicat_raw (envout, env[i], qxestrlen (env[i]));
+	  eicat_raw (envout, (Ibyte *) "\0", 1);
+	}
+
+      eicat_raw (envout, (Ibyte *) "\0", 1);
+      eito_external (envout, Qmswindows_tstr);
+      proc_env = eiextdata (envout);
+    }
   }
 
 #if 0
@@ -908,20 +938,20 @@ nt_create_process (Lisp_Process *p,
        while leaving the real app name as argv[0].  */
     if (is_dos_app)
       {
-	cmdname = (char*) alloca (MAXPATHLEN);
+	cmdname = (Ibyte *) ALLOCA (PATH_MAX);
 	if (egetenv ("CMDPROXY"))
-	  strcpy ((char*)cmdname, egetenv ("CMDPROXY"));
+	  qxestrcpy (cmdname, egetenv ("CMDPROXY"));
 	else
 	  {
-	    strcpy ((char*)cmdname, XSTRING_DATA (Vinvocation_directory));
-	    strcat ((char*)cmdname, "cmdproxy.exe");
+	    qxestrcpy (cmdname, XSTRING_DATA (Vinvocation_directory));
+	    qxestrcat (cmdname, (Ibyte *) "cmdproxy.exe");
 	  }
       }
 #endif
-  
+
   /* Create process */
   {
-    STARTUPINFO si;
+    STARTUPINFOW si;
     PROCESS_INFORMATION pi;
     DWORD err;
     DWORD flags;
@@ -938,7 +968,7 @@ nt_create_process (Lisp_Process *p,
       }
 
     flags = CREATE_SUSPENDED;
-    if (mswindows_windows9x_p ())
+    if (mswindows_windows9x_p)
       flags |= (!NILP (Vmswindows_start_process_share_console)
 		? CREATE_NEW_PROCESS_GROUP
 		: CREATE_NEW_CONSOLE);
@@ -949,9 +979,18 @@ nt_create_process (Lisp_Process *p,
 
     ensure_console_window_exists ();
 
-    err = (CreateProcess (NULL, command_line, NULL, NULL, TRUE, flags,
-			  proc_env, (char *) XSTRING_DATA (cur_dir), &si, &pi)
-	   ? 0 : GetLastError ());
+    {
+      Extbyte *curdirext;
+
+      LISP_STRING_TO_TSTR (cur_dir, curdirext);
+
+      err = (qxeCreateProcess (NULL, command_line, NULL, NULL, TRUE,
+			       (XEUNICODE_P ?
+				flags | CREATE_UNICODE_ENVIRONMENT :
+				flags), proc_env,
+			       curdirext, &si, &pi)
+	     ? 0 : GetLastError ());
+    }
 
     if (do_io)
       {
@@ -960,7 +999,7 @@ nt_create_process (Lisp_Process *p,
 	CloseHandle (hprocout);
 	CloseHandle (hprocerr);
       }
-    
+
     /* Handle process creation failure */
     if (err)
       {
@@ -968,8 +1007,12 @@ nt_create_process (Lisp_Process *p,
 	  {
 	    CloseHandle (hmyshove);
 	    CloseHandle (hmyslurp);
+	    if (separate_err)
+	      CloseHandle (hmyslurp_err);
 	  }
-	signal_cannot_launch (program, GetLastError ());
+	mswindows_report_process_error
+	  ("Error starting",
+	   program, GetLastError ());
       }
 
     /* The process started successfully */
@@ -977,7 +1020,9 @@ nt_create_process (Lisp_Process *p,
       {
 	NT_DATA(p)->h_process = pi.hProcess;
 	NT_DATA(p)->dwProcessId = pi.dwProcessId;
-	init_process_io_handles (p, (void*)hmyslurp, (void*)hmyshove, 0);
+	init_process_io_handles (p, (void *) hmyslurp, (void *) hmyshove,
+				 separate_err ? (void *) hmyslurp_err
+				 : (void *) -1, 0);
       }
     else
       {
@@ -996,16 +1041,16 @@ nt_create_process (Lisp_Process *p,
   }
 }
 
-/* 
+/*
  * This method is called to update status fields of the process
  * structure. If the process has not existed, this method is expected
  * to do nothing.
  *
- * The method is called only for real child processes.  
+ * The method is called only for real child processes.
  */
 
 static void
-nt_update_status_if_terminated (Lisp_Process* p)
+nt_update_status_if_terminated (Lisp_Process *p)
 {
   DWORD exit_code;
   if (GetExitCodeProcess (NT_DATA(p)->h_process, &exit_code)
@@ -1037,7 +1082,7 @@ nt_update_status_if_terminated (Lisp_Process* p)
    unix_send_process... */
 
 static void
-nt_send_process (Lisp_Object proc, struct lstream* lstream)
+nt_send_process (Lisp_Object proc, struct lstream *lstream)
 {
   volatile Lisp_Object vol_proc = proc;
   Lisp_Process *volatile p = XPROCESS (proc);
@@ -1045,12 +1090,12 @@ nt_send_process (Lisp_Object proc, struct lstream* lstream)
   /* use a reasonable-sized buffer (somewhere around the size of the
      stream buffer) so as to avoid inundating the stream with blocked
      data. */
-  Bufbyte chunkbuf[512];
+  Ibyte chunkbuf[512];
   Bytecount chunklen;
 
   while (1)
     {
-      Lstream_data_count writeret;
+      int writeret;
 
       chunklen = Lstream_read (lstream, chunkbuf, 512);
       if (chunklen <= 0)
@@ -1060,9 +1105,9 @@ nt_send_process (Lisp_Object proc, struct lstream* lstream)
       /* Lstream_write() will never successfully write less than the
 	 amount sent in.  In the worst case, it just buffers the
 	 unwritten data. */
-      writeret = Lstream_write (XLSTREAM (DATA_OUTSTREAM(p)), chunkbuf,
+      writeret = Lstream_write (XLSTREAM (DATA_OUTSTREAM (p)), chunkbuf,
 				chunklen);
-      Lstream_flush (XLSTREAM (DATA_OUTSTREAM(p)));
+      Lstream_flush (XLSTREAM (DATA_OUTSTREAM (p)));
       if (writeret < 0)
 	{
 	  p->status_symbol = Qexit;
@@ -1146,37 +1191,51 @@ nt_kill_process_by_pid (int pid, int signo)
 #define SOCK_TIMER_ID 666
 #define XM_SOCKREPLY (WM_USER + 666)
 
+/* Return 0 for success, or error code */
+
 static int
-get_internet_address (Lisp_Object host, struct sockaddr_in *address,
-		      Error_behavior errb)
+get_internet_address (Lisp_Object host, struct sockaddr_in *address)
 {
-  char buf [MAXGETHOSTSTRUCT];
+  Char_Binary buf[MAXGETHOSTSTRUCT];
   HWND hwnd;
   HANDLE hasync;
-  int success = 0;
+  int errcode = 0;
 
   address->sin_family = AF_INET;
 
   /* First check if HOST is already a numeric address */
   {
-    unsigned long inaddr = inet_addr (XSTRING_DATA (host));
+    Extbyte *hostext;
+    unsigned long inaddr;
+
+    LISP_STRING_TO_EXTERNAL (host, hostext, Qmswindows_host_name_encoding);
+    inaddr = inet_addr (hostext);
     if (inaddr != INADDR_NONE)
       {
 	address->sin_addr.s_addr = inaddr;
-	return 1;
+	return 0;
       }
   }
 
   /* Create a window which will receive completion messages */
-  hwnd = CreateWindow ("STATIC", NULL, WS_OVERLAPPED, 0, 0, 1, 1,
-		       NULL, NULL, NULL, NULL);
+  hwnd = qxeCreateWindow (XETEXT ("STATIC"), NULL, WS_OVERLAPPED, 0, 0, 1, 1,
+			  NULL, NULL, NULL, NULL);
   assert (hwnd);
 
   /* Post name resolution request */
-  hasync = WSAAsyncGetHostByName (hwnd, XM_SOCKREPLY, XSTRING_DATA (host),
-				  buf, sizeof (buf));
-  if (hasync == NULL)
-    goto done;
+  {
+    Extbyte *hostext;
+
+    LISP_STRING_TO_EXTERNAL (host, hostext, Qmswindows_host_name_encoding);
+
+    hasync = WSAAsyncGetHostByName (hwnd, XM_SOCKREPLY, hostext,
+				    buf, sizeof (buf));
+    if (hasync == NULL)
+      {
+	errcode = WSAGetLastError ();
+	goto done;
+      }
+  }
 
   /* Set a timer to poll for quit every 250 ms */
   SetTimer (hwnd, SOCK_TIMER_ID, 250, NULL);
@@ -1184,18 +1243,11 @@ get_internet_address (Lisp_Object host, struct sockaddr_in *address,
   while (1)
     {
       MSG msg;
-      GetMessage (&msg, hwnd, 0, 0);
+      qxeGetMessage (&msg, hwnd, 0, 0);
       if (msg.message == XM_SOCKREPLY)
 	{
 	  /* Ok, got an answer */
-	  if (WSAGETASYNCERROR(msg.lParam) == NO_ERROR)
-	    success = 1;
-	  else
-	    {
-	      warn_when_safe(Qstream, Qwarning,
-			     "cannot get IP address for host \"%s\"",
-			     XSTRING_DATA (host));
-	    }
+	  errcode = WSAGETASYNCERROR (msg.lParam);
 	  goto done;
 	}
       else if (msg.message == WM_TIMER && msg.wParam == SOCK_TIMER_ID)
@@ -1205,22 +1257,22 @@ get_internet_address (Lisp_Object host, struct sockaddr_in *address,
 	      WSACancelAsyncRequest (hasync);
 	      KillTimer (hwnd, SOCK_TIMER_ID);
 	      DestroyWindow (hwnd);
-	      REALLY_QUIT;
+	      QUIT;
 	    }
 	}
-      DispatchMessage (&msg);
+      qxeDispatchMessage (&msg);
     }
 
  done:
   KillTimer (hwnd, SOCK_TIMER_ID);
   DestroyWindow (hwnd);
-  if (success)
+  if (!errcode)
     {
       /* BUF starts with struct hostent */
-      struct hostent* he = (struct hostent*) buf;
-      address->sin_addr.s_addr = *(unsigned long*)he->h_addr_list[0];
+      struct hostent *he = (struct hostent *) buf;
+      address->sin_addr.s_addr = * (unsigned long *) he->h_addr_list[0];
     }
-  return success;
+  return errcode;
 }
 
 static Lisp_Object
@@ -1228,7 +1280,7 @@ nt_canonicalize_host_name (Lisp_Object host)
 {
   struct sockaddr_in address;
 
-  if (!get_internet_address (host, &address, ERROR_ME_NOT))
+  if (get_internet_address (host, &address)) /* error */
     return host;
 
   if (address.sin_family == AF_INET)
@@ -1246,107 +1298,252 @@ nt_canonicalize_host_name (Lisp_Object host)
 static void
 nt_open_network_stream (Lisp_Object name, Lisp_Object host,
 			Lisp_Object service,
-			Lisp_Object protocol, void** vinfd, void** voutfd)
+			Lisp_Object protocol, void **vinfd, void **voutfd)
 {
-  /* !!#### not Mule-ized */
   struct sockaddr_in address;
   SOCKET s;
   int port;
   int retval;
+  int errnum;
 
   CHECK_STRING (host);
 
   if (!EQ (protocol, Qtcp))
-    invalid_argument ("Unsupported protocol", protocol);
+    invalid_constant ("Unsupported protocol", protocol);
 
   if (INTP (service))
     port = htons ((unsigned short) XINT (service));
   else
     {
       struct servent *svc_info;
+      Extbyte *servext;
+
       CHECK_STRING (service);
-      svc_info = getservbyname ((char *) XSTRING_DATA (service), "tcp");
+      LISP_STRING_TO_EXTERNAL (service, servext,
+			       Qmswindows_service_name_encoding);
+
+      svc_info = getservbyname (servext, "tcp");
       if (svc_info == 0)
 	invalid_argument ("Unknown service", service);
       port = svc_info->s_port;
     }
 
-  get_internet_address (host, &address, ERROR_ME);
+  retval = get_internet_address (host, &address);
+  if (retval)
+    mswindows_report_winsock_error ("Getting IP address", host,
+				    retval);
   address.sin_port = port;
 
   s = socket (address.sin_family, SOCK_STREAM, 0);
   if (s < 0)
-    report_file_error ("error creating socket", list1 (name));
+    mswindows_report_winsock_error ("Creating socket", name,
+				    WSAGetLastError ());
 
   /* We don't want to be blocked on connect */
   {
     unsigned long nonblock = 1;
     ioctlsocket (s, FIONBIO, &nonblock);
   }
-  
+
   retval = connect (s, (struct sockaddr *) &address, sizeof (address));
   if (retval != NO_ERROR && WSAGetLastError() != WSAEWOULDBLOCK)
-    goto connect_failed;
+    {
+      errnum = WSAGetLastError ();
+      goto connect_failed;
+    }
+
+#if 0 /* PUTA! I thought getsockopt() was failing, so I created the
+	 following based on the code in get_internet_address(), but
+	 it was my own fault down below.  Both versions should work. */
   /* Wait while connection is established */
+  {
+    HWND hwnd;
+
+  /* Create a window which will receive completion messages */
+    hwnd = qxeCreateWindow (XETEXT ("STATIC"), NULL, WS_OVERLAPPED, 0, 0, 1, 1,
+			    NULL, NULL, NULL, NULL);
+    assert (hwnd);
+
+    /* Post request */
+    if (WSAAsyncSelect (s, hwnd, XM_SOCKREPLY, FD_CONNECT))
+      {
+	errnum = WSAGetLastError ();
+	goto done;
+      }
+
+    /* Set a timer to poll for quit every 250 ms */
+    SetTimer (hwnd, SOCK_TIMER_ID, 250, NULL);
+
+    while (1)
+      {
+	MSG msg;
+	GetMessage (&msg, hwnd, 0, 0);
+	if (msg.message == XM_SOCKREPLY)
+	  {
+	    /* Ok, got an answer */
+	    errnum = WSAGETASYNCERROR (msg.lParam);
+	    goto done;
+	  }
+
+	else if (msg.message == WM_TIMER && msg.wParam == SOCK_TIMER_ID)
+	  {
+	    if (QUITP)
+	      {
+		WSAAsyncSelect (s, hwnd, XM_SOCKREPLY, 0);
+		KillTimer (hwnd, SOCK_TIMER_ID);
+		DestroyWindow (hwnd);
+		QUIT;
+	      }
+	  }
+	DispatchMessage (&msg);
+      }
+
+  done:
+    WSAAsyncSelect (s, hwnd, XM_SOCKREPLY, 0);
+    KillTimer (hwnd, SOCK_TIMER_ID);
+    DestroyWindow (hwnd);
+    if (errnum)
+      goto connect_failed;
+  }
+
+#else
   while (1)
     {
-      fd_set fdset;
+      fd_set fdwriteset, fdexceptset;
       struct timeval tv;
       int nsel;
 
       if (QUITP)
 	{
 	  closesocket (s);
-	  REALLY_QUIT;
+	  QUIT;
 	}
 
       /* Poll for quit every 250 ms */
       tv.tv_sec = 0;
       tv.tv_usec = 250 * 1000;
 
-      FD_ZERO (&fdset);
-      FD_SET (s, &fdset);
-      nsel = select (0, NULL, &fdset, &fdset, &tv);
+      FD_ZERO (&fdwriteset);
+      FD_SET (s, &fdwriteset);
+      FD_ZERO (&fdexceptset);
+      FD_SET (s, &fdexceptset);
+      nsel = select (0, NULL, &fdwriteset, &fdexceptset, &tv);
+
+      if (nsel == SOCKET_ERROR)
+	{
+	  errnum = WSAGetLastError ();
+	  goto connect_failed;
+	}
 
       if (nsel > 0)
 	{
 	  /* Check: was connection successful or not? */
-	  tv.tv_usec = 0;
-	  nsel = select (0, NULL, NULL, &fdset, &tv);
-	  if (nsel > 0)
-	    goto connect_failed;
-	  else
+	  if (FD_ISSET (s, &fdwriteset))
 	    break;
+	  else if (FD_ISSET (s, &fdexceptset))
+	    {
+	      int store_me_harder = sizeof (errnum);
+	      /* OK, we finally can get the REAL error code.  Any paths
+		 in this code that lead to a call of WSAGetLastError()
+		 indicate probable logic failure. */
+	      if (getsockopt (s, SOL_SOCKET, SO_ERROR, (char *) &errnum,
+			      &store_me_harder))
+		errnum = WSAGetLastError ();
+	      goto connect_failed;
+	    }
+	  else
+	    {
+	      signal_error (Qinternal_error,
+			    "Porra, esse caralho de um sistema de operacao",
+			    Qunbound);
+	      break;
+	    }
 	}
     }
+#endif
 
   /* We are connected at this point */
-  *vinfd = (void*)s;
+  *vinfd = (void *)s;
   DuplicateHandle (GetCurrentProcess(), (HANDLE)s,
 		   GetCurrentProcess(), (LPHANDLE)voutfd,
 		   0, FALSE, DUPLICATE_SAME_ACCESS);
   return;
 
- connect_failed:  
-  closesocket (s);
-  if (INTP (service))
-    {
-      warn_when_safe (Qstream, Qwarning,
-		      "failure to open network stream to host \"%s\" for service \"%d\"",
-		      XSTRING_DATA (host),
-		      (unsigned short) XINT (service));
-    }
-  else
-    {
-      warn_when_safe (Qstream, Qwarning,
-		      "failure to open network stream to host \"%s\" for service \"%s\"",
-		      XSTRING_DATA (host),
-		      XSTRING_DATA (service));
-    }
-  report_file_error ("connection failed", list2 (host, name));
+ connect_failed:
+  {
+    closesocket (s);
+    mswindows_report_winsock_error ("Connection failed",
+				    list3 (Qunbound, host, service),
+				    errnum);
+  }
 }
 
 #endif
+
+
+DEFUN ("mswindows-set-process-priority", Fmswindows_set_process_priority, 2, 2, "", /*
+  Set the priority of PROCESS to PRIORITY.
+If PROCESS is nil, the priority of Emacs is changed, otherwise the
+priority of the process whose pid is PROCESS is changed.
+PRIORITY should be one of the symbols high, normal, or low;
+any other symbol will be interpreted as normal.
+
+If successful, the return value is t, otherwise nil.
+*/
+       (process, priority))
+{
+  HANDLE proc_handle = GetCurrentProcess ();
+  DWORD  priority_class = NORMAL_PRIORITY_CLASS;
+  Lisp_Object result = Qnil;
+
+  CHECK_SYMBOL (priority);
+
+  if (!NILP (process))
+    {
+      DWORD pid;
+      struct Lisp_Process *p = 0;
+
+      if (PROCESSP (process))
+	{
+	  CHECK_LIVE_PROCESS (process);
+	  p = XPROCESS (process);
+	  pid = NT_DATA (p)->dwProcessId;
+	}
+      else
+	{
+	  CHECK_INT (process);
+
+	  /* Allow pid to be an internally generated one, or one obtained
+	     externally.  This is necessary because real pids on Win95 are
+	     negative.  */
+
+	  pid = XINT (process);
+	  p = find_process_from_pid (pid);
+	  if (p != NULL)
+	    pid = NT_DATA (p)->dwProcessId;
+	}
+
+      /* #### Should we be using the existing process handle from NT_DATA(p)?
+	 Will we fail if we open it a second time? */
+      proc_handle = OpenProcess (PROCESS_SET_INFORMATION, FALSE, pid);
+    }
+
+  if (EQ (priority, Qhigh))
+    priority_class = HIGH_PRIORITY_CLASS;
+  else if (EQ (priority, Qlow))
+    priority_class = IDLE_PRIORITY_CLASS;
+
+  if (proc_handle != NULL)
+    {
+      if (SetPriorityClass (proc_handle, priority_class))
+	result = Qt;
+      if (!NILP (process))
+	CloseHandle (proc_handle);
+    }
+
+  return result;
+}
+
 
 /*-----------------------------------------------------------------------*/
 /* Initialization							 */
@@ -1376,6 +1573,7 @@ process_type_create_nt (void)
 void
 syms_of_process_nt (void)
 {
+  DEFSUBR (Fmswindows_set_process_priority);
   DEFSYMBOL (Qmswindows_construct_process_command_line);
 }
 
