@@ -2,6 +2,7 @@
    expression library, version 0.12.
 
    Copyright (C) 1985, 89, 90, 91, 92, 93, 95 Free Software Foundation, Inc.
+   Copyright (C) 2002 Ben Wing.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,35 +26,24 @@
 
 #ifdef emacs
 #define RE_TRANSLATE_TYPE Lisp_Object
+#define RE_LISP_SHORT_CONTEXT_ARGS_DECL , Lisp_Object lispobj, struct buffer *lispbuf
+#define RE_LISP_SHORT_CONTEXT_ARGS , lispobj, lispbuf
+#define RE_LISP_CONTEXT_ARGS_DECL , Lisp_Object lispobj, struct buffer *lispbuf, struct syntax_cache *scache
+#define RE_LISP_CONTEXT_ARGS , lispobj, lispbuf, scache
 #else
 #define RE_TRANSLATE_TYPE char *
-
-/* type definitions copied from lisp.h */
-#ifndef SIZEOF_EMACS_INT
-# define SIZEOF_EMACS_INT SIZEOF_VOID_P
-#endif
-
-#ifndef EMACS_INT
-# if   SIZEOF_EMACS_INT == SIZEOF_LONG
-#  define EMACS_INT long
-# elif SIZEOF_EMACS_INT == SIZEOF_INT
-#  define EMACS_INT int
-# elif SIZEOF_EMACS_INT == SIZEOF_LONG_LONG
-#  define EMACS_INT long long
-# else
-#  error Unable to determine suitable type for EMACS_INT
-# endif
-#endif
-
-/* Counts of bytes or array elements */
-typedef EMACS_INT Memory_count;
-typedef EMACS_INT Element_count;
-
+#define RE_LISP_SHORT_CONTEXT_ARGS_DECL
+#define RE_LISP_SHORT_CONTEXT_ARGS
+#define RE_LISP_CONTEXT_ARGS_DECL
+#define RE_LISP_CONTEXT_ARGS
+#define Elemcount ssize_t
+#define Bytecount ssize_t
 #endif /* emacs */
 
 /* POSIX says that <sys/types.h> must be included (by the caller) before
    <regex.h>.  */
 
+
 /* The following bits are used to determine the regexp syntax we
    recognize.  The not-set meaning typically corresponds to the syntax
    used by Emacs (the exception is RE_INTERVAL, made for historical
@@ -174,6 +164,12 @@ typedef unsigned reg_syntax_t;
    If not set, then an unmatched ) is invalid.  */
 #define RE_UNMATCHED_RIGHT_PAREN_ORD (RE_NO_SHY_GROUPS << 1)
 
+/* If this bit is set, then \22 will read as a back reference,
+   provided at least 22 non-shy groups have been seen so far.  In all
+   other cases (bit not set, not 22 non-shy groups seen so far), it
+   reads as a back reference \2 followed by a digit 2. */
+#define RE_NO_MULTI_DIGIT_BK_REFS (RE_UNMATCHED_RIGHT_PAREN_ORD << 1)
+
 /* This global variable defines the particular regexp syntax to use (for
    some interfaces).  When a regexp is compiled, the syntax used is
    stored in the pattern buffer, so changing this does not affect
@@ -191,7 +187,7 @@ extern reg_syntax_t re_syntax_options;
    | RE_NO_BK_PARENS            | RE_NO_BK_REFS				\
    | RE_NO_BK_VBAR               | RE_NO_EMPTY_RANGES			\
    | RE_UNMATCHED_RIGHT_PAREN_ORD | RE_NO_SHY_GROUPS			\
-   | RE_NO_MINIMAL_MATCHING)
+   | RE_NO_MINIMAL_MATCHING | RE_NO_MULTI_DIGIT_BK_REFS)
 
 #define RE_SYNTAX_POSIX_AWK 						\
   (RE_SYNTAX_POSIX_EXTENDED | RE_BACKSLASH_ESCAPE_IN_LISTS)
@@ -200,17 +196,18 @@ extern reg_syntax_t re_syntax_options;
   (RE_BK_PLUS_QM              | RE_CHAR_CLASSES				\
    | RE_HAT_LISTS_NOT_NEWLINE | RE_INTERVALS				\
    | RE_NEWLINE_ALT           | RE_NO_SHY_GROUPS			\
-   | RE_NO_MINIMAL_MATCHING)
+   | RE_NO_MINIMAL_MATCHING | RE_NO_MULTI_DIGIT_BK_REFS)
 
 #define RE_SYNTAX_EGREP							\
   (RE_CHAR_CLASSES        | RE_CONTEXT_INDEP_ANCHORS			\
    | RE_CONTEXT_INDEP_OPS | RE_HAT_LISTS_NOT_NEWLINE			\
    | RE_NEWLINE_ALT       | RE_NO_BK_PARENS				\
    | RE_NO_BK_VBAR        | RE_NO_SHY_GROUPS				\
-   | RE_NO_MINIMAL_MATCHING)
+   | RE_NO_MINIMAL_MATCHING | RE_NO_MULTI_DIGIT_BK_REFS)
 
 #define RE_SYNTAX_POSIX_EGREP						\
-  (RE_SYNTAX_EGREP | RE_INTERVALS | RE_NO_BK_BRACES)
+  (RE_SYNTAX_EGREP | RE_INTERVALS | RE_NO_BK_BRACES |			\
+   RE_NO_MULTI_DIGIT_BK_REFS)
 
 /* P1003.2/D11.2, section 4.20.7.1, lines 5078ff.  */
 #define RE_SYNTAX_ED RE_SYNTAX_POSIX_BASIC
@@ -221,7 +218,7 @@ extern reg_syntax_t re_syntax_options;
 #define _RE_SYNTAX_POSIX_COMMON						\
   (RE_CHAR_CLASSES | RE_DOT_NEWLINE      | RE_DOT_NOT_NULL		\
    | RE_INTERVALS  | RE_NO_EMPTY_RANGES | RE_NO_SHY_GROUPS		\
-   | RE_NO_MINIMAL_MATCHING)
+   | RE_NO_MINIMAL_MATCHING | RE_NO_MULTI_DIGIT_BK_REFS)
 
 #define RE_SYNTAX_POSIX_BASIC						\
   (_RE_SYNTAX_POSIX_COMMON | RE_BK_PLUS_QM)
@@ -339,10 +336,10 @@ struct re_pattern_buffer
   unsigned char *buffer;
 
 	/* Number of bytes to which `buffer' points.  */
-  unsigned long allocated;
+  long allocated;
 
 	/* Number of bytes actually used in `buffer'.  */
-  unsigned long used;
+  long used;
 
         /* Syntax setting with which the pattern was compiled.  */
   reg_syntax_t syntax;
@@ -360,18 +357,18 @@ struct re_pattern_buffer
 
 	/* Number of returnable groups found by the compiler. (This does
            not count shy groups.) */
-  Element_count re_nsub;
+  int re_nsub;
 
 	/* Total number of groups found by the compiler. (Including
 	   shy ones.) */
-  Element_count re_ngroups;
+  int re_ngroups;
 
         /* Zero if this pattern cannot match the empty string, one else.
            Well, in truth it's used only in `re_search_2', to see
            whether or not we should use the fastmap, so we don't set
            this absolutely perfectly; see `re_compile_fastmap' (the
            `duplicate' case).  */
-  unsigned can_be_null : 1;
+  unsigned int can_be_null : 1;
 
         /* If REGS_UNALLOCATED, allocate space in the `regs' structure
              for `max (RE_NREGS, re_nsub + 1)' groups.
@@ -380,25 +377,27 @@ struct re_pattern_buffer
 #define REGS_UNALLOCATED 0
 #define REGS_REALLOCATE 1
 #define REGS_FIXED 2
-  unsigned regs_allocated : 2;
+  unsigned int regs_allocated : 2;
 
         /* Set to zero when `regex_compile' compiles a pattern; set to one
            by `re_compile_fastmap' if it updates the fastmap.  */
-  unsigned fastmap_accurate : 1;
+  unsigned int fastmap_accurate : 1;
 
         /* If set, `re_match_2' does not return information about
            subexpressions.  */
-  unsigned no_sub : 1;
+  unsigned int no_sub : 1;
 
         /* If set, a beginning-of-line anchor doesn't match at the
            beginning of the string.  */
-  unsigned not_bol : 1;
+  unsigned int not_bol : 1;
 
         /* Similarly for an end-of-line anchor.  */
-  unsigned not_eol : 1;
+  unsigned int not_eol : 1;
 
         /* If true, an anchor at a newline matches.  */
-  unsigned newline_anchor : 1;
+  unsigned int newline_anchor : 1;
+
+  unsigned int warned_about_incompatible_back_references : 1;
 
 	/* Mapping between back references and groups (may not be
 	   equivalent with shy groups). */
@@ -419,7 +418,7 @@ typedef int regoff_t;
    regex.texinfo for a full description of what registers match.  */
 struct re_registers
 {
-  int num_regs;
+  int num_regs;			/* number of registers allocated */
   regoff_t *start;
   regoff_t *end;
 };
@@ -458,7 +457,8 @@ const char *re_compile_pattern (const char *pattern, int length,
 /* Compile a fastmap for the compiled pattern in BUFFER; used to
    accelerate searches.  Return 0 if successful and -2 if was an
    internal error.  */
-int re_compile_fastmap (struct re_pattern_buffer *buffer);
+int re_compile_fastmap (struct re_pattern_buffer *buffer
+			RE_LISP_SHORT_CONTEXT_ARGS_DECL);
 
 
 /* Search in the string STRING (with length LENGTH) for the pattern
@@ -468,27 +468,31 @@ int re_compile_fastmap (struct re_pattern_buffer *buffer);
    information in REGS (if REGS and BUFFER->no_sub are nonzero).  */
 int re_search (struct re_pattern_buffer *buffer, const char *string,
 	       int length, int start, int range,
-	       struct re_registers *regs);
+	       struct re_registers *regs RE_LISP_CONTEXT_ARGS_DECL);
 
 
 /* Like `re_search', but search in the concatenation of STRING1 and
    STRING2.  Also, stop searching at index START + STOP.  */
 int re_search_2 (struct re_pattern_buffer *buffer, const char *string1,
 		 int length1, const char *string2, int length2, int start,
-		 int range, struct re_registers *regs, int stop);
+		 int range, struct re_registers *regs, int stop
+		 RE_LISP_CONTEXT_ARGS_DECL);
 
+#ifndef emacs /* never used by XEmacs */
 
 /* Like `re_search', but return how many characters in STRING the regexp
    in BUFFER matched, starting at position START.  */
 int re_match (struct re_pattern_buffer *buffer, const char *string,
-	      int length, int start, struct re_registers *regs);
+	      int length, int start, struct re_registers *regs
+	      RE_LISP_CONTEXT_ARGS_DECL);
 
+#endif /* not emacs */
 
 /* Relates to `re_match' as `re_search_2' relates to `re_search'.  */
 int re_match_2 (struct re_pattern_buffer *buffer, const char *string1,
 		int length1, const char *string2, int length2,
-		int start, struct re_registers *regs, int stop);
-
+		int start, struct re_registers *regs, int stop
+		RE_LISP_CONTEXT_ARGS_DECL);
 
 /* Set REGS to hold NUM_REGS registers, storing them in STARTS and
    ENDS.  Subsequent matches using BUFFER and REGS will use this memory
@@ -503,7 +507,7 @@ int re_match_2 (struct re_pattern_buffer *buffer, const char *string1,
    PATTERN_BUFFER will allocate its own register data, without
    freeing the old data.  */
 void re_set_registers (struct re_pattern_buffer *buffer,
-		       struct re_registers *regs, unsigned num_regs,
+		       struct re_registers *regs, int num_regs,
 		       regoff_t *starts, regoff_t *ends);
 
 #ifdef _REGEX_RE_COMP
@@ -513,13 +517,11 @@ int re_exec (const char *);
 #endif
 
 /* POSIX compatibility.  */
-/* #### Arrgh, not any more.  But I don't have time to deal with this
-   properly, and I don't know if we should care. - sjt */
 int regcomp (regex_t *preg, const char *pattern, int cflags);
-int regexec (const regex_t *preg, const char *string, Element_count nmatch,
+int regexec (const regex_t *preg, const char *string, size_t nmatch,
 	     regmatch_t pmatch[], int eflags);
-Memory_count regerror (int errcode, const regex_t *preg, char *errbuf,
-		       Memory_count errbuf_size);
+size_t regerror (int errcode, const regex_t *preg, char *errbuf,
+		 size_t errbuf_size);
 void regfree (regex_t *preg);
 
 #endif /* INCLUDED_regex_h_ */
