@@ -1,15 +1,15 @@
-/* Definitions file for GNU Emacs running on HPUX release 7.0.
+/* Definitions file for XEmacs running on HPUX release 7.0.
    Based on AT&T System V.2.
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
-This file is part of GNU Emacs.
+This file is part of XEmacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+XEmacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
-GNU Emacs is distributed in the hope that it will be useful,
+XEmacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
@@ -55,12 +55,6 @@ Boston, MA 02111-1307, USA.  */
 
 #define FIRST_PTY_LETTER 'p'
 
-/*
- *	Define HAVE_PTYS if the system supports pty devices.
- */
-
-#define HAVE_PTYS
-
 /* If your system uses COFF (Common Object File Format) then define the
    preprocessor symbol "COFF". */
 
@@ -157,59 +151,6 @@ Boston, MA 02111-1307, USA.  */
 /* This is needed for HPUX version 6.2; it may not be needed for 6.2.1.  */
 #define SHORT_CAST_BUG
 
-#if defined(HAVE_GRANTPT) && defined(HAVE_UNLOCKPT) && defined(HAVE_PTSNAME)
-/* UNIX98 PTYs are available.
-   Added by Florian Weimer <Florian.Weimer@RUS.Uni-Stuttgart.DE>,
-   RUS-CERT, University of Stuttgart.  Based on Emacs code for DGUX. */
-
-#ifdef emacs
-#include <grp.h>
-#include <sys/stropts.h>
-#endif
-
-#define PTY_ITERATION for (i = 0; i < 1; i++)
-/* no iteration at all */
-
-/* the master PTY device */
-#define PTY_NAME_SPRINTF strcpy (pty_name, "/dev/ptmx");
-
-/* This sets the name of the slave side of the PTY.  grantpt(3) and
-   unlockpt(3) may fork a subprocess, so keep sigchld_handler() from
-   intercepting that death.  grantpt() behavior on HP-UX differs from
-   what's specified in the man page: the group of the slave PTY is set
-   to the user's primary group, and we fix that.  */
-
-#define PTY_TTY_NAME_SPRINTF			\
-  {						\
-    char *ptsname(), *ptyname;			\
-    struct group *getgrnam (), *tty_group = getgrnam ("tty"); \
-    if (tty_group == NULL)                      \
-      fatal ("group tty not found");            \
-						\
-    EMACS_BLOCK_SIGNAL (SIGCHLD);		\
-    if (grantpt(fd) == -1)			\
-      fatal("could not grant slave pty");	\
-    if (!(ptyname = ptsname(fd)))		\
-      fatal ("could not enable slave pty");	\
-    strncpy(pty_name, ptyname, sizeof(pty_name)); \
-    pty_name[sizeof(pty_name) - 1] = 0;		\
-    if (chown (pty_name, (uid_t) -1, tty_group->gr_gid) == -1) \
-      fatal ("could not chown slave pty");      \
-    if (unlockpt(fd) == -1)			\
-      fatal("could not unlock slave pty");	\
-    EMACS_UNBLOCK_SIGNAL (SIGCHLD);		\
-  }
-
-/* Push various streams modules onto a PTY channel. */
-
-#define SETUP_SLAVE_PTY \
-  if (ioctl (xforkin, I_PUSH, "ptem") == -1)	\
-    fatal ("ioctl I_PUSH ptem", errno);		\
-  if (ioctl (xforkin, I_PUSH, "ldterm") == -1)	\
-    fatal ("ioctl I_PUSH ldterm", errno);
-
-#else /* no UNIX98 PTYs */
-
 /* This is how to get the device name of the tty end of a pty.  */
 #define PTY_TTY_NAME_SPRINTF \
             sprintf (pty_name, "/dev/pty/tty%c%x", c, i);
@@ -218,7 +159,5 @@ Boston, MA 02111-1307, USA.  */
 #define PTY_NAME_SPRINTF \
 	sprintf (pty_name, "/dev/ptym/pty%c%x", c, i);
 
-#endif /* UNIX 98 PTYs */
-
 /* This triggers a conditional in xfaces.c.  */
 #define XOS_NEEDS_TIME_H
