@@ -1,4 +1,4 @@
-/* scrollbar implementation -- GTK interface.
+/* scrollbar implementation -- X interface.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1994 Amdhal Corporation.
    Copyright (C) 1995 Sun Microsystems, Inc.
@@ -27,14 +27,13 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "console-gtk.h"
-#include "glyphs-gtk.h"
-#include "gui-gtk.h"
-#include "scrollbar-gtk.h"
-
-#include "frame.h"
+#include "frame-impl.h"
 #include "window.h"
 
+#include "console-gtk-impl.h"
+#include "glyphs-gtk.h"
+#include "scrollbar-gtk.h"
+
 static gboolean scrollbar_cb (GtkAdjustment *adj, gpointer user_data);
 
 /* Used to prevent changing the size of the slider while drag
@@ -57,16 +56,14 @@ gtk_inhibit_scrollbar_slider_size_change (void)
 static void
 gtk_free_scrollbar_instance (struct scrollbar_instance *instance)
 {
-  if (instance->scrollbar_data)
+  if (SCROLLBAR_GTK_WIDGET (instance))
     {
-      if (SCROLLBAR_GTK_WIDGET (instance))
-	{
-	  gtk_widget_hide_all (SCROLLBAR_GTK_WIDGET (instance));
-	  gtk_widget_destroy (SCROLLBAR_GTK_WIDGET (instance));
-	}
-
-      xfree (instance->scrollbar_data);
+      gtk_widget_hide_all (SCROLLBAR_GTK_WIDGET (instance));
+      gtk_widget_destroy (SCROLLBAR_GTK_WIDGET (instance));
     }
+
+  if (instance->scrollbar_data)
+    xfree (instance->scrollbar_data);
 }
 
 /* A device method. */
@@ -100,20 +97,10 @@ gtk_create_scrollbar_instance (struct frame *f, int vertical,
   SCROLLBAR_GTK_VDRAG_ORIG_VALUE (instance) = -1;
   SCROLLBAR_GTK_LAST_VALUE (instance) = adj->value;
 
-  gtk_object_set_data (GTK_OBJECT (adj), GTK_DATA_GUI_IDENTIFIER,
-		       (void *) SCROLLBAR_GTK_ID (instance));
-  gtk_object_set_data (GTK_OBJECT (adj), GTK_DATA_FRAME_IDENTIFIER, f);
-  gtk_object_set_data (GTK_OBJECT (adj), "xemacs::sb_instance", instance);
+  gtk_object_set_data (GTK_OBJECT (adj), "xemacs::gui_id", (void *) SCROLLBAR_GTK_ID (instance));
+  gtk_object_set_data (GTK_OBJECT (adj), "xemacs::frame", f);
 
   sb = GTK_SCROLLBAR (vertical ? gtk_vscrollbar_new (adj) : gtk_hscrollbar_new (adj));
-  /* With gtk version > 1.2.8 the gtk code does not call
-     gtk_widget_request_size() on the newly created scrollbars
-     anymore (catering to theme engines).
-     #### Maybe it is better to postpone this call to just before
-     gtk_widget_show() is called on the scrollbar? */
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2 && GTK_BINARY_AGE > 8
-  gtk_widget_size_request(GTK_WIDGET(sb), &(GTK_WIDGET(sb)->requisition));
-#endif  
   SCROLLBAR_GTK_WIDGET (instance) = GTK_WIDGET (sb);
 
   gtk_signal_connect (GTK_OBJECT (adj),"value-changed",
@@ -125,6 +112,20 @@ gtk_create_scrollbar_instance (struct frame *f, int vertical,
 		      GTK_SIGNAL_FUNC (scrollbar_drag_hack_cb), (gpointer) 0);
 
   gtk_fixed_put (GTK_FIXED (FRAME_GTK_TEXT_WIDGET (f)), SCROLLBAR_GTK_WIDGET (instance), 0, 0);
+
+  /*
+  ** With gtk version > 1.2.8 the scrollbars in gtk-xemacs and xemacs
+  ** from CVS are invisible. In fact they are not invisible but very
+  ** thin (0 pixels wide). This is so, because the new gtk code does
+  ** not call gtk_widget_request_size() on the newly created
+  ** scrollbars anymore. this change was done to allow the theme
+  ** engines to manipulate the scrollbar width. This patch calls
+  ** gtk_widget_request_size with the newly created scollbars. Maybe
+  ** it is better to postpone this call just before the
+  ** gtk_widget_show() call is done on the scrolbar.
+  */
+  gtk_widget_size_request(GTK_WIDGET(sb), &(GTK_WIDGET(sb)->requisition));
+
   gtk_widget_hide (SCROLLBAR_GTK_WIDGET (instance));
 }
 
@@ -220,21 +221,6 @@ gtk_update_scrollbar_instance_status (struct window *w, int active, int size,
 	      gtk_widget_set_usize (wid,
 				    pos_data->scrollbar_width,
 				    pos_data->scrollbar_height);
-
-	      /*
-		UGLY! UGLY! UGLY!  Changes to wid->allocation are queued and
-		not performed until the GTK event loop.  However, when the
-		fontlock progress bar is run, the vertical scrollbar's height
-		is change and then changed back before events are again
-		processed.  This means that the change back is not seen and
-		the scrollbar is left too short.  Fix this by making the
-		change manually so the test above sees the change.  This does
-		not seem to cause problems in other cases.
-	       */
-
-	      wid->allocation.width = pos_data->scrollbar_width;
-	      wid->allocation.height = pos_data->scrollbar_height;
-
 	      modified_p = 1;
 	    }
 
@@ -246,21 +232,6 @@ gtk_update_scrollbar_instance_status (struct window *w, int active, int size,
 			      wid,
 			      pos_data->scrollbar_x,
 			      pos_data->scrollbar_y);
-
-	      /*
-		UGLY! UGLY! UGLY!  Changes to wid->allocation are queued and
-		not performed until the GTK event loop.  However, when the
-		fontlock progress bar is run, the horizontal scrollbar's
-		position is change and then changed back before events are
-		again processed.  This means that the change back is not seen
-		and the scrollbar is left in the wrong position.  Fix this by
-		making the change manually so the test above sees the change.
-		This does not seem to cause problems in other cases.
-	       */
-
-	      wid->allocation.x = pos_data->scrollbar_x;
-	      wid->allocation.y = pos_data->scrollbar_y;
-
 	      modified_p = 1;
 	    }
 
@@ -366,7 +337,7 @@ gtk_scrollbar_loop (enum gtk_scrollbar_loop type, Lisp_Object window,
 		}
 	      break;
 	    default:
-	      ABORT ();
+	      abort ();
 	    }
 	}
 
@@ -384,7 +355,8 @@ find_scrollbar_window_mirror (struct frame *f, GUI_ID id)
   if (f->mirror_dirty)
     update_frame_window_mirror (f);
   return gtk_scrollbar_loop (GTK_FIND_SCROLLBAR_WINDOW_MIRROR, f->root_window,
-			     f->root_mirror, id, (GdkWindow *) NULL);
+			     XWINDOW_MIRROR (f->root_mirror), id,
+			     (GdkWindow *) NULL);
 }
 
 static gboolean
@@ -392,9 +364,9 @@ scrollbar_cb (GtkAdjustment *adj, gpointer user_data)
 {
   /* This function can GC */
   int vertical = (int) user_data;
-  struct frame *f = gtk_object_get_data (GTK_OBJECT (adj), GTK_DATA_FRAME_IDENTIFIER);
-  struct scrollbar_instance *instance = gtk_object_get_data (GTK_OBJECT (adj), "xemacs::sb_instance");
-  GUI_ID id = (GUI_ID) gtk_object_get_data (GTK_OBJECT (adj), GTK_DATA_GUI_IDENTIFIER);
+  struct frame *f = gtk_object_get_data (GTK_OBJECT (adj), "xemacs::frame");
+  struct scrollbar_instance *instance;
+  GUI_ID id = (GUI_ID) gtk_object_get_data (GTK_OBJECT (adj), "xemacs::gui_id");
   Lisp_Object win, frame;
   struct window_mirror *mirror;
   Lisp_Object event_type = Qnil;
@@ -440,7 +412,7 @@ scrollbar_cb (GtkAdjustment *adj, gpointer user_data)
       event_data = Fcons (win, make_int ((int)adj->value));
       break;
     default:
-      ABORT();
+      abort();
     }
 
   signal_special_gtk_user_event (frame, event_type, event_data);
@@ -451,11 +423,10 @@ scrollbar_cb (GtkAdjustment *adj, gpointer user_data)
 static void
 gtk_scrollbar_pointer_changed_in_window (struct window *w)
 {
-  Lisp_Object window;
+  Lisp_Object window = wrap_window (w);
 
-  XSETWINDOW (window, w);
-  gtk_scrollbar_loop (GTK_SET_SCROLLBAR_POINTER, window, find_window_mirror (w),
-		      0, (GdkWindow *) NULL);
+  gtk_scrollbar_loop (GTK_SET_SCROLLBAR_POINTER, window,
+		      find_window_mirror (w), 0, (GdkWindow *) NULL);
 }
 
 /* #### BILL!!! This comment is not true for Gtk - should it be? */
@@ -464,13 +435,9 @@ gtk_scrollbar_pointer_changed_in_window (struct window *w)
 void
 gtk_update_frame_scrollbars (struct frame *f)
 {
-  /* Consider this code to be "in_display" so that we ABORT() if Fsignal()
-     gets called. */
-  in_display++;
-  gtk_scrollbar_loop (GTK_UPDATE_FRAME_SCROLLBARS, f->root_window, f->root_mirror,
+  gtk_scrollbar_loop (GTK_UPDATE_FRAME_SCROLLBARS, f->root_window,
+		      XWINDOW_MIRROR (f->root_mirror),
 		      0, (GdkWindow *) NULL);
-  in_display--;
-  if (in_display < 0) ABORT ();
 }
 
 #ifdef MEMORY_USAGE_STATS
