@@ -21,17 +21,17 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: Not in FSF. */
 
+/* This file Mule-ized by Ben Wing, 5-15-01. */
+
 #include <config.h>
 #include "lisp.h"
 
+#include "sound.h"
+
+#include "sysfile.h"
+#include "sysproc.h" /* netinet/in.h for ntohl() etc. */
+
 #include <audio.h>
-#include <sys/file.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <string.h>
-#include <netinet/in.h>		/* for ntohl() etc. */
 
 /* Configuration options */
 
@@ -91,14 +91,6 @@ Boston, MA 02111-1307, USA.  */
 #define DEFAULT_CHANNEL_COUNT	     1
 #define DEFAULT_FORMAT	      AFmulaw8
 
-/* Exports */
-
-/* all compilers on machines that have the SGI audio library
-   understand prototypes, right? */
-
-extern void play_sound_file (char *, int);
-extern void play_sound_data (unsigned char *, int, int);
-
 /* Data structures */
 
 /* an AudioContext describes everything we want to know about how a
@@ -182,8 +174,8 @@ AudioContextRec;
 /* Forward declarations */
 
 static Lisp_Object close_sound_file (Lisp_Object);
-static AudioContext audio_initialize (unsigned char *, int, int);
-static void play_internal (unsigned char *, int, AudioContext);
+static AudioContext audio_initialize (UChar_Binary *, int, int);
+static int play_internal (UChar_Binary *, int, AudioContext);
 static void drain_audio_port (AudioContext);
 static void write_mulaw_8_chunk (void *, void *, AudioContext);
 static void write_linear_chunk (void *, void *, AudioContext);
@@ -202,21 +194,18 @@ static int parse_snd_header (void*, long, AudioContext);
   (!strncmp(".snd", (char *)(address), 4))
 
 static Lisp_Object
-close_sound_file (closure)
-     Lisp_Object closure;
+close_sound_file (Lisp_Object closure)
 {
   close (XINT (closure));
   return Qnil;
 }
 
 void
-play_sound_file (sound_file, volume)
-     char * sound_file;
-     int volume;
+play_sound_file (Extbyte *sound_file, int volume)
 {
   int count = specpdl_depth ();
   int input_fd;
-  unsigned char buffer[CHUNKSIZE];
+  UChar_Binary buffer[CHUNKSIZE];
   int bytes_read;
   AudioContext ac = (AudioContext) 0;
 
@@ -245,7 +234,7 @@ play_sound_file (sound_file, volume)
       play_internal (buffer, bytes_read, ac);
     }
   drain_audio_port (ac);
-  unbind_to (count, Qnil);
+  unbind_to (count);
 }
 
 static long
@@ -256,8 +245,7 @@ saved_device_state[] = {
 };
 
 static Lisp_Object
-restore_audio_port (closure)
-     Lisp_Object closure;
+restore_audio_port (Lisp_Object closure)
 {
   Lisp_Object * contents = XVECTOR_DATA (closure);
   saved_device_state[1] = XINT (contents[0]);
@@ -267,28 +255,24 @@ restore_audio_port (closure)
   return Qnil;
 }
 
-void
-play_sound_data (data, length, volume)
-     unsigned char * data;
-     int length;
-     int volume;
+int
+play_sound_data (UChar_Binary *data, int length, int volume)
 {
   int count = specpdl_depth ();
   AudioContext ac;
+  int result;
 
   ac = audio_initialize (data, length, volume);
   if (ac == (AudioContext) 0)
-    return;
-  play_internal (data, length, ac);
+    return 0;
+  result = play_internal (data, length, ac);
   drain_audio_port (ac);
-  unbind_to (count, Qnil);
+  unbind_to (count);
+  return result;
 }
 
 static AudioContext
-audio_initialize (data, length, volume)
-     unsigned char * data;
-     int length;
-     int volume;
+audio_initialize (UChar_Binary *data, int length, int volume)
 {
   Lisp_Object audio_port_state[3];
   static AudioContextRec desc;
@@ -303,7 +287,7 @@ audio_initialize (data, length, volume)
   if (LOOKING_AT_SND_HEADER_P (data))
     {
       if (parse_snd_header (data, length, & desc)==-1)
-	report_file_error ("decoding .snd header", Qnil);
+	report_sound_error ("decoding .snd header", Qunbound);
     }
   else
 #endif
@@ -331,21 +315,18 @@ audio_initialize (data, length, volume)
   return ac;
 }
 
-static void
-play_internal (data, length, ac)
-     unsigned char * data;
-     int length;
-     AudioContext ac;
+static int
+play_internal (UChar_Binary *data, int length, AudioContext ac)
 {
-  unsigned char * limit;
+  UChar_Binary * limit;
   if (ac == (AudioContext) 0)
-    return;
+    return 0;
 
-  data = ac->ac_data;
+  data = (UChar_Binary *) ac->ac_data;
   limit = data + ac->ac_size;
   while (data < limit)
     {
-      unsigned char * chunklimit = data + CHUNKSIZE;
+      UChar_Binary * chunklimit = data + CHUNKSIZE;
 
       if (chunklimit > limit)
 	chunklimit = limit;
@@ -355,11 +336,12 @@ play_internal (data, length, ac)
       (* ac->ac_write_chunk_function) (data, chunklimit, ac);
       data = chunklimit;
     }
+
+  return 1;
 }
 
 static void
-drain_audio_port (ac)
-     AudioContext ac;
+drain_audio_port (AudioContext ac)
 {
   while (ALgetfilled (ac->ac_port) > 0)
     sginap(1);
@@ -375,10 +357,9 @@ drain_audio_port (ac)
 #include "libst.h"
 #else /* not USE_MULAW_DECODE_TABLE */
 static int
-st_ulaw_to_linear (u)
-     int u;
+st_ulaw_to_linear (int u)
 {
-  static CONST short table[] = {0,132,396,924,1980,4092,8316,16764};
+  static const short table[] = {0,132,396,924,1980,4092,8316,16764};
   int u1 = ~u;
   short exponent = (u1 >> 4) & 0x07;
   int mantissa = u1 & 0x0f;
@@ -388,13 +369,10 @@ st_ulaw_to_linear (u)
 #endif /* not USE_MULAW_DECODE_TABLE */
 
 static void
-write_mulaw_8_chunk (buffer, chunklimit, ac)
-     void * buffer;
-     void * chunklimit;
-     AudioContext ac;
+write_mulaw_8_chunk (void *buffer, void *chunklimit, AudioContext ac)
 {
-  unsigned char * data = (unsigned char *) buffer;
-  unsigned char * limit = (unsigned char *) chunklimit;
+  UChar_Binary * data = (UChar_Binary *) buffer;
+  UChar_Binary * limit = (UChar_Binary *) chunklimit;
   short * obuf, * bufp;
   long n_samples = limit - data;
 
@@ -409,17 +387,14 @@ write_mulaw_8_chunk (buffer, chunklimit, ac)
 
 #if HAVE_LINEAR
 static void
-write_linear_chunk (data, limit, ac)
-     void * data;
-     void * limit;
-     AudioContext ac;
+write_linear_chunk (void *data, void *limit, AudioContext ac)
 {
   unsigned n_samples;
 
   switch (ac->ac_format)
     {
     case AFlinear16: n_samples = (short *) limit - (short *) data; break;
-    case AFlinear8:  n_samples =  (char *) limit -  (char *) data; break;
+    case AFlinear8:  n_samples =  (Char_Binary *) limit -  (Char_Binary *) data; break;
     default: n_samples =  (long *) limit -  (long *) data; break;
     }
   ALwritesamps (ac->ac_port, data, (long) n_samples);
@@ -427,10 +402,7 @@ write_linear_chunk (data, limit, ac)
 
 #if HAVE_LINEAR_32
 static void
-write_linear_32_chunk (buffer, chunklimit, ac)
-     void * buffer;
-     void * chunklimit;
-     AudioContext ac;
+write_linear_32_chunk (void *buffer, void *chunklimit, AudioContext ac)
 {
   long * data = (long *) buffer;
   long * limit = (long *) chunklimit;
@@ -448,8 +420,7 @@ write_linear_32_chunk (buffer, chunklimit, ac)
 #endif /* HAVE_LINEAR */
 
 static AudioContext
-initialize_audio_port (desc)
-     AudioContext desc;
+initialize_audio_port (AudioContext desc)
 {
   /* we can't use the same port for mono and stereo */
   static AudioContextRec mono_port_state
@@ -482,7 +453,7 @@ initialize_audio_port (desc)
     {
       if ((open_audio_port (return_ac, desc))==-1)
 	{
-	  report_file_error ("Open audio port", Qnil);
+	  report_sound_error ("Open audio port", Qunbound);
 	  return (AudioContext) 0;
 	}
     }
@@ -516,9 +487,7 @@ initialize_audio_port (desc)
 }
 
 static int
-open_audio_port (return_ac, desc)
-     AudioContext return_ac;
-     AudioContext desc;
+open_audio_port (AudioContext return_ac, AudioContext desc)
 {
   ALconfig config = ALnewconfig();
   long params[2];
@@ -541,16 +510,14 @@ open_audio_port (return_ac, desc)
   ALfreeconfig (config);
   if (return_ac->ac_port==0)
     {
-      report_file_error ("Opening audio output port", Qnil);
+      report_sound_error ("Opening audio output port", Qunbound);
       return -1;
     }
   return 0;
 }
 
 static int
-set_channels (config, nchan)
-     ALconfig config;
-     unsigned nchan;
+set_channels (ALconfig config, unsigned int nchan)
 {
   switch (nchan)
     {
@@ -559,17 +526,15 @@ set_channels (config, nchan)
     case 2: ALsetchannels (config, AL_STEREO); break;
 #endif /* HAVE_STEREO */
     default:
-      report_file_error ("Unsupported channel count",
-			 Fcons (make_int (nchan), Qnil));
+      report_sound_error ("Unsupported channel count",
+			  make_int (nchan));
       return -1;
     }
   return 0;
 }
 
 static int
-set_output_format (config, format)
-     ALconfig config;
-     AudioFormat format;
+set_output_format (ALconfig config, AudioFormat format)
 {
   long samplesize;
   long old_samplesize;
@@ -598,8 +563,7 @@ set_output_format (config, format)
 #endif
 #endif
     default:
-      report_file_error ("Unsupported audio format",
-			 Fcons (make_int (format), Qnil));
+      report_sound_error ("Unsupported audio format", make_int (format));
       return -1;
     }
   old_samplesize = ALgetwidth (config);
@@ -610,8 +574,7 @@ set_output_format (config, format)
 }
 
 static void
-adjust_audio_volume (device)
-     AudioDevice device;
+adjust_audio_volume (AudioDevice device)
 {
   long params[4];
   params[0] = AL_LEFT_SPEAKER_GAIN;
@@ -622,8 +585,7 @@ adjust_audio_volume (device)
 }
 
 static void
-get_current_volumes (device)
-     AudioDevice device;
+get_current_volumes (AudioDevice device)
 {
   long params[4];
   params[0] = AL_LEFT_SPEAKER_GAIN;
@@ -645,7 +607,7 @@ typedef struct
   int dataFormat;
   int samplingRate;
   int channelCount;
-  char info[4];
+  Char_Binary info[4];
 }
 SNDSoundStruct;
 #define SOUND_TO_HOST_INT(x) ntohl(x)
@@ -679,10 +641,7 @@ typedef enum
 SNDFormatCode;
 
 static int
-parse_snd_header (header, length, desc)
-     void * header;
-     long length;
-     AudioContext desc;
+parse_snd_header (void *header, long length, AudioContext desc)
 {
 #define hp ((SNDSoundStruct *) (header))
   long limit;
@@ -720,8 +679,8 @@ parse_snd_header (header, length, desc)
     }
   desc->ac_output_rate = SOUND_TO_HOST_INT (hp->samplingRate);
   desc->ac_nchan = SOUND_TO_HOST_INT (hp->channelCount);
-  desc->ac_data = (char *) header + SOUND_TO_HOST_INT (hp->dataLocation);
-  limit = (char *) header + length - (char *) desc->ac_data;
+  desc->ac_data = (Char_Binary *) header + SOUND_TO_HOST_INT (hp->dataLocation);
+  limit = (Char_Binary *) header + length - (Char_Binary *) desc->ac_data;
   desc->ac_size = SOUND_TO_HOST_INT (hp->dataSize);
   if (desc->ac_size > limit) desc->ac_size = limit;
   return 0;
