@@ -1,6 +1,6 @@
 /* Generic specifier list implementation
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995 Ben Wing
+   Copyright (C) 1995, 1996, 2002 Ben Wing
 
 This file is part of XEmacs.
 
@@ -83,7 +83,7 @@ Boston, MA 02111-1307, USA.  */
      same time.
 */
 
-extern const struct struct_description specifier_methods_description;
+extern const struct sized_memory_description specifier_methods_description;
 
 struct specifier_methods
 {
@@ -154,7 +154,7 @@ struct specifier_methods
 
      This method must presume that both INSTANTIATOR and MATCHSPEC are
      already validated by the corresponding validate_* methods, and
-     may ABORT if they are invalid.
+     may abort if they are invalid.
 
      Return value is an instance, which is returned immediately to the
      caller, or Qunbound to continue instantiation lookup chain.
@@ -194,7 +194,23 @@ struct specifier_methods
   void (*after_change_method) (Lisp_Object specifier,
 			       Lisp_Object locale);
 
-  const struct lrecord_description *extra_description;
+  /* Specifier extra data: Specifier objects can have extra data, specific
+     to the type of specifier, stored at the end of the object.  To have
+     this, a specifier declares a structure of type `struct TYPE_specifier'
+     containing the data and uses DEFINE_SPECIFIER_TYPE_WITH_DATA and
+     INITIALIZE_SPECIFIER_TYPE_WITH_DATA instead of the plain versions.
+     Then, a pointer to the `struct TYPE_specifier' can be obtained from a
+     specifier object using SPECIFIER_TYPE_DATA.
+
+     A data description of the extra data must also be provided, in the
+     form of a memory_description named TYPE_specifier_description. */
+
+  /* Description of extra data structure; initialized when
+     INITIALIZE_SPECIFIER_TYPE_WITH_DATA is called. */
+  const struct sized_memory_description *extra_description;
+
+  /* Size of extra data structure; initialized when
+     INITIALIZE_SPECIFIER_TYPE_WITH_DATA is called. */
   int extra_data_size;
 };
 
@@ -241,7 +257,7 @@ typedef struct Lisp_Specifier Lisp_Specifier;
 
 DECLARE_LRECORD (specifier, Lisp_Specifier);
 #define XSPECIFIER(x) XRECORD (x, specifier, Lisp_Specifier)
-#define XSETSPECIFIER(x, p) XSETRECORD (x, p, specifier)
+#define wrap_specifier(p) wrap_record (p, specifier)
 #define SPECIFIERP(x) RECORDP (x, specifier)
 #define CHECK_SPECIFIER(x) CHECK_RECORD (x, specifier)
 #define CONCHECK_SPECIFIER(x) CONCHECK_RECORD (x, specifier)
@@ -261,16 +277,15 @@ DECLARE_LRECORD (specifier, Lisp_Specifier);
 
 /***** Defining new specifier types *****/
 
-#define specifier_data_offset offsetof (Lisp_Specifier, data)
-extern const struct lrecord_description specifier_empty_extra_description[];
+extern const struct sized_memory_description specifier_empty_extra_description;
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 #define DECLARE_SPECIFIER_TYPE(type)					\
 extern struct specifier_methods * type##_specifier_methods;		\
-INLINE_HEADER struct type##_specifier *					\
-error_check_##type##_specifier_data (Lisp_Specifier *sp);		\
-INLINE_HEADER struct type##_specifier *					\
+DECLARE_INLINE_HEADER (							\
+struct type##_specifier *						\
 error_check_##type##_specifier_data (Lisp_Specifier *sp)		\
+)									\
 {									\
   if (SPECIFIERP (sp->magic_parent))					\
     {									\
@@ -282,10 +297,10 @@ error_check_##type##_specifier_data (Lisp_Specifier *sp)		\
   assert (SPECIFIER_TYPE_P (sp, type));					\
   return (struct type##_specifier *) sp->data;				\
 }									\
-INLINE_HEADER Lisp_Specifier *						\
-error_check_##type##_specifier_type (Lisp_Object obj);			\
-INLINE_HEADER Lisp_Specifier *						\
+DECLARE_INLINE_HEADER (							\
+Lisp_Specifier *							\
 error_check_##type##_specifier_type (Lisp_Object obj)			\
+)									\
 {									\
   Lisp_Specifier *sp = XSPECIFIER (obj);				\
   assert (SPECIFIER_TYPE_P (sp, type));					\
@@ -295,20 +310,28 @@ DECLARE_NOTHING
 #else
 #define DECLARE_SPECIFIER_TYPE(type)					\
 extern struct specifier_methods * type##_specifier_methods
-#endif /* ERROR_CHECK_TYPECHECK */
+#endif /* ERROR_CHECK_TYPES */
 
 #define DEFINE_SPECIFIER_TYPE(type)					\
 struct specifier_methods * type##_specifier_methods
 
-#define INITIALIZE_SPECIFIER_TYPE(type, obj_name, pred_sym) do {		\
-  type##_specifier_methods = xnew_and_zero (struct specifier_methods);		\
-  type##_specifier_methods->name = obj_name;					\
-  type##_specifier_methods->extra_description =					\
-    specifier_empty_extra_description;						\
-  defsymbol_nodump (&type##_specifier_methods->predicate_symbol, pred_sym);	\
-  add_entry_to_specifier_type_list (Q##type, type##_specifier_methods);		\
-  dump_add_root_struct_ptr (&type##_specifier_methods,				\
-			    &specifier_methods_description);			\
+#define DEFINE_SPECIFIER_TYPE_WITH_DATA(type)				    \
+struct specifier_methods * type##_specifier_methods;			    \
+static const struct sized_memory_description type##_specifier_description_0 \
+  = {									    \
+  sizeof (struct type##_specifier),					    \
+  type##_specifier_description						    \
+}
+
+#define INITIALIZE_SPECIFIER_TYPE(type, obj_name, pred_sym) do {	    \
+  type##_specifier_methods = xnew_and_zero (struct specifier_methods);	    \
+  type##_specifier_methods->name = obj_name;				    \
+  type##_specifier_methods->extra_description =				    \
+    &specifier_empty_extra_description;					    \
+  defsymbol_nodump (&type##_specifier_methods->predicate_symbol, pred_sym); \
+  add_entry_to_specifier_type_list (Q##type, type##_specifier_methods);	    \
+  dump_add_root_struct_ptr (&type##_specifier_methods,			    \
+			    &specifier_methods_description);		    \
 } while (0)
 
 #define REINITIALIZE_SPECIFIER_TYPE(type) do {				\
@@ -321,7 +344,7 @@ do {									\
   type##_specifier_methods->extra_data_size =				\
     sizeof (struct type##_specifier);					\
   type##_specifier_methods->extra_description = 			\
-    type##_specifier_description;					\
+    &type##_specifier_description_0;					\
 } while (0)
 
 /* Declare that specifier-type TYPE has method METH; used in
@@ -343,7 +366,7 @@ do {									\
 
 #define GHOST_SPECIFIER(sp) XSPECIFIER ((sp)->fallback)
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 # define SPECIFIER_TYPE_DATA(sp, type) \
    error_check_##type##_specifier_data (sp)
 #else
@@ -354,18 +377,12 @@ do {									\
      : (sp)->data))
 #endif
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 # define XSPECIFIER_TYPE(x, type)	\
    error_check_##type##_specifier_type (x)
-# define XSETSPECIFIER_TYPE(x, p, type)	do		\
-{							\
-  XSETSPECIFIER (x, p);					\
-  assert (SPECIFIER_TYPEP (XSPECIFIER(x), type));	\
-} while (0)
 #else
 # define XSPECIFIER_TYPE(x, type) XSPECIFIER (x)
-# define XSETSPECIFIER_TYPE(x, p, type) XSETSPECIFIER (x, p)
-#endif /* ERROR_CHECK_TYPE_CHECK */
+#endif /* ERROR_CHECK_TYPES */
 
 #define SPECIFIER_TYPEP(x, type)			\
   (SPECIFIERP (x) && SPECIFIER_TYPE_P (XSPECIFIER (x), type))
@@ -417,7 +434,7 @@ struct specifier_caching
 
 /* #### get image instances out of domains! */
 
-/* #### I think the following should ABORT() rather than return nil
+/* #### I think the following should abort() rather than return nil
    when an invalid domain is given; much more likely we'll catch design
    errors early. --ben */
 
@@ -506,35 +523,30 @@ void kill_specifier_buffer_locals (Lisp_Object buffer);
 
 DECLARE_SPECIFIER_TYPE (generic);
 #define XGENERIC_SPECIFIER(x) XSPECIFIER_TYPE (x, generic)
-#define XSETGENERIC_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, generic)
 #define GENERIC_SPECIFIERP(x) SPECIFIER_TYPEP (x, generic)
 #define CHECK_GENERIC_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, generic)
 #define CONCHECK_GENERIC_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, generic)
 
 DECLARE_SPECIFIER_TYPE (integer);
 #define XINTEGER_SPECIFIER(x) XSPECIFIER_TYPE (x, integer)
-#define XSETINTEGER_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, integer)
 #define INTEGER_SPECIFIERP(x) SPECIFIER_TYPEP (x, integer)
 #define CHECK_INTEGER_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, integer)
 #define CONCHECK_INTEGER_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, integer)
 
 DECLARE_SPECIFIER_TYPE (natnum);
 #define XNATNUM_SPECIFIER(x) XSPECIFIER_TYPE (x, natnum)
-#define XSETNATNUM_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, natnum)
 #define NATNUM_SPECIFIERP(x) SPECIFIER_TYPEP (x, natnum)
 #define CHECK_NATNUM_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, natnum)
 #define CONCHECK_NATNUM_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, natnum)
 
 DECLARE_SPECIFIER_TYPE (boolean);
 #define XBOOLEAN_SPECIFIER(x) XSPECIFIER_TYPE (x, boolean)
-#define XSETBOOLEAN_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, boolean)
 #define BOOLEAN_SPECIFIERP(x) SPECIFIER_TYPEP (x, boolean)
 #define CHECK_BOOLEAN_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, boolean)
 #define CONCHECK_BOOLEAN_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, boolean)
 
 DECLARE_SPECIFIER_TYPE (display_table);
 #define XDISPLAYTABLE_SPECIFIER(x) XSPECIFIER_TYPE (x, display_table)
-#define XSETDISPLAYTABLE_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, display_table)
 #define DISPLAYTABLE_SPECIFIERP(x) SPECIFIER_TYPEP (x, display_table)
 #define CHECK_DISPLAYTABLE_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, display_table)
 #define CONCHECK_DISPLAYTABLE_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, display_table)
